@@ -9,7 +9,7 @@ LIST="af ar as ast be be@latin bg bn bn_IN br ca ca@valencia crh cs csb cy da de
 # 2. run this script. It will create an i18n dir in rekonq sources ($RK_SRCS variable, set it to your source path) 
 # dir with all the listed translations (eg: italian translation = rekonq_it.po file) 
 # plus the CMakeLists.txt file needed to compile them.
-RK_SRCS=/home/adjam/TMP
+RK_SRCS=/SW/SRC/rekonq
 
 # 3. Uncomment the "ADD_SUBDIRECTORY( i18n )" line in main CMakeLists.txt file.
 
