@@ -2,9 +2,9 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2010 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2009 by Paweł Prażak <pawelprazak at gmail dot com>
-* Copyright (C) 2009-2010 by Lionel Chauvin <megabigbug@yahoo.fr>
+* Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -41,6 +41,7 @@
 #include "iconmanager.h"
 #include "mainview.h"
 #include "mainwindow.h"
+#include "messagebar.h"
 #include "opensearchmanager.h"
 #include "sessionmanager.h"
 #include "stackedurlbar.h"
@@ -50,25 +51,33 @@
 
 // KDE Includes
 #include <KCmdLineArgs>
-#include <KStandardDirs>
+#include <KIcon>
 #include <KMessageBox>
+#include <KStandardDirs>
+#include <KStartupInfo>
 #include <ThreadWeaver/Weaver>
+#include <KAction>
+#include <KWindowSystem>
+#include <KGlobal>
+#include <KCharsets>
 
+// Qt Includes
+#include <QVBoxLayout>
 
-QWeakPointer<AdBlockManager> Application::s_adblockManager;
-QWeakPointer<BookmarkProvider> Application::s_bookmarkProvider;
-QWeakPointer<HistoryManager> Application::s_historyManager;
-QWeakPointer<IconManager> Application::s_iconManager;
-QWeakPointer<OpenSearchManager> Application::s_opensearchManager;
-QWeakPointer<SessionManager> Application::s_sessionManager;
 
 using namespace ThreadWeaver;
 
+
 Application::Application()
         : KUniqueApplication()
+        , _privateBrowsingAction(0)
 {
     connect(Weaver::instance(), SIGNAL(jobDone(ThreadWeaver::Job*)),
             this, SLOT(loadResolvedUrl(ThreadWeaver::Job*)));
+
+    _privateBrowsingAction = new KAction(KIcon("view-media-artist"), i18n("Private &Browsing"), this);
+    _privateBrowsingAction->setCheckable(true);
+    connect(_privateBrowsingAction, SIGNAL(triggered(bool)), this, SLOT(setPrivateBrowsingMode(bool)));
 }
 
 
@@ -79,26 +88,11 @@ Application::~Application()
     ReKonfig::setRecoverOnCrash(0);
     saveConfiguration();
 
-    foreach(QWeakPointer<MainWindow> window, m_mainWindows)
+    Q_FOREACH(QWeakPointer<MainWindow> window, m_mainWindows)
     {
         delete window.data();
         window.clear();
     }
-
-    delete s_bookmarkProvider.data();
-    s_bookmarkProvider.clear();
-
-    delete s_historyManager.data();
-    s_historyManager.clear();
-
-    delete s_sessionManager.data();
-    s_sessionManager.clear();
-
-    delete s_adblockManager.data();
-    s_adblockManager.clear();
-
-    delete s_opensearchManager.data();
-    s_opensearchManager.clear();
 }
 
 
@@ -115,7 +109,9 @@ int Application::newInstance()
     // so, we have 8 possible cases...
     bool isFirstLoad = m_mainWindows.isEmpty();
     bool areThereArguments = (args->count() > 0);
-    bool isRekonqCrashed = (ReKonfig::recoverOnCrash() == 1);
+    bool isRekonqCrashed = ReKonfig::recoverOnCrash();
+    // note that isRekonqCrashed is always true if it is not the first load
+    // !isFirstLoad -> isRekonqCrashed
 
     kDebug() << "is first load? " << isFirstLoad;
     kDebug() << "are there arguments? " << areThereArguments;
@@ -123,74 +119,78 @@ int Application::newInstance()
 
     int exitValue = 1 * isFirstLoad + 2 * areThereArguments + 4 * isRekonqCrashed;
 
-    if(isRekonqCrashed)
+    if (isRekonqCrashed && isFirstLoad)
     {
-        if( isFirstLoad  && sessionManager()->restoreSession() )
-        {
-            kDebug() << "session restored from crash";
-        }
-    }
-    else
-    {
-        if( isFirstLoad && ReKonfig::startupBehaviour() == 2 )
-        {
-            sessionManager()->restoreSession();
-            kDebug() << "session restored following settings";
-            if(areThereArguments)
-                loadUrl( KUrl("about:blank"), Rekonq::NewFocusedTab);
-        }
+        loadUrl(KUrl("about:closedTabs"), Rekonq::NewWindow);
+        MessageBar *msgBar = new MessageBar(i18n("It seems rekonq was not closed properly. Do you want "
+                                            "to restore the last saved session?")
+                                            , mainWindow()->currentTab()
+                                            , QMessageBox::Warning
+                                            , MessageBar::Yes | MessageBar::No);
+
+        connect(msgBar, SIGNAL(accepted()), sessionManager(), SLOT(restoreSession()));
+        mainWindow()->currentTab()->insertBar(msgBar);
     }
 
-    if(areThereArguments)
+    if (areThereArguments)
     {
         KUrl::List urlList;
-        for(int i = 0; i < args->count(); ++i)
+        for (int i = 0; i < args->count(); ++i)
         {
             const KUrl u = args->url(i);
             if (u.isLocalFile() && QFile::exists(u.toLocalFile())) // "rekonq somefile.html" case
                 urlList += u;
             else
-                urlList += KUrl( args->arg(i) ); // "rekonq kde.org" || "rekonq kde:kdialog" case
+                urlList += KUrl(args->arg(i));   // "rekonq kde.org" || "rekonq kde:kdialog" case
         }
 
-        if (isFirstLoad)
+        if (isFirstLoad && !isRekonqCrashed)
         {
             // No windows in the current desktop? No windows at all?
             // Create a new one and load there sites...
-            loadUrl(urlList.at(0), Rekonq::CurrentTab);
+            loadUrl(urlList.at(0), Rekonq::NewWindow);
         }
         else
         {
-            if(ReKonfig::openTabNoWindow())
+            if (ReKonfig::openTabNoWindow())
+            {
                 loadUrl(urlList.at(0), Rekonq::NewTab);
+                if (!mainWindow()->isActiveWindow())
+                    KWindowSystem::demandAttention(mainWindow()->winId(), true);
+            }
             else
                 loadUrl(urlList.at(0), Rekonq::NewWindow);
         }
-        
+
         for (int i = 1; i < urlList.count(); ++i)
-            loadUrl( urlList.at(i), Rekonq::NewTab);
+            loadUrl(urlList.at(i), Rekonq::NewTab);
+
+        KStartupInfo::appStarted();
+
     }
     else
     {
-        if (isFirstLoad)  // we are starting rekonq, for the first time with no args: use startup behaviour
+
+        if (isFirstLoad && !isRekonqCrashed)  // we are starting rekonq, for the first time with no args: use startup behaviour
         {
             switch (ReKonfig::startupBehaviour())
             {
             case 0: // open home page
-                mainWindow()->homePage();
+                newMainWindow()->homePage();
                 break;
             case 1: // open new tab page
-                loadUrl(KUrl("about:home"));
+                loadUrl(KUrl("about:home"), Rekonq::NewWindow);
                 break;
             case 2: // restore session
-                // NOTE: this has just been considered
+                sessionManager()->restoreSession();
+                kDebug() << "session restored following settings";
                 break;
             default:
-                mainWindow()->homePage();
+                newMainWindow()->homePage();
                 break;
             }
         }
-        else    // rekonq has just been started. Just open a new window
+        else if (!isFirstLoad)   // rekonq has just been started. Just open a new window
         {
             switch (ReKonfig::newTabsBehaviour())
             {
@@ -203,15 +203,15 @@ int Application::newInstance()
             case 2: // homepage
                 loadUrl(KUrl(ReKonfig::homePage()) , Rekonq::NewWindow);
                 break;
-           default:
+            default:
                 loadUrl(KUrl("about:blank") , Rekonq::NewWindow);
                 break;
-           }
+            }
 
         }
     }
 
-    if(isFirstLoad)
+    if (isFirstLoad)
     {
         // give me some time to do the other things..
         QTimer::singleShot(100, this, SLOT(postLaunch()));
@@ -234,16 +234,15 @@ void Application::postLaunch()
 
     setWindowIcon(KIcon("rekonq"));
 
-    Application::historyManager();
-    Application::sessionManager();
+    historyManager();
+    sessionManager()->setSessionManagementEnabled(true);
 
     // bookmarks loading
-    connect(Application::bookmarkProvider(), SIGNAL(openUrl(const KUrl&, const Rekonq::OpenType&)),
-            Application::instance(), SLOT(loadUrl(const KUrl&, const Rekonq::OpenType&)));
+    connect(bookmarkProvider(), SIGNAL(openUrl(const KUrl&, const Rekonq::OpenType&)),
+            instance(), SLOT(loadUrl(const KUrl&, const Rekonq::OpenType&)));
 
     // crash recovering
-    int n = ReKonfig::recoverOnCrash();
-    ReKonfig::setRecoverOnCrash(++n);
+    ReKonfig::setRecoverOnCrash(ReKonfig::recoverOnCrash() + 1);
     saveConfiguration();
 }
 
@@ -256,13 +255,13 @@ void Application::saveConfiguration() const
 
 MainWindow *Application::mainWindow()
 {
-    if (m_mainWindows.isEmpty())
-        return newMainWindow();
-
     MainWindow *active = qobject_cast<MainWindow*>(QApplication::activeWindow());
 
     if (!active)
     {
+        if(m_mainWindows.isEmpty())
+            return 0;
+
         return m_mainWindows.at(0).data();
     }
     return active;
@@ -271,53 +270,51 @@ MainWindow *Application::mainWindow()
 
 HistoryManager *Application::historyManager()
 {
-    if (s_historyManager.isNull())
+    if (m_historyManager.isNull())
     {
-        s_historyManager = new HistoryManager();
-        QWebHistoryInterface::setDefaultInterface(s_historyManager.data());
+        m_historyManager = new HistoryManager(instance());
     }
-    return s_historyManager.data();
+    return m_historyManager.data();
 }
 
 
 BookmarkProvider *Application::bookmarkProvider()
 {
-    if (s_bookmarkProvider.isNull())
+    if (m_bookmarkProvider.isNull())
     {
-        s_bookmarkProvider = new BookmarkProvider(instance());
+        m_bookmarkProvider = new BookmarkProvider(instance());
     }
-    return s_bookmarkProvider.data();
+    return m_bookmarkProvider.data();
 }
 
 
 SessionManager *Application::sessionManager()
 {
-    if (s_sessionManager.isNull())
+    if (m_sessionManager.isNull())
     {
-        s_sessionManager = new SessionManager(instance());
+        m_sessionManager = new SessionManager(instance());
     }
-    return s_sessionManager.data();
+    return m_sessionManager.data();
 }
 
 
 OpenSearchManager *Application::opensearchManager()
 {
-    if (s_opensearchManager.isNull())
+    if (m_opensearchManager.isNull())
     {
-        s_opensearchManager = new OpenSearchManager(instance());
-        s_opensearchManager.data()->setSearchProvider("google"); //TODO: use other suggestion engines
+        m_opensearchManager = new OpenSearchManager(instance());
     }
-    return s_opensearchManager.data();
+    return m_opensearchManager.data();
 }
 
 
 IconManager *Application::iconManager()
 {
-    if (s_iconManager.isNull())
+    if (m_iconManager.isNull())
     {
-        s_iconManager = new IconManager(instance());
+        m_iconManager = new IconManager(instance());
     }
-    return s_iconManager.data();
+    return m_iconManager.data();
 }
 
 
@@ -342,8 +339,8 @@ void Application::loadUrl(const KUrl& url, const Rekonq::OpenType& type)
     switch (type)
     {
     case Rekonq::NewTab:
-        if( ReKonfig::openTabNoWindow() )
-            tab = w->mainView()->newWebTab( !ReKonfig::openTabsBack() );
+        if (ReKonfig::openTabNoWindow())
+            tab = w->mainView()->newWebTab(!ReKonfig::openTabsBack());
         else
         {
             w = newMainWindow();
@@ -365,8 +362,9 @@ void Application::loadUrl(const KUrl& url, const Rekonq::OpenType& type)
 
     // rapidly show first loading url..
     int tabIndex = w->mainView()->indexOf(tab);
-    Q_ASSERT( tabIndex != -1 );
+    Q_ASSERT(tabIndex != -1);
     UrlBar *barForTab = qobject_cast<UrlBar *>(w->mainView()->widgetBar()->widget(tabIndex));
+    barForTab->activateSuggestions(false);
     barForTab->setQUrl(url);
 
     WebView *view = tab->view();
@@ -407,11 +405,11 @@ MainWindowList Application::mainWindowList()
 
 AdBlockManager *Application::adblockManager()
 {
-    if (s_adblockManager.isNull())
+    if (m_adblockManager.isNull())
     {
-        s_adblockManager = new AdBlockManager(instance());
+        m_adblockManager = new AdBlockManager(instance());
     }
-    return s_adblockManager.data();
+    return m_adblockManager.data();
 }
 
 
@@ -434,7 +432,7 @@ void Application::loadResolvedUrl(ThreadWeaver::Job *job)
 void Application::newWindow()
 {
     loadUrl(KUrl("about:home"), Rekonq::NewWindow);
-    mainWindow()->mainView()->urlBar()->setFocus();
+    mainWindow()->mainView()->currentUrlBar()->setFocus();
 }
 
 
@@ -447,6 +445,8 @@ void Application::updateConfiguration()
         MainView *mv = w.data()->mainView();
         mv->updateTabBar();
 
+        mv->tabBar()->setAnimatedTabHighlighting( ReKonfig::animatedTabHighlighting() );
+
         if (b)
             mv->tabBar()->setSelectionBehaviorOnRemove(QTabBar::SelectPreviousTab);
         else
@@ -456,10 +456,10 @@ void Application::updateConfiguration()
     QWebSettings *defaultSettings = QWebSettings::globalSettings();
 
     // =========== Fonts ==============
-    defaultSettings->setFontFamily(QWebSettings::StandardFont, ReKonfig::standardFontFamily() );
-    defaultSettings->setFontFamily(QWebSettings::FixedFont, ReKonfig::fixedFontFamily() );
-    defaultSettings->setFontFamily(QWebSettings::SerifFont, ReKonfig::serifFontFamily() );
-    defaultSettings->setFontFamily(QWebSettings::SansSerifFont, ReKonfig::sansSerifFontFamily() );
+    defaultSettings->setFontFamily(QWebSettings::StandardFont, ReKonfig::standardFontFamily());
+    defaultSettings->setFontFamily(QWebSettings::FixedFont, ReKonfig::fixedFontFamily());
+    defaultSettings->setFontFamily(QWebSettings::SerifFont, ReKonfig::serifFontFamily());
+    defaultSettings->setFontFamily(QWebSettings::SansSerifFont, ReKonfig::sansSerifFontFamily());
     defaultSettings->setFontFamily(QWebSettings::CursiveFont, ReKonfig::cursiveFontFamily());
     defaultSettings->setFontFamily(QWebSettings::FantasyFont, ReKonfig::fantasyFontFamily());
 
@@ -473,12 +473,15 @@ void Application::updateConfiguration()
     kDebug() << "Logical Dot per Inch Y: " << logDpiY;
 
     float toPix = (logDpiY < 96.0)
-        ? 96.0/72.0
-        : logDpiY/72.0 ;
+                  ? 96.0 / 72.0
+                  : logDpiY / 72.0 ;
 
-    defaultSettings->setFontSize(QWebSettings::DefaultFontSize, qRound(defaultFontSize * toPix) );
-    defaultSettings->setFontSize(QWebSettings::MinimumFontSize, qRound(minimumFontSize * toPix) );
+    defaultSettings->setFontSize(QWebSettings::DefaultFontSize, qRound(defaultFontSize * toPix));
+    defaultSettings->setFontSize(QWebSettings::MinimumFontSize, qRound(minimumFontSize * toPix));
 
+    // encodings
+    QString enc = ReKonfig::defaultEncoding();
+    defaultSettings->setDefaultTextEncoding(enc);
 
     // ================ WebKit ============================
     defaultSettings->setAttribute(QWebSettings::AutoLoadImages, ReKonfig::autoLoadImages());
@@ -496,6 +499,9 @@ void Application::updateConfiguration()
     else
         defaultSettings->setAttribute(QWebSettings::PluginsEnabled, true);
 
+    // Enabling WebKit "Page Cache" feature: http://webkit.org/blog/427/webkit-page-cache-i-the-basics/
+    defaultSettings->setMaximumPagesInCache(3);
+
     // ===== HTML 5 features WebKit support ======
     defaultSettings->setAttribute(QWebSettings::OfflineStorageDatabaseEnabled, ReKonfig::offlineStorageDatabaseEnabled());
     defaultSettings->setAttribute(QWebSettings::OfflineWebApplicationCacheEnabled, ReKonfig::offlineWebApplicationCacheEnabled());
@@ -514,8 +520,41 @@ void Application::updateConfiguration()
         defaultSettings->setUserStyleSheetUrl(ReKonfig::userCSS());
 
     // ====== load Settings on main classes
-    Application::historyManager()->loadSettings();
-    Application::adblockManager()->loadSettings();
+    historyManager()->loadSettings();
+    adblockManager()->loadSettings();
+    if (!ReKonfig::useFavicon())
+        mainWindow()->setWindowIcon(KIcon("rekonq"));
+    else
+        mainWindow()->changeWindowIcon(mainWindow()->mainView()->currentIndex());
+
+    // hovering unfocused tabs options
+    switch (ReKonfig::hoveringTabOption())
+    {
+    case 0: // tab previews
+    case 3: // nothing
+        for (int i = 0; i < mainWindow()->mainView()->tabBar()->count(); i++)
+        {
+            mainWindow()->mainView()->tabBar()->setTabToolTip(i, "");
+        }
+        break;
+
+    case 1: // title previews
+        for (int i = 0; i < mainWindow()->mainView()->tabBar()->count(); i++)
+        {
+            mainWindow()->mainView()->tabBar()->setTabToolTip(i, mainWindow()->mainView()->tabText(i).remove('&'));
+        }
+        break;
+
+    case 2: // url previews
+        for (int i = 0; i < mainWindow()->mainView()->tabBar()->count(); i++)
+        {
+            mainWindow()->mainView()->tabBar()->setTabToolTip(i, mainWindow()->mainView()->webTab(i)->url().toMimeDataString());
+        }
+        break;
+
+    default: // non extant case
+        break;
+    }
 
     defaultSettings = 0;
 }
@@ -575,3 +614,55 @@ bool Application::clearDownloadsHistory()
     QFile downloadFile(downloadFilePath);
     return downloadFile.remove();
 }
+
+
+void Application::setPrivateBrowsingMode(bool b)
+{
+// NOTE
+// to let work nicely Private Browsing, we need the following:
+// - enable WebKit Private Browsing mode :)
+// - treat all cookies as session cookies
+//  (so that they do not get saved to a persistent storage). Available from KDE SC 4.5.72, see BUG: 250122
+// - favicons (fixed in rekonq 0.5.87)
+// - save actual session (to restore it when Private Mode is closed) and stop storing it
+// - disable history saving
+
+    QWebSettings *settings = QWebSettings::globalSettings();
+    bool isJustEnabled = settings->testAttribute(QWebSettings::PrivateBrowsingEnabled);
+    if (isJustEnabled == b)
+        return;     // uhm... something goes wrong...
+
+    if (b)
+    {
+        QString caption = i18n("Are you sure you want to turn on private browsing?");
+        QString text = i18n("<b>%1</b>"
+                            "<p>rekonq will save your current tabs for when you'll stop private browsing the net.</p>", caption);
+
+        int button = KMessageBox::warningContinueCancel(mainWindow(), text, caption, KStandardGuiItem::cont(), KStandardGuiItem::cancel(), i18n("don't ask again"));
+        if (button != KMessageBox::Continue)
+            return;
+
+        settings->setAttribute(QWebSettings::PrivateBrowsingEnabled, true);
+        _privateBrowsingAction->setChecked(true);
+
+        Q_FOREACH(const QWeakPointer<MainWindow> &w, m_mainWindows)
+        {
+            w.data()->close();
+        }
+        loadUrl(KUrl("about:home"), Rekonq::NewWindow);
+    }
+    else
+    {
+        Q_FOREACH(const QWeakPointer<MainWindow> &w, m_mainWindows)
+        {
+            w.data()->close();
+        }
+
+        settings->setAttribute(QWebSettings::PrivateBrowsingEnabled, false);
+        _privateBrowsingAction->setChecked(false);
+
+        loadUrl(KUrl("about:blank"), Rekonq::NewWindow);
+        if (!sessionManager()->restoreSession())
+            loadUrl(KUrl("about:home"), Rekonq::NewWindow);
+    }
+}
