@@ -2,9 +2,9 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2010 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2009 by Paweł Prażak <pawelprazak at gmail dot com>
-* Copyright (C) 2009-2010 by Lionel Chauvin <megabigbug@yahoo.fr>
+* Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -50,7 +50,12 @@ class MainWindow;
 class OpenSearchManager;
 class SessionManager;
 
-namespace ThreadWeaver {class Job;}
+class KAction;
+
+namespace ThreadWeaver
+{
+class Job;
+}
 
 
 typedef QList< QWeakPointer<MainWindow> > MainWindowList;
@@ -78,6 +83,7 @@ public:
 
 // ---------------------------------------------------------------------------------------------------------------
 
+#define rApp Application::instance()
 
 /**
   *
@@ -96,18 +102,23 @@ public:
     MainWindow *newMainWindow(bool withTab = true);
     MainWindowList mainWindowList();
 
-    static HistoryManager *historyManager();
-    static BookmarkProvider *bookmarkProvider();
-    static SessionManager *sessionManager();
-    static AdBlockManager *adblockManager();
-    static OpenSearchManager *opensearchManager();
-    static IconManager *iconManager();
+    HistoryManager *historyManager();
+    BookmarkProvider *bookmarkProvider();
+    SessionManager *sessionManager();
+    AdBlockManager *adblockManager();
+    OpenSearchManager *opensearchManager();
+    IconManager *iconManager();
 
     // DOWNLOADS MANAGEMENT METHODS
     void addDownload(const QString &srcUrl, const QString &destUrl);
     DownloadList downloads();
     bool clearDownloadsHistory();
 
+    KAction *privateBrowsingAction()
+    {
+        return _privateBrowsingAction;
+    };
+
 public slots:
     /**
      * Save application's configuration
@@ -124,7 +135,6 @@ public slots:
     void removeMainWindow(MainWindow *window);
 
 private slots:
-
     /**
      * Any actions that can be delayed until the window is visible
      */
@@ -134,15 +144,20 @@ private slots:
 
     void updateConfiguration();
 
+    // the general place to set private browsing
+    void setPrivateBrowsingMode(bool);
+
 private:
-    static QWeakPointer<HistoryManager> s_historyManager;
-    static QWeakPointer<BookmarkProvider> s_bookmarkProvider;
-    static QWeakPointer<SessionManager> s_sessionManager;
-    static QWeakPointer<AdBlockManager> s_adblockManager;
-    static QWeakPointer<OpenSearchManager> s_opensearchManager;
-    static QWeakPointer<IconManager> s_iconManager;
+    QWeakPointer<HistoryManager> m_historyManager;
+    QWeakPointer<BookmarkProvider> m_bookmarkProvider;
+    QWeakPointer<SessionManager> m_sessionManager;
+    QWeakPointer<AdBlockManager> m_adblockManager;
+    QWeakPointer<OpenSearchManager> m_opensearchManager;
+    QWeakPointer<IconManager> m_iconManager;
 
     MainWindowList m_mainWindows;
+
+    KAction *_privateBrowsingAction;
 };
 
 #endif // APPLICATION_H
