@@ -3,7 +3,7 @@
 * This file is a part of the rekonq project
 *
 * Copyright (C) 2009 by Nils Weigel <nehlsen at gmail dot com>
-* Copyright (C) 2010 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2010-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -74,7 +74,7 @@ QVariant BtmItem::data(int role) const
         if (m_kbm.isGroup() || m_kbm.isSeparator())
             return KIcon(m_kbm.icon());
         else
-            return Application::iconManager()->iconForUrl(KUrl(m_kbm.url()));
+            return rApp->iconManager()->iconForUrl(KUrl(m_kbm.url()));
     }
 
     if (role == Qt::UserRole)
@@ -160,7 +160,7 @@ BookmarksTreeModel::BookmarksTreeModel(QObject *parent)
         , m_root(0)
 {
     resetModel();
-    connect(Application::bookmarkProvider()->bookmarkManager(), SIGNAL(changed(const QString &, const QString &)), this, SLOT(bookmarksChanged(const QString &)));
+    connect(rApp->bookmarkProvider()->bookmarkManager(), SIGNAL(changed(const QString &, const QString &)), this, SLOT(bookmarksChanged(const QString &)));
 }
 
 
@@ -278,13 +278,13 @@ bool BookmarksTreeModel::dropMimeData(const QMimeData *data, Qt::DropAction acti
         return false;
 
     QByteArray addresses = data->data("application/rekonq-bookmark");
-    KBookmark bookmark = Application::bookmarkProvider()->bookmarkManager()->findByAddress(QString::fromLatin1(addresses.data()));
+    KBookmark bookmark = rApp->bookmarkProvider()->bookmarkManager()->findByAddress(QString::fromLatin1(addresses.data()));
 
     KBookmarkGroup root;
     if (parent.isValid())
         root = bookmarkForIndex(parent).toGroup();
     else
-        root = Application::bookmarkProvider()->rootGroup();
+        root = rApp->bookmarkProvider()->rootGroup();
 
     QModelIndex destIndex = index(row, column, parent);
 
@@ -298,7 +298,7 @@ bool BookmarksTreeModel::dropMimeData(const QMimeData *data, Qt::DropAction acti
         root.addBookmark(bookmark);
     }
 
-    Application::bookmarkProvider()->bookmarkManager()->emitChanged();
+    rApp->bookmarkProvider()->bookmarkManager()->emitChanged();
 
     return true;
 }
@@ -334,22 +334,22 @@ void BookmarksTreeModel::bookmarksChanged(const QString &groupAddress)
         BtmItem *node = m_root;
         QModelIndex nodeIndex;
 
-        QStringList indexChain( groupAddress.split('/', QString::SkipEmptyParts) );
+        QStringList indexChain(groupAddress.split('/', QString::SkipEmptyParts));
         bool ok;
         int i;
-        foreach (const QString &sIndex, indexChain)
+        foreach(const QString &sIndex, indexChain)
         {
-            i = sIndex.toInt( &ok );
-            if( !ok )
+            i = sIndex.toInt(&ok);
+            if (!ok)
                 break;
 
-            if( i < 0 || i >= node->childCount() )
+            if (i < 0 || i >= node->childCount())
                 break;
 
-            node = node->child( i );
-            nodeIndex = index( i, 0, nodeIndex );
+            node = node->child(i);
+            nodeIndex = index(i, 0, nodeIndex);
         }
-        populate(node, Application::bookmarkProvider()->bookmarkManager()->findByAddress(groupAddress).toGroup());
+        populate(node, rApp->bookmarkProvider()->bookmarkManager()->findByAddress(groupAddress).toGroup());
         endResetModel();
     }
 
@@ -359,7 +359,7 @@ void BookmarksTreeModel::bookmarksChanged(const QString &groupAddress)
 
 void BookmarksTreeModel::resetModel()
 {
-    setRoot(Application::bookmarkProvider()->rootGroup());
+    setRoot(rApp->bookmarkProvider()->rootGroup());
 }
 
 
