@@ -4,7 +4,7 @@
 *
 * Copyright (C) 2007-2008 Trolltech ASA. All rights reserved
 * Copyright (C) 2008 Benjamin C. Meyer <ben@meyerhome.net>
-* Copyright (C) 2008-2010 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -59,7 +59,7 @@
 #include <QtAlgorithms>
 
 
-static const unsigned int HISTORY_VERSION = 23;
+static const unsigned int HISTORY_VERSION = 24;
 
 
 HistoryManager::HistoryManager(QObject *parent)
@@ -68,10 +68,9 @@ HistoryManager::HistoryManager(QObject *parent)
         , m_historyLimit(0)
         , m_historyTreeModel(0)
 {
-    kDebug() << "Loading HistoryManager...";
-
     connect(this, SIGNAL(entryAdded(const HistoryItem &)), m_saveTimer, SLOT(changeOccurred()));
     connect(this, SIGNAL(entryRemoved(const HistoryItem &)), m_saveTimer, SLOT(changeOccurred()));
+    connect(m_saveTimer, SIGNAL(saveNeeded()), this, SLOT(save()));
 
     load();
 
@@ -81,18 +80,12 @@ HistoryManager::HistoryManager(QObject *parent)
 
     // QWebHistoryInterface will delete the history manager
     QWebHistoryInterface::setDefaultInterface(this);
-    kDebug() << "Loading HistoryManager... DONE";
 }
 
 
 HistoryManager::~HistoryManager()
 {
     m_saveTimer->saveIfNeccessary();
-        
-    delete m_historyFilterModel;
-    delete m_historyTreeModel;
-
-    delete m_saveTimer;
 }
 
 
@@ -107,7 +100,7 @@ void HistoryManager::addHistoryEntry(const QString &url)
     QWebSettings *globalSettings = QWebSettings::globalSettings();
     if (globalSettings->testAttribute(QWebSettings::PrivateBrowsingEnabled))
         return;
-    
+
     QUrl cleanUrl(url);
 
     // don't store about: urls (home page related)
@@ -116,7 +109,27 @@ void HistoryManager::addHistoryEntry(const QString &url)
 
     cleanUrl.setPassword(QString());
     cleanUrl.setHost(cleanUrl.host().toLower());
-    HistoryItem item(cleanUrl.toString(), QDateTime::currentDateTime());
+    QString checkUrlString = cleanUrl.toString();
+
+    HistoryItem item;
+
+    // NOTE
+    // check if the url has just been visited.
+    // if so, remove previous entry from history, update and prepend it
+    if (historyContains(checkUrlString))
+    {
+        int index = m_historyFilterModel->historyLocation(checkUrlString);
+        item = m_history.at(index);
+        m_history.removeOne(item);
+        emit entryRemoved(item);
+
+        item.dateTime = QDateTime::currentDateTime();
+        item.visitCount++;
+    }
+    else
+    {
+        item = HistoryItem(checkUrlString, QDateTime::currentDateTime());
+    }
 
     m_history.prepend(item);
     emit entryAdded(item);
@@ -129,7 +142,7 @@ void HistoryManager::addHistoryEntry(const QString &url)
 void HistoryManager::setHistory(const QList<HistoryItem> &history, bool loadedAndSorted)
 {
     m_history = history;
- 
+
     // verify that it is sorted by date
     if (!loadedAndSorted)
         qSort(m_history.begin(), m_history.end());
@@ -179,7 +192,7 @@ void HistoryManager::checkForExpired()
     }
 
     if (nextTimeout > 0)
-        QTimer::singleShot( nextTimeout * 1000, this, SLOT(checkForExpired()) );
+        QTimer::singleShot(nextTimeout * 1000, this, SLOT(checkForExpired()));
 }
 
 
@@ -187,16 +200,16 @@ void HistoryManager::updateHistoryEntry(const KUrl &url, const QString &title)
 {
     QString urlString = url.url();
     urlString.remove(QL1S("www."));
-    if(urlString.startsWith(QL1S("http")) && urlString.endsWith(QL1C('/')))
-        urlString.remove(urlString.length()-1,1);
-    
+    if (urlString.startsWith(QL1S("http")) && urlString.endsWith(QL1C('/')))
+        urlString.remove(urlString.length() - 1, 1);
+
     for (int i = 0; i < m_history.count(); ++i)
     {
         QString itemUrl = m_history.at(i).url;
         itemUrl.remove(QL1S("www."));
-        if(itemUrl.startsWith(QL1S("http")) && itemUrl.endsWith(QL1C('/')))
-            itemUrl.remove(itemUrl.length()-1,1);
-        
+        if (itemUrl.startsWith(QL1S("http")) && itemUrl.endsWith(QL1C('/')))
+            itemUrl.remove(itemUrl.length() - 1, 1);
+
         if (urlString == itemUrl)
         {
             m_history[i].title = title;
@@ -204,7 +217,7 @@ void HistoryManager::updateHistoryEntry(const KUrl &url, const QString &title)
             m_saveTimer->changeOccurred();
             if (m_lastSavedUrl.isEmpty())
                 m_lastSavedUrl = m_history.at(i).url;
-            
+
             emit entryUpdated(i);
             break;
         }
@@ -242,10 +255,11 @@ QList<HistoryItem> HistoryManager::find(const QString &text)
 
         QStringList words = text.split(' ');
         bool matches = true;
-        foreach (const QString &word, words)
+        foreach(const QString &word, words)
         {
             if (!url.contains(word, Qt::CaseInsensitive)
-                && !item.title.contains(word, Qt::CaseInsensitive)) {
+                    && !item.title.contains(word, Qt::CaseInsensitive))
+            {
                 matches = false;
                 break;
             }
@@ -253,7 +267,7 @@ QList<HistoryItem> HistoryManager::find(const QString &text)
         if (matches)
             list << item;
     }
-    
+
     return list;
 }
 
@@ -274,25 +288,25 @@ void HistoryManager::loadSettings()
     int days;
     switch (historyExpire)
     {
-    case 0: 
-        days = 1; 
+    case 0:
+        days = 1;
         break;
-    case 1: 
-        days = 7; 
+    case 1:
+        days = 7;
         break;
-    case 2: 
-        days = 14; 
+    case 2:
+        days = 14;
         break;
-    case 3: 
-        days = 30; 
+    case 3:
+        days = 30;
         break;
-    case 4: 
-        days = 365; 
+    case 4:
+        days = 365;
         break;
-    case 5: 
-        days = -1; 
+    case 5:
+        days = -1;
         break;
-    default: 
+    default:
         days = -1;
         break;
     }
@@ -329,14 +343,30 @@ void HistoryManager::load()
         buffer.close();
         buffer.setBuffer(&data);
         buffer.open(QIODevice::ReadOnly);
-        quint32 ver;
-        stream >> ver;
-        if (ver != HISTORY_VERSION)
-            continue;
+        quint32 version;
+        stream >> version;
+
         HistoryItem item;
-        stream >> item.url;
-        stream >> item.dateTime;
-        stream >> item.title;
+
+        switch (version)
+        {
+        case HISTORY_VERSION:   // default case
+            stream >> item.url;
+            stream >> item.dateTime;
+            stream >> item.title;
+            stream >> item.visitCount;
+            break;
+
+        case 23:    // this will be used to upgrade previous structure...
+            stream >> item.url;
+            stream >> item.dateTime;
+            stream >> item.title;
+            item.visitCount = 1;
+            break;
+
+        default:
+            continue;
+        };
 
         if (!item.dateTime.isValid())
             continue;
@@ -416,7 +446,7 @@ void HistoryManager::save()
         QByteArray data;
         QDataStream stream(&data, QIODevice::WriteOnly);
         HistoryItem item = m_history.at(i);
-        stream << HISTORY_VERSION << item.url << item.dateTime << item.title;
+        stream << HISTORY_VERSION << item.url << item.dateTime << item.title << item.visitCount;
         out << data;
     }
     tempFile.close();
