@@ -4,7 +4,7 @@
 *
 * Copyright (C) 2007-2008 Trolltech ASA. All rights reserved
 * Copyright (C) 2008 Benjamin C. Meyer <ben@meyerhome.net>
-* Copyright (C) 2008-2010 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -47,6 +47,7 @@
 #include <QSortFilterProxyModel>
 #include <QWebHistoryInterface>
 
+#include <math.h>
 
 /**
  * Elements in this class represent an history item
@@ -55,31 +56,41 @@
 class HistoryItem
 {
 public:
-    HistoryItem() {}
+    HistoryItem()
+    {}
+
     explicit HistoryItem(const QString &u,
                          const QDateTime &d = QDateTime(),
                          const QString &t = QString()
                         )
             : title(t)
-            , url(u),
-            dateTime(d)
+            , url(u)
+            , dateTime(d)
+            , visitCount(1)
     {}
 
     inline bool operator==(const HistoryItem &other) const
     {
         return other.title == title
-               && other.url == url && other.dateTime == dateTime;
+               && other.url == url
+               && other.dateTime == dateTime;
+    }
+
+    inline qreal relevance() const
+    {
+        return log(visitCount) - log(dateTime.daysTo(QDateTime::currentDateTime()) + 1);
     }
 
     // history is sorted in reverse
     inline bool operator <(const HistoryItem &other) const
     {
-        return dateTime > other.dateTime;
+        return relevance() > other.relevance();
     }
 
     QString title;
     QString url;
     QDateTime dateTime;
+    int visitCount;
 };
 
 
@@ -100,12 +111,6 @@ class REKONQ_TESTS_EXPORT HistoryManager : public QWebHistoryInterface
 {
     Q_OBJECT
 
-signals:
-    void historyReset();
-    void entryAdded(const HistoryItem &item);
-    void entryRemoved(const HistoryItem &item);
-    void entryUpdated(int offset);
-
 public:
     HistoryManager(QObject *parent = 0);
     ~HistoryManager();
@@ -117,18 +122,33 @@ public:
 
     QList<HistoryItem> find(const QString &text);
 
-    QList<HistoryItem> history() const { return m_history; };
+    QList<HistoryItem> history() const
+    {
+        return m_history;
+    };
     void setHistory(const QList<HistoryItem> &history, bool loadedAndSorted = false);
 
     // History manager keeps around these models for use by the completer and other classes
-    HistoryFilterModel *historyFilterModel() const { return m_historyFilterModel; };
-    HistoryTreeModel *historyTreeModel() const { return m_historyTreeModel; };
+    HistoryFilterModel *historyFilterModel() const
+    {
+        return m_historyFilterModel;
+    };
+    HistoryTreeModel *historyTreeModel() const
+    {
+        return m_historyTreeModel;
+    };
 
-public slots:
+Q_SIGNALS:
+    void historyReset();
+    void entryAdded(const HistoryItem &item);
+    void entryRemoved(const HistoryItem &item);
+    void entryUpdated(int offset);
+
+public Q_SLOTS:
     void clear();
     void loadSettings();
 
-private slots:
+private Q_SLOTS:
     void save();
     void checkForExpired();
 
