@@ -4,7 +4,7 @@
 *
 * Copyright (C) 2007-2008 Trolltech ASA. All rights reserved
 * Copyright (C) 2008 Benjamin C. Meyer <ben@meyerhome.net>
-* Copyright (C) 2008-2010 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -146,7 +146,7 @@ QVariant HistoryModel::data(const QModelIndex &index, int role) const
     case Qt::DecorationRole:
         if (index.column() == 0)
         {
-            return Application::iconManager()->iconForUrl(item.url);
+            return rApp->iconManager()->iconForUrl(item.url);
         }
     case Qt::ToolTipRole:
         QString tooltip = "";
@@ -192,8 +192,8 @@ bool HistoryModel::removeRows(int row, int count, const QModelIndex &parent)
 
 
 HistoryFilterModel::HistoryFilterModel(QAbstractItemModel *sourceModel, QObject *parent)
-        : QAbstractProxyModel(parent),
-        m_loaded(false)
+        : QAbstractProxyModel(parent)
+        , m_loaded(false)
 {
     setSourceModel(sourceModel);
 }
@@ -354,10 +354,18 @@ void HistoryFilterModel::load() const
 
 void HistoryFilterModel::sourceRowsInserted(const QModelIndex &parent, int start, int end)
 {
-    Q_ASSERT(start == end && start == 0);
+    //Q_ASSERT(start == end && start == 0);
     Q_UNUSED(end);
+    
+    if(start != 0)
+    {
+        kDebug() << "STARTING from a NON zero position...";
+        return;
+    }
+    
     if (!m_loaded)
         return;
+    
     QModelIndex idx = sourceModel()->index(start, 0, parent);
     QString url = idx.data(HistoryModel::UrlStringRole).toString();
     if (m_historyHash.contains(url))
