@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2010 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -47,9 +47,9 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
                      REKONQ_VERSION,
                      ki18n(description),
                      KAboutData::License_GPL_V3,
-                     ki18n("(C) 2008-2010 Andrea Diamantini"),
+                     ki18n("(C) 2008-2011 Andrea Diamantini"),
                      KLocalizedString(),
-                     "http://rekonq.sourceforge.net"
+                     "http://rekonq.kde.org"
                     );
 
     // --------------- about authors -----------------------------
@@ -94,7 +94,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
                     "");
 
     about.addAuthor(ki18n("Nikhil Marathe"),
-                    ki18n("Bugfixing, Working on Extensions support (not yet available)"),
+                    ki18n("Bugfixing"),
                     "nsm.nikhil@gmail.com",
                     "");
 
@@ -114,10 +114,20 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
                     "http://identi.ca/jonan");
 
     about.addAuthor(ki18n("Benjamin Poulain"),
-                    ki18n("The \"QtWebKit guy\". Code quality improvements"),
+                    ki18n("The \"QtWebKit guy\". Adblock (new) implementation. Code quality improvements"),
                     "ikipou@gmail.com",
                     "http://www.openyourcode.org/");
 
+    about.addAuthor(ki18n("Pierre Rossi"),
+                    ki18n("Urlbar, tests, new tab page, bars... and more"),
+                    "pierre.rossi@gmail.com",
+                    "");
+
+    about.addAuthor(ki18n("Furkan Uzumcu"),
+                    ki18n("A lot of improvements, especially on usability"),
+                    "furkanuzumcu@gmail.com",
+                    "");
+
     // --------------- about credits -----------------------------
     about.addCredit(ki18n("Dawit Alemayehu"),
                     ki18n("KDEWebKit (main) developer. And KIO. And KUriFilter. And more.."),
@@ -169,6 +179,11 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
                     "jondeandres@gmail.com",
                     "");
 
+    about.addCredit(ki18n("Johannes Troscher"),
+                    ki18n("QGraphicsEffect expert. Tabbar highlight animation"),
+                    "fritz_van_tomas@hotmail.com",
+                    "");
+
     // Initialize command line args
     KCmdLineArgs::init(argc, argv, &about);
 
@@ -189,10 +204,12 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
         kWarning() << "rekonq is already running!";
         return 0;
     }
+
 #if defined(Q_WS_X11)
     // On X11, the raster engine gives better performance than native.
-    QApplication::setGraphicsSystem(QString::fromLatin1("raster"));
+    QApplication::setGraphicsSystem(QL1S("raster"));
 #endif
+
     Application app;
 
     return app.exec();
