@@ -2,9 +2,9 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2010 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2009 by Paweł Prażak <pawelprazak at gmail dot com>
-* Copyright (C) 2009-2010 by Lionel Chauvin <megabigbug@yahoo.fr>
+* Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 * Copyright (C) 2010 by Matthieu Gicquel <matgic78 at gmail dot com>
 *
 *
@@ -56,10 +56,12 @@
 #include <QtGui/QMovie>
 #include <QtGui/QToolButton>
 
+//Hack:
+QString temporaryUglyHackString = "";
 
 MainView::MainView(MainWindow *parent)
         : KTabWidget(parent)
-        , _widgetBar(new StackedUrlBar(this))
+        , m_widgetBar(new StackedUrlBar(this))
         , m_addTabButton(0)
         , m_currentTabIndex(0)
         , m_parentWindow(parent)
@@ -87,7 +89,7 @@ MainView::MainView(MainWindow *parent)
     connect(tabBar, SIGNAL(detachTab(int)),         this,   SLOT(detachTab(int)));
 
     connect(tabBar, SIGNAL(tabCloseRequested(int)), this,   SLOT(closeTab(int)));
-    connect(tabBar, SIGNAL(tabMoved(int, int)),     _widgetBar, SLOT(moveBar(int, int)));
+    connect(tabBar, SIGNAL(tabMoved(int, int)),     m_widgetBar, SLOT(moveBar(int, int)));
 
     // current page index changing
     connect(this, SIGNAL(currentChanged(int)), this, SLOT(currentChanged(int)));
@@ -96,19 +98,12 @@ MainView::MainView(MainWindow *parent)
 }
 
 
-MainView::~MainView()
-{
-    delete _widgetBar;
-    delete m_addTabButton;
-}
-
-
 void MainView::postLaunch()
 {
-    QStringList list = Application::sessionManager()->closedSites();
-    foreach(const QString &line, list)
+    QStringList list = rApp->sessionManager()->closedSites();
+    Q_FOREACH(const QString &line, list)
     {
-        if(line.startsWith( QL1S("about") ))
+        if (line.startsWith(QL1S("about")))
             break;
         QString title = line;
         QString url = title;
@@ -118,8 +113,7 @@ void MainView::postLaunch()
     }
 
     // Session Manager
-    connect(this, SIGNAL(tabsChanged()), Application::sessionManager(), SLOT(saveSession()));
-    connect(this, SIGNAL(currentChanged(int)), Application::sessionManager(), SLOT(saveSession()));
+    connect(this, SIGNAL(tabsChanged()), rApp->sessionManager(), SLOT(saveSession()));
 
     m_addTabButton->setDefaultAction(m_parentWindow->actionByName("new_tab"));
 
@@ -128,8 +122,55 @@ void MainView::postLaunch()
 }
 
 
-void MainView::updateTabButtonPosition()
+TabBar *MainView::tabBar() const
 {
+    TabBar *tabBar = qobject_cast<TabBar *>(KTabWidget::tabBar());
+    return tabBar;
+}
+
+
+UrlBar *MainView::currentUrlBar() const
+{
+    return currentWebTab()->urlBar();
+}
+
+
+WebTab *MainView::currentWebTab() const
+{
+    int i = currentIndex();
+    WebTab *b = webTab(i);
+    if(b)
+        return b;
+
+    kDebug() << "We failed to find the current tab!!! Let's go sure with the first one...";
+    return webTab(0);
+}
+
+
+void MainView::updateTabBar()
+{
+    if (ReKonfig::alwaysShowTabBar() || count() > 1)
+    {
+        if (tabBar()->isHidden())
+        {
+            tabBar()->show();
+        }
+
+        // this to ensure tab button visibility also on new window creation
+        if (m_addTabButton->isHidden())
+        {
+            m_addTabButton->show();
+        }
+
+    }
+    else
+    {
+        tabBar()->hide();
+        m_addTabButton->hide();
+        return;
+    }
+
+    // update tab button position
     static bool ButtonInCorner = false;
 
     int tabWidgetWidth = frameSize().width();
@@ -147,7 +188,6 @@ void MainView::updateTabButtonPosition()
         if (ButtonInCorner)
         {
             setCornerWidget(0);
-            m_addTabButton->show();
             ButtonInCorner = false;
         }
 
@@ -162,70 +202,18 @@ void MainView::updateTabButtonPosition()
 }
 
 
-TabBar *MainView::tabBar() const
-{
-    TabBar *tabBar = qobject_cast<TabBar *>(KTabWidget::tabBar());
-    return tabBar;
-}
-
-
-UrlBar *MainView::urlBar() const
-{
-    return _widgetBar->urlBar(m_currentTabIndex);
-}
-
-
-WebTab *MainView::currentWebTab() const
-{
-    return webTab(currentIndex());
-}
-
-
-void MainView::updateTabBar()
-{
-    if (ReKonfig::alwaysShowTabBar())
-    {
-        if (!isTabBarHidden())
-        {
-            if (tabBar()->isHidden())
-            {
-                tabBar()->show();
-                m_addTabButton->show();
-            }
-            updateTabButtonPosition();
-        }
-        return;
-    }
-
-    if (tabBar()->count() == 1)
-    {
-        tabBar()->hide();
-        m_addTabButton->hide();
-    }
-    else if (!isTabBarHidden())
-    {
-        if (tabBar()->isHidden())
-        {
-            tabBar()->show();
-            m_addTabButton->show();
-        }
-        updateTabButtonPosition();
-    }
-}
-
-
-void MainView::setTabBarHidden(bool hide)
-{
-    m_addTabButton->setVisible(!hide);
-    KTabWidget::setTabBarHidden(hide);
-}
-
-
 void MainView::webReload()
 {
     WebTab *webTab = currentWebTab();
-    QAction *action = webTab->view()->page()->action(QWebPage::Reload);
-    action->trigger();
+    if (webTab->view()->url().scheme() != QL1S("about"))
+    {
+        QAction *action = webTab->view()->page()->action(QWebPage::Reload);
+        action->trigger();
+    }
+    else
+    {
+        webTab->view()->setUrl(webTab->page()->loadingUrl());
+    }
 }
 
 
@@ -256,10 +244,10 @@ void MainView::currentChanged(int index)
     WebTab *tab = this->webTab(index);
     if (!tab)
         return;
-
+    
     // retrieve the old webview (that where we move from)
     WebTab *oldTab = this->webTab(m_currentTabIndex);
-
+    
     // set current index
     m_currentTabIndex = index;
 
@@ -278,23 +266,23 @@ void MainView::currentChanged(int index)
             this, SIGNAL(linkHovered(const QString&)));
 
     emit currentTitle(tab->view()->title());
-    _widgetBar->setCurrentIndex(index);
+    m_widgetBar->setCurrentIndex(index);
 
     // clean up "status bar"
     emit showStatusBarMessage(QString());
 
     // notify UI to eventually switch stop/reload button
-    int progr = tab->progress();
-    if (progr == 0)
-        emit browserTabLoading(false);
-    else
-        emit browserTabLoading(true);
+    emit browserTabLoading(tab->isPageLoading());
 
     // set focus to the current webview
     if (tab->url().scheme() == QL1S("about"))
-        _widgetBar->currentWidget()->setFocus();
+        m_widgetBar->currentWidget()->setFocus();
     else
         tab->view()->setFocus();
+
+    tabBar()->resetTabHighlighted(index);
+
+    emit tabsChanged();
 }
 
 
@@ -313,29 +301,32 @@ WebTab *MainView::webTab(int index) const
 
 WebTab *MainView::newWebTab(bool focused)
 {
-    WebTab* tab = new WebTab(this);
-    UrlBar *bar = new UrlBar(tab);
+    WebTab *tab = new WebTab(this);
 
     // connecting webview with mainview
     connect(tab->view(), SIGNAL(loadStarted()), this, SLOT(webViewLoadStarted()));
     connect(tab->view(), SIGNAL(loadFinished(bool)), this, SLOT(webViewLoadFinished(bool)));
-    connect(tab->view(), SIGNAL(titleChanged(const QString &)), this, SLOT(webViewTitleChanged(const QString &)));
+    connect(tab, SIGNAL(titleChanged(const QString &)), this, SLOT(webViewTitleChanged(const QString &)));
     connect(tab->view(), SIGNAL(urlChanged(const QUrl &)), this, SLOT(webViewUrlChanged(const QUrl &)));
     connect(tab->view(), SIGNAL(iconChanged()), this, SLOT(webViewIconChanged()));
+    connect(tab->view(), SIGNAL(openPreviousInHistory()), this, SIGNAL(openPreviousInHistory()));
+    connect(tab->view(), SIGNAL(openNextInHistory()), this, SIGNAL(openNextInHistory()));
 
     // connecting webPage signals with mainview
-    connect(tab->view()->page(), SIGNAL(windowCloseRequested()), this, SLOT(windowCloseRequested()));
-    connect(tab->view()->page(), SIGNAL(printRequested(QWebFrame *)), this, SIGNAL(printRequested(QWebFrame *)));
+    connect(tab->page(), SIGNAL(windowCloseRequested()), this, SLOT(windowCloseRequested()));
+    connect(tab->page(), SIGNAL(printRequested(QWebFrame *)), this, SIGNAL(printRequested(QWebFrame *)));
 
-    if ( ReKonfig::openTabsNearCurrent() )
+    if (ReKonfig::openTabsNearCurrent())
     {
         insertTab(currentIndex() + 1, tab, i18n("(Untitled)"));
-        _widgetBar->insertWidget(currentIndex() + 1, bar);
+        temporaryUglyHackString = tabText(currentIndex() + 1);
+        m_widgetBar->insertWidget(currentIndex() + 1, tab->urlBar());
     }
     else
     {
         addTab(tab, i18n("(Untitled)"));
-        _widgetBar->addWidget(bar);
+        temporaryUglyHackString = tabText(count() - 1);
+        m_widgetBar->addWidget(tab->urlBar());
     }
     updateTabBar();
 
@@ -343,8 +334,12 @@ WebTab *MainView::newWebTab(bool focused)
     {
         setCurrentWidget(tab);
     }
-
-    emit tabsChanged();
+    else
+    {
+        // if tab is not focused,
+        // current index doesn't change...
+        emit tabsChanged();
+    }
 
     return tab;
 }
@@ -360,7 +355,7 @@ void MainView::newTab()
         w->load(KUrl("about:home"));
         break;
     case 1: // blank page
-        urlBar()->clear();
+        currentUrlBar()->clear();
         break;
     case 2: // homepage
         w->load(KUrl(ReKonfig::homePage()));
@@ -368,7 +363,7 @@ void MainView::newTab()
     default:
         break;
     }
-    urlBar()->setFocus();
+    currentUrlBar()->setFocus();
 }
 
 
@@ -433,7 +428,7 @@ void MainView::cloneTab(int index)
 
     KUrl url = webTab(index)->url();
 
-    Application::instance()->loadUrl(url, Rekonq::NewTab);
+    rApp->loadUrl(url, Rekonq::NewTab);
 
     updateTabBar();
 }
@@ -447,7 +442,7 @@ void MainView::closeTab(int index, bool del)
     {
         WebView *w = currentWebTab()->view();
 
-        if( currentWebTab()->url().protocol() == QL1S("about") )
+        if (currentWebTab()->url().protocol() == QL1S("about"))
             return;
 
         switch (ReKonfig::newTabsBehaviour())
@@ -455,7 +450,7 @@ void MainView::closeTab(int index, bool del)
         case 0: // new tab page
         case 1: // blank page
             w->load(KUrl("about:home"));
-            urlBar()->setFocus();
+            currentUrlBar()->setFocus();
             break;
         case 2: // homepage
             w->load(KUrl(ReKonfig::homePage()));
@@ -471,11 +466,11 @@ void MainView::closeTab(int index, bool del)
     if (index < 0 || index >= count())
         return;
 
-    WebTab *tab = webTab(index);
-    if (!tab)
+    WebTab *tabToClose = webTab(index);
+    if (!tabToClose)
         return;
 
-    if (tab->view()->isModified())
+    if (tabToClose->view()->isModified())
     {
         int risp = KMessageBox::warningContinueCancel(this,
                    i18n("This tab contains changes that have not been submitted.\n"
@@ -486,32 +481,34 @@ void MainView::closeTab(int index, bool del)
             return;
     }
 
-    if (!tab->url().isEmpty()
-        && !QWebSettings::globalSettings()->testAttribute(QWebSettings::PrivateBrowsingEnabled)
+    if (!tabToClose->url().isEmpty()
+            && !QWebSettings::globalSettings()->testAttribute(QWebSettings::PrivateBrowsingEnabled)
        )
     {
-        QString title = tab->view()->title();
-        QString url = tab->url().prettyUrl();
+        const int recentlyClosedTabsLimit = 10;
+        QString title = tabToClose->view()->title();
+        QString url = tabToClose->url().prettyUrl();
         HistoryItem item(url, QDateTime(), title);
         m_recentlyClosedTabs.removeAll(item);
+        if (m_recentlyClosedTabs.count() == recentlyClosedTabsLimit)
+            m_recentlyClosedTabs.removeLast();
         m_recentlyClosedTabs.prepend(item);
     }
 
     removeTab(index);
     updateTabBar();        // UI operation: do it ASAP!!
 
-    UrlBar *urlbar = _widgetBar->urlBar(index);
-    _widgetBar->removeWidget(urlbar);
-
-    _widgetBar->setCurrentIndex(m_currentTabIndex);
+    m_widgetBar->removeWidget(tabToClose->urlBar());
+    m_widgetBar->setCurrentIndex(m_currentTabIndex);
 
     if (del)
     {
-        tab->deleteLater();
-        urlbar->deleteLater();
+        tabToClose->deleteLater();
     }
 
-    emit tabsChanged();
+    // if tab was not focused, current index does not change...
+    if (index != currentIndex())
+        emit tabsChanged();
 }
 
 
@@ -570,10 +567,12 @@ void MainView::webViewLoadFinished(bool ok)
 void MainView::webViewIconChanged()
 {
     WebView *view = qobject_cast<WebView *>(sender());
-    int index = indexOf(view->parentWidget());
+    WebTab *tab = qobject_cast<WebTab *>(view->parent());
+    int index = indexOf(tab);
+
     if (-1 != index)
     {
-        KIcon icon = Application::iconManager()->iconForUrl(view->url());
+        KIcon icon = rApp->iconManager()->iconForUrl(tab->url());
         QLabel *label = animatedLoading(index, false);
         QMovie *movie = label->movie();
         delete movie;
@@ -585,12 +584,12 @@ void MainView::webViewIconChanged()
 
 void MainView::webViewTitleChanged(const QString &title)
 {
-    QString viewTitle = title.isEmpty()? i18n("(Untitled)") : title;
+    QString viewTitle = title.isEmpty() ? i18n("(Untitled)") : title;
     QString tabTitle = viewTitle;
     tabTitle.replace('&', "&&");
 
-    WebView *view = qobject_cast<WebView *>(sender());
-    int index = indexOf(view->parentWidget());
+    WebTab *tab = qobject_cast<WebTab *>(sender());
+    int index = indexOf(tab);
     if (-1 != index)
     {
         setTabText(index, tabTitle);
@@ -599,7 +598,14 @@ void MainView::webViewTitleChanged(const QString &title)
     {
         emit currentTitle(viewTitle);
     }
-    Application::historyManager()->updateHistoryEntry(view->url(), tabTitle);
+    else
+    {
+        if (tabTitle != temporaryUglyHackString)
+            tabBar()->setTabHighlighted(index);
+    }
+    rApp->historyManager()->updateHistoryEntry(tab->url(), tabTitle);
+    if (ReKonfig::hoveringTabOption() == 1)
+        tabBar()->setTabToolTip(index, tabTitle.remove('&'));
 }
 
 
@@ -611,6 +617,9 @@ void MainView::webViewUrlChanged(const QUrl &url)
     {
         tabBar()->setTabData(index, url);
     }
+    if (ReKonfig::hoveringTabOption() == 2)
+        tabBar()->setTabToolTip(index, webTab(index)->url().toMimeDataString());
+
     emit tabsChanged();
 }
 
@@ -633,13 +642,13 @@ void MainView::previousTab()
 }
 
 
-void MainView::openClosedTabs()
+void MainView::openLastClosedTab()
 {
-    foreach (const HistoryItem &item, recentlyClosedTabs())
-    {
-        Application::instance()->loadUrl( KUrl(item.url), Rekonq::NewTab);
-    }
-    m_recentlyClosedTabs.clear();
+    if (m_recentlyClosedTabs.isEmpty())
+        return;
+
+    const HistoryItem item = m_recentlyClosedTabs.takeFirst();
+    rApp->loadUrl(KUrl(item.url), Rekonq::NewTab);
 }
 
 
@@ -648,11 +657,11 @@ void MainView::openClosedTab()
     KAction *action = qobject_cast<KAction *>(sender());
     if (action)
     {
-        Application::instance()->loadUrl(action->data().toUrl(), Rekonq::NewTab);
+        rApp->loadUrl(action->data().toUrl(), Rekonq::NewTab);
 
         QString title = action->text();
         title = title.remove('&');
-        HistoryItem item(action->data().toString(), QDateTime(), title );
+        HistoryItem item(action->data().toString(), QDateTime(), title);
         m_recentlyClosedTabs.removeAll(item);
     }
 }
@@ -664,9 +673,9 @@ void MainView::switchToTab()
     QAction *sender = static_cast<QAction*>(QObject::sender());
     int index = sender->data().toInt();
     index -= 1; // to compensate for off by 1 presented to the user
-    if( index < 0 || index >= count() )
+    if (index < 0 || index >= count())
         return;
-    setCurrentIndex( index );
+    setCurrentIndex(index);
 }
 
 
@@ -713,20 +722,20 @@ void MainView::detachTab(int index, MainWindow *toWindow)
     if (u.scheme() == QL1S("about"))
     {
         closeTab(index);
-        Application::instance()->loadUrl(u, Rekonq::NewWindow);
+        rApp->loadUrl(u, Rekonq::NewWindow);
     }
     else
     {
         QString label = tab->view()->title();
-        UrlBar *bar = _widgetBar->urlBar(index);
+        UrlBar *bar = tab->urlBar();
         closeTab(index, false);
 
         MainWindow *w;
-        if( toWindow == NULL )
-            w = Application::instance()->newMainWindow(false);
+        if (toWindow == NULL)
+            w = rApp->newMainWindow(false);
         else
             w = toWindow;
-        w->mainView()->addTab(tab, Application::iconManager()->iconForUrl(u), label);
+        w->mainView()->addTab(tab, rApp->iconManager()->iconForUrl(u), label);
         w->mainView()->widgetBar()->insertWidget(0, bar);
         w->mainView()->updateTabBar();
     }
