@@ -49,11 +49,6 @@ PanelTreeView::PanelTreeView(QWidget *parent)
 }
 
 
-PanelTreeView::~PanelTreeView()
-{
-}
-
-
 void PanelTreeView::mousePressEvent(QMouseEvent *event)
 {
     const QModelIndex index = indexAt(event->pos());
@@ -100,12 +95,12 @@ void PanelTreeView::mouseReleaseEvent(QMouseEvent *event)
         return;
 
     if (event->button() == Qt::MidButton || event->modifiers() == Qt::ControlModifier)
-        validOpenUrl(qVariantValue< KUrl >(index.data(Qt::UserRole)), Rekonq::NewTab);
+        emit openUrl(qVariantValue< KUrl >(index.data(Qt::UserRole)), Rekonq::NewTab);
 
     else if (event->button() == Qt::LeftButton)
     {
         if (model()->rowCount(index) == 0)
-            validOpenUrl(qVariantValue< KUrl >(index.data(Qt::UserRole)));
+            emit openUrl(qVariantValue< KUrl >(index.data(Qt::UserRole)));
         else
             setExpanded(index, !isExpanded(index));
     }
@@ -123,7 +118,7 @@ void PanelTreeView::keyPressEvent(QKeyEvent *event)
     if (event->key() == Qt::Key_Return)
     {
         if (model()->rowCount(index) == 0)
-            validOpenUrl(qVariantValue< KUrl >(index.data(Qt::UserRole)));
+            openUrl(qVariantValue< KUrl >(index.data(Qt::UserRole)));
         else
             setExpanded(index, !isExpanded(index));
     }
@@ -135,16 +130,6 @@ void PanelTreeView::keyPressEvent(QKeyEvent *event)
 }
 
 
-void PanelTreeView::validOpenUrl(const KUrl &url, Rekonq::OpenType openType)
-{
-    // To workaround a crash when the url is about:blank
-    if (url.url() == "about:blank")
-        emit openUrl(KUrl("about:home"), openType);
-    else
-        emit openUrl(url, openType);
-}
-
-
 void PanelTreeView::mouseMoveEvent(QMouseEvent *event)
 {
     QTreeView::mouseMoveEvent(event);
@@ -164,7 +149,7 @@ void PanelTreeView::openInCurrentTab()
     if (!index.isValid())
         return;
 
-    validOpenUrl(qVariantValue< KUrl >(index.data(Qt::UserRole)));
+    emit openUrl(qVariantValue< KUrl >(index.data(Qt::UserRole)));
 }
 
 
@@ -174,7 +159,7 @@ void PanelTreeView::copyToClipboard()
     if (!index.isValid())
         return;
 
-    QClipboard *cb = Application::clipboard();
+    QClipboard *cb = QApplication::clipboard();
     cb->setText(qVariantValue< KUrl >(index.data(Qt::UserRole)).url());
 }
 
@@ -185,7 +170,7 @@ void PanelTreeView::openInNewTab()
     if (!index.isValid())
         return;
 
-    validOpenUrl(qVariantValue< KUrl >(index.data(Qt::UserRole)), Rekonq::NewTab);
+    emit openUrl(qVariantValue< KUrl >(index.data(Qt::UserRole)), Rekonq::NewTab);
 }
 
 
@@ -195,5 +180,5 @@ void PanelTreeView::openInNewWindow()
     if (!index.isValid())
         return;
 
-    validOpenUrl(qVariantValue< KUrl >(index.data(Qt::UserRole)), Rekonq::NewWindow);
+    emit openUrl(qVariantValue< KUrl >(index.data(Qt::UserRole)), Rekonq::NewWindow);
 }
