@@ -3,7 +3,7 @@
 * This file is a part of the rekonq project
 *
 * Copyright (C) 2010 by Matthieu Gicquel <matgic78 at gmail dot com>
-* Copyright (C) 2010 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2010-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -81,15 +81,10 @@ PreviewSelectorBar::PreviewSelectorBar(int index, QWidget* parent)
 }
 
 
-PreviewSelectorBar::~PreviewSelectorBar()
-{
-}
-
-
 void PreviewSelectorBar::verifyUrl()
 {
 
-    if (Application::instance()->mainWindow()->currentTab()->page()->mainFrame()->url().scheme() != "about")
+    if (rApp->mainWindow()->currentTab()->page()->mainFrame()->url().scheme() != "about")
     {
         m_button->setEnabled(true);
         m_button->setToolTip("");
@@ -97,7 +92,7 @@ void PreviewSelectorBar::verifyUrl()
     else
     {
         m_button->setEnabled(false);
-        m_button->setToolTip(i18n("You can not add this webpage as favorite"));
+        m_button->setToolTip(i18n("You cannot add this webpage as favorite"));
     }
 }
 
@@ -120,7 +115,7 @@ void PreviewSelectorBar::loadFinished()
 
 void PreviewSelectorBar::clicked()
 {
-    WebPage *page = Application::instance()->mainWindow()->currentTab()->page();
+    WebPage *page = rApp->mainWindow()->currentTab()->page();
 
     if (page)
     {
@@ -147,13 +142,3 @@ void PreviewSelectorBar::clicked()
 
     destroy();
 }
-
-
-void PreviewSelectorBar::destroy()
-{
-    if (parentWidget() && parentWidget()->layout())
-    {
-        parentWidget()->layout()->removeWidget(this);
-    }
-    this->deleteLater();
-}
