@@ -3,7 +3,7 @@
 * This file is a part of the rekonq project
 *
 * Copyright (C) 2010 by Matthieu Gicquel <matgic78 at gmail dot com>
-* Copyright (C) 2010 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2010-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -43,9 +43,11 @@ class REKONQ_TESTS_EXPORT PreviewSelectorBar : public NotificationBar
 
 public:
     PreviewSelectorBar(int index, QWidget *parent);
-    ~PreviewSelectorBar();
 
-    inline void setIndex(int index) { m_previewIndex = index; }
+    inline void setIndex(int index)
+    {
+        m_previewIndex = index;
+    }
 
 private slots:
     void clicked();
@@ -55,8 +57,6 @@ private slots:
 
     void verifyUrl();
 
-    void destroy();
-
 private:
     QPushButton *m_button;
     QLabel *m_label;
