@@ -2,9 +2,9 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2009-2010 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2009-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2009 by Yoram Bar-Haim <<yoram.b at zend dot com>
-* Copyright (C) 2009-2010 by Lionel Chauvin <megabigbug@yahoo.fr>
+* Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -46,25 +46,18 @@
 
 SessionManager::SessionManager(QObject *parent)
         : QObject(parent)
-        , m_safe(true)
+        , m_safe(false)
 {
     m_sessionFilePath = KStandardDirs::locateLocal("appdata" , "session");
 }
 
 
-SessionManager::~SessionManager()
-{
-}
-
-
 void SessionManager::saveSession()
 {
-    if (!m_safe)
+    if (!m_safe || QWebSettings::globalSettings()->testAttribute(QWebSettings::PrivateBrowsingEnabled))
         return;
     m_safe = false;
 
-    if( QWebSettings::globalSettings()->testAttribute(QWebSettings::PrivateBrowsingEnabled) )
-        return;
 
     QFile sessionFile(m_sessionFilePath);
     if (!sessionFile.open(QFile::WriteOnly | QFile::Truncate))
@@ -73,7 +66,7 @@ void SessionManager::saveSession()
         return;
     }
     QTextStream out(&sessionFile);
-    MainWindowList wl = Application::instance()->mainWindowList();
+    MainWindowList wl = rApp->mainWindowList();
     Q_FOREACH(const QWeakPointer<MainWindow> &w, wl)
     {
         out << "window\n";
@@ -106,13 +99,22 @@ bool SessionManager::restoreSession()
 
     QTextStream in(&sessionFile);
     QString line;
+    bool windowAlreadyOpen = rApp->mainWindowList().count();
     do
     {
         line = in.readLine();
         if (line == QL1S("window"))
         {
             line = in.readLine();
-            Application::instance()->loadUrl( KUrl(line), Rekonq::NewWindow);
+            if (windowAlreadyOpen)
+            {
+                rApp->loadUrl(KUrl(line), Rekonq::CurrentTab);
+                windowAlreadyOpen = false;
+            }
+            else
+            {
+                rApp->loadUrl(KUrl(line), Rekonq::NewWindow);
+            }
         }
         else
         {
@@ -124,7 +126,7 @@ bool SessionManager::restoreSession()
                 if (ok)
                 {
                     // Get last mainwindow created which will be first one in mainwindow list
-                    MainWindowList wl = Application::instance()->mainWindowList();
+                    MainWindowList wl = rApp->mainWindowList();
                     if (wl.count() > 0)
                     {
                         MainView *mv = wl[0].data()->mainView();
@@ -134,7 +136,7 @@ bool SessionManager::restoreSession()
             }
             else
             {
-                Application::instance()->loadUrl( KUrl(line), Rekonq::NewFocusedTab);
+                rApp->loadUrl(KUrl(line), Rekonq::NewFocusedTab);
             }
         }
     }
@@ -164,7 +166,7 @@ QStringList SessionManager::closedSites()
         line = in.readLine();
         if (line != QL1S("window"))
         {
-            if(line == QL1S("currenttab"))
+            if (line == QL1S("currenttab"))
             {
                 in.readLine();  // drop out the next field, containing the index of the current tab..
             }
