@@ -2,9 +2,9 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2009-2010 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2009-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2009 by Yoram Bar-Haim <<yoram.b at zend dot com>
-* Copyright (C) 2009-2010 by Lionel Chauvin <megabigbug@yahoo.fr>
+* Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -48,11 +48,17 @@ class REKONQ_TESTS_EXPORT SessionManager : public QObject
     Q_OBJECT
 public:
     SessionManager(QObject *parent = 0);
-    ~SessionManager();
-    bool restoreSession();
+
+    inline void setSessionManagementEnabled(bool on)
+    {
+        m_safe = on;
+    }
 
     QStringList closedSites();
 
+public slots:
+    bool restoreSession();
+
 private slots:
     void saveSession();
 
