@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2010 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2010-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -48,9 +48,9 @@ GeneralWidget::GeneralWidget(QWidget *parent)
     connect(setHomeToCurrentPageButton, SIGNAL(clicked()), this, SLOT(setHomeToCurrentPage()));
 
     disableHomeSettings(ReKonfig::useNewTabPage());
-    
+
     connect(kcfg_useNewTabPage, SIGNAL(toggled(bool)), this, SLOT(disableHomeSettings(bool)));
-    
+
     checkKGetPresence();
 }
 
@@ -73,7 +73,7 @@ void GeneralWidget::hasChanged()
 
 void GeneralWidget::setHomeToCurrentPage()
 {
-    MainWindow *mw = Application::instance()->mainWindow();
+    MainWindow *mw = rApp->mainWindow();
     WebTab *webTab = mw->currentTab();
     if (webTab)
     {
@@ -95,7 +95,7 @@ void GeneralWidget::checkKGetPresence()
         kcfg_kgetDownload->setDisabled(true);
         kcfg_kgetList->setDisabled(true);
         kcfg_kgetDownload->setToolTip(i18n("Install KGet to enable rekonq to use KGet as download manager"));
-        
+
     }
     else
     {
