@@ -2,8 +2,8 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2010 by Andrea Diamantini <adjam7 at gmail dot com>
-* Copyright (C) 2009-2010 by Lionel Chauvin <megabigbug@yahoo.fr>
+* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -37,6 +37,7 @@
 #include "mainwindow.h"
 #include "webtab.h"
 #include "searchengine.h"
+#include "opensearchmanager.h"
 
 // Widget Includes
 #include "adblockwidget.h"
@@ -70,7 +71,7 @@ private:
     WebKitWidget *webkitWidg;
     NetworkWidget *networkWidg;
     AdBlockWidget *adBlockWidg;
-    
+
     KCModuleProxy *ebrowsingModule;
 
     KShortcutsEditor *shortcutsEditor;
@@ -94,13 +95,13 @@ Private::Private(SettingsDialog *parent)
     tabsWidg->layout()->setMargin(0);
     pageItem = parent->addPage(tabsWidg, i18n("Tabs"));
     pageItem->setIcon(KIcon("tab-duplicate"));
-    
+
     // -- 3
     appearanceWidg = new AppearanceWidget(parent);
     appearanceWidg->layout()->setMargin(0);
     pageItem = parent->addPage(appearanceWidg, i18n("Appearance"));
     pageItem->setIcon(KIcon("preferences-desktop-font"));
-    
+
     // -- 4
     webkitWidg = new WebKitWidget(parent);
     webkitWidg->layout()->setMargin(0);
@@ -108,7 +109,7 @@ Private::Private(SettingsDialog *parent)
     QString webkitIconPath = KStandardDirs::locate("appdata", "pics/webkit-icon.png");
     KIcon webkitIcon = KIcon(QIcon(webkitIconPath));
     pageItem->setIcon(webkitIcon);
-    
+
     // -- 5
     networkWidg = new NetworkWidget(parent);
     networkWidg->layout()->setMargin(0);
@@ -122,17 +123,22 @@ Private::Private(SettingsDialog *parent)
     pageItem->setIcon(KIcon("preferences-web-browser-adblock"));
 
     // -- 7
-    shortcutsEditor = new KShortcutsEditor(Application::instance()->mainWindow()->actionCollection(), parent);
+    shortcutsEditor = new KShortcutsEditor(rApp->mainWindow()->actionCollection(), parent);
     pageItem = parent->addPage(shortcutsEditor , i18n("Shortcuts"));
     pageItem->setIcon(KIcon("configure-shortcuts"));
 
     // -- 8
     KCModuleInfo ebrowsingInfo("ebrowsing.desktop");
     ebrowsingModule = new KCModuleProxy(ebrowsingInfo, parent);
-    pageItem = parent->addPage(ebrowsingModule, i18n(ebrowsingInfo.moduleName().toLocal8Bit()));
-    pageItem->setIcon(KIcon(ebrowsingInfo.icon()));
+    pageItem = parent->addPage(ebrowsingModule, i18n("Search Engines"));
+    KIcon wsIcon("edit-web-search");
+    if (wsIcon.isNull())
+    {
+        wsIcon = KIcon("preferences-web-browser-shortcuts");
+    }
+    pageItem->setIcon(wsIcon);
 
-    // WARNING 
+    // WARNING
     // remember wheh changing here that the smallest netbooks
     // have a 1024x576 resolution. So DON'T bother that limits!!
     parent->setMinimumSize(700, 525);
@@ -151,7 +157,7 @@ SettingsDialog::SettingsDialog(QWidget *parent)
     setModal(true);
 
     readConfig();
-    
+
     // update buttons
     connect(d->generalWidg,     SIGNAL(changed(bool)), this, SLOT(updateButtons()));
     connect(d->tabsWidg,        SIGNAL(changed(bool)), this, SLOT(updateButtons()));
@@ -187,7 +193,7 @@ void SettingsDialog::saveSettings()
         return;
 
     ReKonfig::self()->writeConfig();
-    
+
     d->generalWidg->save();
     d->tabsWidg->save();
     d->appearanceWidg->save();
@@ -196,8 +202,9 @@ void SettingsDialog::saveSettings()
     d->adBlockWidg->save();
     d->shortcutsEditor->save();
     d->ebrowsingModule->save();
-    
+
     SearchEngine::reload();
+    rApp->opensearchManager()->removeDeletedEngines();
 
     updateButtons();
     emit settingsChanged("ReKonfig");
@@ -222,8 +229,8 @@ bool SettingsDialog::hasChanged()
 bool SettingsDialog::isDefault()
 {
     bool isDef = KConfigDialog::isDefault();
-    
-    if(isDef)
+
+    if (isDef)
     {
         // check our private widget values
         isDef = d->appearanceWidg->isDefault();
