@@ -3,9 +3,9 @@
 * This file is a part of the rekonq project
 *
 * Copyright (C) 2008 Benjamin C. Meyer <ben@meyerhome.net>
-* Copyright (C) 2008-2010 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2009 by Paweł Prażak <pawelprazak at gmail dot com>
-* Copyright (C) 2009-2010 by Lionel Chauvin <megabigbug@yahoo.fr>
+* Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -41,28 +41,41 @@
 #include "webpage.h"
 #include "webtab.h"
 #include "websnap.h"
+#include "tabhighlighteffect.h"
 
 // KDE Includes
 #include <KActionMenu>
 #include <KMenu>
 #include <KPassivePopup>
 #include <KToolBar>
+#include <KColorScheme>
 
 // Qt Includes
 #include <QtGui/QLabel>
 #include <QtGui/QLayout>
 #include <QtGui/QMouseEvent>
 #include <QtGui/QToolButton>
+#include <QPropertyAnimation>
+#include <QStyleOptionFrameV3>
 
 
 #define BASE_WIDTH_DIVISOR    4
 #define MIN_WIDTH_DIVISOR     8
 
 
+static inline QByteArray highlightPropertyName(int index)
+{
+    return QByteArray("hAnim").append(QByteArray::number(index));
+}
+
+
 TabBar::TabBar(QWidget *parent)
-        : KTabBar(parent)
-        , m_actualIndex(-1)
-        , m_currentTabPreviewIndex(-1)
+    : KTabBar(parent)
+    , m_actualIndex(-1)
+    , m_currentTabPreviewIndex(-1)
+    , m_isFirstTimeOnTab(true)
+    , m_tabHighlightEffect(new TabHighlightEffect(this))
+    , m_animationMapper(new QSignalMapper(this))
 {
     setElideMode(Qt::ElideRight);
 
@@ -74,11 +87,11 @@ TabBar::TabBar(QWidget *parent)
 
     connect(this, SIGNAL(contextMenu(int, const QPoint &)), this, SLOT(contextMenu(int, const QPoint &)));
     connect(this, SIGNAL(emptyAreaContextMenu(const QPoint &)), this, SLOT(emptyAreaContextMenu(const QPoint &)));
-}
 
+    connect(m_animationMapper, SIGNAL(mapped(int)), this, SLOT(removeAnimation(int)));
+    setGraphicsEffect(m_tabHighlightEffect);
 
-TabBar::~TabBar()
-{
+    setAnimatedTabHighlighting( ReKonfig::animatedTabHighlighting() );
 }
 
 
@@ -152,6 +165,9 @@ void TabBar::detachTab()
 
 void TabBar::showTabPreview()
 {
+    if (m_isFirstTimeOnTab)
+        m_isFirstTimeOnTab = false;
+
     //delete previous tab preview
     delete m_previewPopup.data();
     m_previewPopup.clear();
@@ -166,7 +182,7 @@ void TabBar::showTabPreview()
         return;
 
     // no previews during load
-    if (indexedTab->progress() != 0)
+    if (indexedTab->isPageLoading())
         return;
 
     int w = tabSizeHint(m_currentTabPreviewIndex).width();
@@ -178,7 +194,7 @@ void TabBar::showTabPreview()
     m_previewPopup.data()->setFixedSize(w, h);
 
     QLabel *l = new QLabel();
-    l->setPixmap( WebSnap::renderTabPreview(*indexedTab->page(), w, h) );
+    l->setPixmap(WebSnap::renderTabPreview(*indexedTab->page(), w, h));
 
     m_previewPopup.data()->setView(l);
     m_previewPopup.data()->layout()->setAlignment(Qt::AlignTop);
@@ -189,6 +205,26 @@ void TabBar::showTabPreview()
 }
 
 
+void TabBar::hideEvent(QHideEvent *event)
+{
+    if (!event->spontaneous())
+    {
+        qobject_cast<MainView *>(parent())->addTabButton()->hide();
+    }
+    KTabBar::hideEvent(event);
+}
+
+
+void TabBar::showEvent(QShowEvent *event)
+{
+    KTabBar::showEvent(event);
+    if (!event->spontaneous())
+    {
+        qobject_cast<MainView *>(parent())->addTabButton()->show();
+    }
+}
+
+
 void TabBar::mouseMoveEvent(QMouseEvent *event)
 {
     if (count() == 1)
@@ -196,30 +232,12 @@ void TabBar::mouseMoveEvent(QMouseEvent *event)
         return;
     }
 
-    if (event->buttons() & Qt::LeftButton)
-    {
-        // hide addNewTabButton when moving tabs
-        MainView *view = qobject_cast<MainView *>(parent());
-        QTimer::singleShot(200, view->addTabButton(), SLOT(hide()));
-    }
-
     KTabBar::mouseMoveEvent(event);
 
-    if (ReKonfig::alwaysShowTabPreviews())
+    if (ReKonfig::hoveringTabOption() == 0)
     {
         //Find the tab under the mouse
-        int i = 0;
-        int tabIndex = -1;
-        while (i < count()
-                && tabIndex == -1
-              )
-        {
-            if (tabRect(i).contains(event->pos()))
-            {
-                tabIndex = i;
-            }
-            i++;
-        }
+        const int tabIndex = tabAt(event->pos());
 
         // if found and not the current tab then show tab preview
         if (tabIndex != -1
@@ -229,7 +247,11 @@ void TabBar::mouseMoveEvent(QMouseEvent *event)
            )
         {
             m_currentTabPreviewIndex = tabIndex;
-            QTimer::singleShot(200, this, SLOT( showTabPreview() ) );
+
+            // if first time over tab, apply a small delay. If not, show it now!
+            m_isFirstTimeOnTab
+            ? QTimer::singleShot(200, this, SLOT(showTabPreview()))
+            : showTabPreview();
         }
 
         // if current tab or not found then hide previous tab preview
@@ -247,7 +269,7 @@ void TabBar::mouseMoveEvent(QMouseEvent *event)
 
 void TabBar::leaveEvent(QEvent *event)
 {
-    if (ReKonfig::alwaysShowTabPreviews())
+    if (ReKonfig::hoveringTabOption() == 0)
     {
         //if leave tabwidget then hide previous tab preview
         if (!m_previewPopup.isNull())
@@ -255,6 +277,7 @@ void TabBar::leaveEvent(QEvent *event)
             m_previewPopup.data()->hide();
         }
         m_currentTabPreviewIndex = -1;
+        m_isFirstTimeOnTab = true;
     }
 
     KTabBar::leaveEvent(event);
@@ -263,7 +286,7 @@ void TabBar::leaveEvent(QEvent *event)
 
 void TabBar::mousePressEvent(QMouseEvent *event)
 {
-    if (ReKonfig::alwaysShowTabPreviews())
+    if (ReKonfig::hoveringTabOption() == 0)
     {
         if (!m_previewPopup.isNull())
         {
@@ -287,20 +310,20 @@ void TabBar::contextMenu(int tab, const QPoint &pos)
     m_actualIndex = tab;
 
     KMenu menu;
-    MainWindow *mainWindow = Application::instance()->mainWindow();
+    MainWindow *mainWindow = rApp->mainWindow();
 
-    menu.addAction(mainWindow->actionByName( QL1S("new_tab") ));
-    menu.addAction(mainWindow->actionByName( QL1S("clone_tab") ));
+    menu.addAction(mainWindow->actionByName(QL1S("new_tab")));
+    menu.addAction(mainWindow->actionByName(QL1S("clone_tab")));
     if (count() > 1)
-        menu.addAction(mainWindow->actionByName( QL1S("detach_tab") ));
-    menu.addAction(mainWindow->actionByName( QL1S("open_closed_tabs") ));
-    menu.addAction(mainWindow->actionByName( QL1S("closed_tab_menu") ));
+        menu.addAction(mainWindow->actionByName(QL1S("detach_tab")));
+    menu.addAction(mainWindow->actionByName(QL1S("open_last_closed_tab")));
+    menu.addAction(mainWindow->actionByName(QL1S("closed_tab_menu")));
     menu.addSeparator();
-    menu.addAction(mainWindow->actionByName( QL1S("close_tab") ));
-    menu.addAction(mainWindow->actionByName( QL1S("close_other_tabs") ));
+    menu.addAction(mainWindow->actionByName(QL1S("close_tab")));
+    menu.addAction(mainWindow->actionByName(QL1S("close_other_tabs")));
     menu.addSeparator();
-    menu.addAction(mainWindow->actionByName( QL1S("reload_tab") ));
-    menu.addAction(mainWindow->actionByName( QL1S("reload_all_tabs") ));
+    menu.addAction(mainWindow->actionByName(QL1S("reload_tab")));
+    menu.addAction(mainWindow->actionByName(QL1S("reload_all_tabs")));
 
     menu.exec(pos);
 }
@@ -311,37 +334,27 @@ void TabBar::emptyAreaContextMenu(const QPoint &pos)
     setupHistoryActions();
 
     KMenu menu;
-    MainWindow *mainWindow = Application::instance()->mainWindow();
+    MainWindow *mainWindow = rApp->mainWindow();
 
-    menu.addAction(mainWindow->actionByName( QL1S("new_tab") ));
-    menu.addAction(mainWindow->actionByName( QL1S("open_closed_tabs") ));
-    menu.addAction(mainWindow->actionByName( QL1S("closed_tab_menu") ));
+    menu.addAction(mainWindow->actionByName(QL1S("new_tab")));
+    menu.addAction(mainWindow->actionByName(QL1S("open_last_closed_tab")));
+    menu.addAction(mainWindow->actionByName(QL1S("closed_tab_menu")));
     menu.addSeparator();
-    menu.addAction(mainWindow->actionByName( QL1S("reload_all_tabs") ));
+    menu.addAction(mainWindow->actionByName(QL1S("reload_all_tabs")));
 
     KToolBar *mainBar = mainWindow->toolBar("mainToolBar");
-    if(!mainBar->isVisible())
+    if (!mainBar->isVisible())
     {
-        menu.addAction( mainBar->toggleViewAction() );
+        menu.addAction(mainBar->toggleViewAction());
     }
 
     menu.exec(pos);
 }
 
 
-void TabBar::mouseReleaseEvent(QMouseEvent *event)
-{
-    MainView *mv = qobject_cast<MainView *>(parent());
-    QTimer::singleShot(200, mv->addTabButton(), SLOT(show()));
-
-    KTabBar::mouseReleaseEvent(event);
-}
-
-
 void TabBar::tabRemoved(int index)
 {
-    Q_UNUSED(index)
-    if (ReKonfig::alwaysShowTabPreviews())
+    if (ReKonfig::hoveringTabOption() == 0)
     {
         if (!m_previewPopup.isNull())
         {
@@ -349,36 +362,123 @@ void TabBar::tabRemoved(int index)
         }
         m_currentTabPreviewIndex = -1;
     }
+
+    if (ReKonfig::animatedTabHighlighting())
+        removeAnimation(index);
 }
 
 
 void TabBar::setupHistoryActions()
 {
-    MainWindow *w = Application::instance()->mainWindow();
+    MainWindow *w = rApp->mainWindow();
     MainView *mv = qobject_cast<MainView *>(parent());
 
-    QAction *openClosedTabsAction = w->actionByName( QL1S("open_closed_tabs") );
-    openClosedTabsAction->setEnabled( mv->recentlyClosedTabs().size() > 0 );
+    QAction *openLastClosedTabAction = w->actionByName(QL1S("open_last_closed_tab"));
+    openLastClosedTabAction->setEnabled(mv->recentlyClosedTabs().size() > 0);
 
     // update closed tabs menu
-    KActionMenu *am = qobject_cast<KActionMenu *>( w->actionByName( QL1S("closed_tab_menu") ));
+    KActionMenu *am = qobject_cast<KActionMenu *>(w->actionByName(QL1S("closed_tab_menu")));
     if (!am)
         return;
 
-    bool isEnabled = ( mv->recentlyClosedTabs().size() > 0 );
+    bool isEnabled = (mv->recentlyClosedTabs().size() > 0);
     am->setEnabled(isEnabled);
 
     if (am->menu())
         am->menu()->clear();
 
-    if(!isEnabled)
+    if (!isEnabled)
         return;
 
-    foreach (const HistoryItem &item, mv->recentlyClosedTabs())
+    Q_FOREACH(const HistoryItem &item, mv->recentlyClosedTabs())
     {
-        KAction *a = new KAction(Application::iconManager()->iconForUrl(item.url), item.title, this);
+        KAction *a = new KAction(rApp->iconManager()->iconForUrl(item.url), item.title, this);
         a->setData(item.url);
         connect(a, SIGNAL(triggered()), mv, SLOT(openClosedTab()));
         am->addAction(a);
     }
 }
+
+
+QRect TabBar::tabTextRect(int index)
+{
+    QStyleOptionTabV3 option;
+    initStyleOption(&option, index);
+    return style()->subElementRect(QStyle::SE_TabBarTabText, &option, this);
+}
+
+
+void TabBar::setTabHighlighted(int index)
+{
+    const QByteArray propertyName = highlightPropertyName(index);
+    const QColor highlightColor = KColorScheme(QPalette::Active, KColorScheme::Window).foreground(KColorScheme::PositiveText).color();
+
+    if (tabTextColor(index) != highlightColor)
+    {
+        if (ReKonfig::animatedTabHighlighting())
+        {
+            m_tabHighlightEffect->setEnabled(true);
+            m_tabHighlightEffect->setProperty(propertyName, qreal(0.9));
+            QPropertyAnimation *anim = new QPropertyAnimation(m_tabHighlightEffect, propertyName);
+            m_highlightAnimation.insert(propertyName, anim);
+
+            //setup the animation
+            anim->setStartValue(0.9);
+            anim->setEndValue(0.0);
+            anim->setDuration(500);
+            anim->setLoopCount(2);
+            anim->start(QAbstractAnimation::DeleteWhenStopped);
+
+            m_animationMapper->setMapping(anim, index);
+            connect(anim, SIGNAL(finished()), m_animationMapper, SLOT(map()));
+        }
+ 
+        setTabTextColor(index, highlightColor);
+    }
+}
+
+
+void TabBar::resetTabHighlighted(int index)
+{
+    if (ReKonfig::animatedTabHighlighting())
+        removeAnimation(index);
+
+    setTabTextColor(index, palette().text().color());
+}
+
+
+void TabBar::removeAnimation(int index)
+{
+    const QByteArray propertyName = highlightPropertyName(index);
+    m_tabHighlightEffect->setProperty(propertyName, QVariant()); //destroy the property
+
+    QPropertyAnimation *anim = m_highlightAnimation.take(propertyName);
+    m_animationMapper->removeMappings(anim);
+    delete anim;
+
+    if (m_highlightAnimation.isEmpty())
+        m_tabHighlightEffect->setEnabled(false);
+}
+
+
+void TabBar::setAnimatedTabHighlighting(bool enabled)
+{
+    if (enabled)
+        m_tabHighlightEffect->setEnabled(true);
+    else
+    {
+        m_tabHighlightEffect->setEnabled(false);
+
+        //cleanup
+        QHashIterator<QByteArray, QPropertyAnimation*> i(m_highlightAnimation);
+        while (i.hasNext())
+        {
+            i.next();
+            m_tabHighlightEffect->setProperty(i.key(), QVariant()); //destroy the property
+
+            QPropertyAnimation *anim = m_highlightAnimation.take(i.key());
+            m_animationMapper->removeMappings(anim);
+            delete anim;
+        }
+    }
+}
