@@ -3,9 +3,9 @@
 * This file is a part of the rekonq project
 *
 * Copyright (C) 2008 Benjamin C. Meyer <ben@meyerhome.net>
-* Copyright (C) 2008-2010 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2009 by Paweł Prażak <pawelprazak at gmail dot com>
-* Copyright (C) 2009-2010 by Lionel Chauvin <megabigbug@yahoo.fr>
+* Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -37,8 +37,13 @@
 // KDE Includes
 #include <KTabBar>
 
+//Qt Includes
+#include <QSignalMapper>
+
 // Forward Declarations
 class KPassivePopup;
+class TabHighlightEffect;
+class QPropertyAnimation;
 
 
 /**
@@ -51,8 +56,12 @@ class REKONQ_TESTS_EXPORT TabBar : public KTabBar
     Q_OBJECT
 
 public:
-    TabBar(QWidget *parent);
-    ~TabBar();
+    explicit TabBar(QWidget *parent);
+
+    void setTabHighlighted(int index);
+    void resetTabHighlighted(int index);
+    QRect tabTextRect(int index);
+    void setAnimatedTabHighlighting(bool enabled);
 
 signals:
     void cloneTab(int index);
@@ -68,11 +77,13 @@ protected:
      */
     virtual QSize tabSizeHint(int index) const;
 
+    virtual void hideEvent(QHideEvent *event);
+    virtual void showEvent(QShowEvent *event);
     virtual void mouseMoveEvent(QMouseEvent *event);
     virtual void leaveEvent(QEvent *event);
     virtual void mousePressEvent(QMouseEvent *event);
-    virtual void mouseReleaseEvent(QMouseEvent *event);
-    virtual void tabRemoved(int);
+
+    virtual void tabRemoved(int index);
 
 private slots:
     void cloneTab();
@@ -86,9 +97,10 @@ private slots:
 
     void showTabPreview();
 
+    void removeAnimation(int index);
+
 private:
     void setupHistoryActions();
-
     friend class MainView;
 
     /**
@@ -102,6 +114,12 @@ private:
      * the index of the tab preview shown
      */
     int m_currentTabPreviewIndex;
+    bool m_isFirstTimeOnTab;
+
+    //highlightAnimation
+    TabHighlightEffect *m_tabHighlightEffect;
+    QHash<QByteArray, QPropertyAnimation*> m_highlightAnimation;
+    QSignalMapper *m_animationMapper;
 };
 
 #endif
