@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2009 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2009-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -66,26 +66,20 @@ CompletionWidget::CompletionWidget(QWidget *parent)
 }
 
 
-void CompletionWidget::insertSearchList(const UrlSearchList &list, const QString& text)
+void CompletionWidget::insertItems(const UrlSearchList &list, const QString& text, int offset)
 {
-    if (!isVisible())
-    {
-        UrlResolver::setSearchEngine(SearchEngine::defaultEngine());
-    }
-
-    _list = list;
-    int i = 0;
-    foreach(const UrlSearchItem &item, _list)
+    Q_FOREACH(const UrlSearchItem &item, list)
     {
         ListItem *suggestion = ListItemFactory::create(item, text, this);
-        suggestion->setBackgroundRole(i % 2 ? QPalette::AlternateBase : QPalette::Base);
-        connect(suggestion, 
-                SIGNAL(itemClicked(ListItem *, Qt::MouseButton, Qt::KeyboardModifiers)), 
-                this, 
+        suggestion->setBackgroundRole(offset % 2 ? QPalette::AlternateBase : QPalette::Base);
+        connect(suggestion,
+                SIGNAL(itemClicked(ListItem *, Qt::MouseButton, Qt::KeyboardModifiers)),
+                this,
                 SLOT(itemChosen(ListItem *, Qt::MouseButton, Qt::KeyboardModifiers)));
+        connect(suggestion, SIGNAL(updateList()), this, SLOT(updateList()));
         connect(this, SIGNAL(nextItemSubChoice()), suggestion, SLOT(nextItemSubChoice()));
-        
-        suggestion->setObjectName(QString::number(i++));
+
+        suggestion->setObjectName(QString::number(offset++));
         layout()->addWidget(suggestion);
     }
 }
@@ -96,32 +90,19 @@ void CompletionWidget::updateSearchList(const UrlSearchList &list, const QString
     static int counter = 0;
     counter++;
     kDebug() << counter;
-    if(_hasSuggestions || _typedString != text)
+    if (_hasSuggestions || _typedString != text)
         return;
     _hasSuggestions = true;
-    
+
     if (_resList.count() > 0)
     {
         clear();
-        insertSearchList(_resList, text);
-    
-        UrlSearchList sugList = list.mid(0,4);
 
-        // add new suggestions to the list
-        int offset = _list.count();
-        Q_FOREACH(const UrlSearchItem &item, sugList)
-        {
-            ListItem *suggestion = ListItemFactory::create(item, text, this);
-            suggestion->setBackgroundRole(offset % 2 ? QPalette::AlternateBase : QPalette::Base);
-            connect(suggestion, 
-                    SIGNAL(itemClicked(ListItem *, Qt::MouseButton, Qt::KeyboardModifiers)), 
-                    this, 
-                    SLOT(itemChosen(ListItem *, Qt::MouseButton, Qt::KeyboardModifiers)));
-            connect(this, SIGNAL(nextItemSubChoice()), suggestion, SLOT(nextItemSubChoice()));
-            
-            suggestion->setObjectName(QString::number(offset++));
-            layout()->addWidget(suggestion);
-        }
+        insertItems(_resList, text);
+        _list = _resList;
+
+        UrlSearchList sugList = list.mid(0, 4);
+        insertItems(sugList, text, _list.count());
         _list.append(sugList);
         popup();
     }
@@ -138,7 +119,7 @@ void CompletionWidget::sizeAndPosition()
         QWidget *widget = layout()->itemAt(i)->widget();
         h += widget->sizeHint().height();
     }
-    setFixedSize(_parent->width(),h+5);
+    setFixedSize(_parent->width(), h + 5);
 
     // position
     QPoint p = _parent->mapToGlobal(QPoint(0, 0));
@@ -224,10 +205,10 @@ bool CompletionWidget::eventFilter(QObject *obj, QEvent *ev)
 
     // hide conditions of the CompletionWidget
     if (wid
-        && ((wid == _parent 
-            && (type == QEvent::Move || type == QEvent::Resize)) || ((wid->windowFlags() & Qt::Window)
-            && (type == QEvent::Move || type == QEvent::Hide || type == QEvent::WindowDeactivate)
-            && wid == _parent->window()) || (type == QEvent::MouseButtonPress && !isAncestorOf(wid)))
+            && ((wid == _parent
+                 && (type == QEvent::Move || type == QEvent::Resize)) || ((wid->windowFlags() & Qt::Window)
+                         && (type == QEvent::Move || type == QEvent::Hide || type == QEvent::WindowDeactivate)
+                         && wid == _parent->window()) || (type == QEvent::MouseButtonPress && !isAncestorOf(wid)))
        )
     {
         hide();
@@ -239,7 +220,7 @@ bool CompletionWidget::eventFilter(QObject *obj, QEvent *ev)
     {
         ListItem *child;
         UrlBar *w;
-        
+
         if (type == QEvent::KeyPress)
         {
             QKeyEvent *kev = static_cast<QKeyEvent *>(ev);
@@ -274,6 +255,13 @@ bool CompletionWidget::eventFilter(QObject *obj, QEvent *ev)
             case Qt::Key_Enter:
             case Qt::Key_Return:
                 w = qobject_cast<UrlBar *>(parent());
+                if (kev->modifiers() == Qt::AltModifier)
+                {
+                    if (kev->key() == Qt::Key_Return || kev->key() == Qt::Key_Enter)
+                    {
+                        emit chosenUrl(w->text(), Rekonq::NewFocusedTab);
+                    }
+                }
 
                 if (!w->text().startsWith(QL1S("http://"), Qt::CaseInsensitive))
                 {
@@ -300,29 +288,41 @@ bool CompletionWidget::eventFilter(QObject *obj, QEvent *ev)
                             host += append;
                             url.setHost(host);
                         }
-                        
+
                         emit chosenUrl(url, Rekonq::CurrentTab);
                     }
                 }
 
-                
-                if( _currentIndex == -1)
+
+                if (_currentIndex == -1)
                     _currentIndex = 0;
                 child = findChild<ListItem *>(QString::number(_currentIndex));
-                if(child && _currentIndex!=0) //the completionwidget is visible and the user had press down
+                if (child && _currentIndex != 0) //the completionwidget is visible and the user had press down
                 {
+                    kDebug() << "USING LISTITEM URL: " << child->url();
+                    kDebug() << "USING LISTITEM TITLE: " << child->text();
+
                     //we can use the url of the listitem
                     emit chosenUrl(child->url(), Rekonq::CurrentTab);
                 }
                 else //the user type too fast (completionwidget not visible or suggestion not downloaded)
                 {
                     UrlResolver res(w->text());
-                    emit chosenUrl(res.orderedSearchItems().first().url, Rekonq::CurrentTab);
+                    UrlSearchList list = res.orderedSearchItems();
+                    if(list.isEmpty())
+                    {
+                        kDebug() << "Url Search List EMPTY!!!";
+                        emit chosenUrl(KUrl(_typedString), Rekonq::CurrentTab);
+                    }
+                    else
+                    {
+                        emit chosenUrl(list.first().url, Rekonq::CurrentTab);
+                    }
                 }
                 kev->accept();
                 hide();
                 return true;
-                
+
             case Qt::Key_Escape:
                 hide();
                 return true;
@@ -338,11 +338,11 @@ void CompletionWidget::setVisible(bool visible)
 {
     if (visible)
     {
-        Application::instance()->installEventFilter(this);
+        rApp->installEventFilter(this);
     }
     else
     {
-        Application::instance()->removeEventFilter(this);
+        rApp->removeEventFilter(this);
     }
 
 
@@ -352,8 +352,8 @@ void CompletionWidget::setVisible(bool visible)
 
 void CompletionWidget::itemChosen(ListItem *item, Qt::MouseButton button, Qt::KeyboardModifiers modifier)
 {
-    if (button == Qt::MidButton 
-        || modifier == Qt::ControlModifier)
+    if (button == Qt::MidButton
+            || modifier == Qt::ControlModifier)
     {
         emit chosenUrl(item->url(), Rekonq::NewFocusedTab);
     }
@@ -365,10 +365,16 @@ void CompletionWidget::itemChosen(ListItem *item, Qt::MouseButton button, Qt::Ke
 }
 
 
+void CompletionWidget::updateList()
+{
+    suggestUrls(_typedString);
+}
+
+
 void CompletionWidget::suggestUrls(const QString &text)
 {
     _typedString = text;
-    
+
     QWidget *w = qobject_cast<QWidget *>(parent());
     if (!w->hasFocus())
         return;
@@ -379,8 +385,15 @@ void CompletionWidget::suggestUrls(const QString &text)
         return;
     }
 
+    if (!isVisible())
+    {
+        UrlResolver::setSearchEngine(SearchEngine::defaultEngine());
+    }
+
     UrlResolver *res = new UrlResolver(text);
     connect(res, SIGNAL(suggestionsReady(const UrlSearchList &, const QString &)), this, SLOT(updateSearchList(const UrlSearchList &, const QString &)));
-    res->computeSuggestions();
     _resList = res->orderedSearchItems();
+
+    // NOTE: It's important to call this AFTER orderedSearchItems() to let everything work
+    res->computeSuggestions();
 }
