@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2009 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2009-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -56,13 +56,14 @@ public:
 private slots:
     void itemChosen(ListItem *item, Qt::MouseButton = Qt::LeftButton, Qt::KeyboardModifiers = Qt::NoModifier);
     void updateSearchList(const UrlSearchList &list, const QString& text);
+    void updateList();
 
 signals:
     void chosenUrl(const KUrl &, Rekonq::OpenType);
     void nextItemSubChoice();
 
 private:
-    void insertSearchList(const UrlSearchList &list, const QString& text);
+    void insertItems(const UrlSearchList &list, const QString& text, int offset = 0);
 
     void popup();
     void clear();
@@ -82,7 +83,7 @@ private:
 
     QString _typedString;
     bool _hasSuggestions;
-    
+
     UrlSearchList _resList;
 };
 
