@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2009 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2009-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -42,6 +42,8 @@
 // KDE Includes
 #include <KIcon>
 #include <KAction>
+#include <kio/jobclasses.h>
+#include <kio/scheduler.h>
 
 // Qt Includes
 #include <QActionGroup>
@@ -56,17 +58,18 @@
 #include <QFile>
 #include <QTextDocument>
 
+
 ListItem::ListItem(const UrlSearchItem &item, QWidget *parent)
         : QWidget(parent)
         , m_option()
         , m_url(item.url)
-{    
+{
     m_option.initFrom(this);
-    m_option.direction = Qt::LeftToRight;   
+    m_option.direction = Qt::LeftToRight;
 
     // use the same application palette (hence, the same colors)
     // Qt docs says that using this cctor is possible & fast (qt:qpalette)
-    QPalette p(Application::palette());
+    QPalette p(rApp->palette());
     setPalette(p);
 
     deactivate();
@@ -79,7 +82,6 @@ ListItem::~ListItem()
 }
 
 
-
 void ListItem::activate()
 {
     m_option.state |= QStyle::State_Selected;
@@ -96,7 +98,7 @@ void ListItem::deactivate()
 
 void ListItem::paintEvent(QPaintEvent *event)
 {
-    Q_UNUSED(event);   
+    Q_UNUSED(event);
 
     QWidget::paintEvent(event);
     QPainter painter(this);
@@ -164,11 +166,16 @@ TypeIconLabel::TypeIconLabel(int type, QWidget *parent)
     hLayout->setAlignment(Qt::AlignRight);
     setLayout(hLayout);
 
-    if (type & UrlSearchItem::Search) hLayout->addWidget(getIcon("edit-find"));
-    if (type & UrlSearchItem::Browse) hLayout->addWidget(getIcon("applications-internet"));
-    if (type & UrlSearchItem::Bookmark) hLayout->addWidget(getIcon("rating"));
-    if (type & UrlSearchItem::History) hLayout->addWidget(getIcon("view-history"));
-    if (type & UrlSearchItem::Suggestion) hLayout->addWidget(getIcon("help-hint"));
+    if (type & UrlSearchItem::Search)
+        hLayout->addWidget(getIcon("edit-find"));
+    if (type & UrlSearchItem::Browse)
+        hLayout->addWidget(getIcon("applications-internet"));
+    if (type & UrlSearchItem::Bookmark)
+        hLayout->addWidget(getIcon("rating"));
+    if (type & UrlSearchItem::History)
+        hLayout->addWidget(getIcon("view-history"));
+    if (type & UrlSearchItem::Suggestion)
+        hLayout->addWidget(getIcon("help-hint"));
 }
 
 
@@ -188,56 +195,126 @@ QLabel *TypeIconLabel::getIcon(QString icon)
 IconLabel::IconLabel(const QString &icon, QWidget *parent)
         : QLabel(parent)
 {
-    QPixmap pixmapIcon = Application::iconManager()->iconForUrl(KUrl(icon)).pixmap(16);
+    QPixmap pixmapIcon = rApp->iconManager()->iconForUrl(KUrl(icon)).pixmap(16);
     setFixedSize(16, 16);
     setPixmap(pixmapIcon);
 }
 
 
 IconLabel::IconLabel(const KIcon &icon, QWidget *parent)
-    : QLabel(parent)
+        : QLabel(parent)
 {
     QPixmap pixmapIcon = icon.pixmap(16);
     setFixedSize(16, 16);
     setPixmap(pixmapIcon);
 }
 
+
 // ---------------------------------------------------------------
 
 
+static QString highlightWordsInText(const QString &text, const QStringList &words)
+{
+    QString ret = text;
+    QBitArray boldSections(ret.size());
+    foreach(const QString &wordToPointOut, words)
+    {
+        int index = ret.indexOf(wordToPointOut, 0, Qt::CaseInsensitive);
+        while (index > -1)
+        {
+            boldSections.fill(true, index, index + wordToPointOut.size());
+            index = ret.indexOf(wordToPointOut, index + wordToPointOut.size(), Qt::CaseInsensitive);
+        }
+    }
+
+    
+    if (boldSections.isEmpty())
+        return ret;
+    
+    int numSections = 0;
+    for (int i = 0; i < boldSections.size() - 1; ++i)
+    {
+        if (boldSections.testBit(i) && !boldSections.testBit(i + 1))
+            ++numSections;
+    }
+    if (boldSections.testBit(boldSections.size() - 1)) //last char was still part of a bold section
+        ++numSections;
+    const int tagLength = 7; // length of "<b>" and "</b>" we're going to add for each bold section.
+    ret.reserve(ret.size() + numSections * tagLength);
+    bool bold = false;
+    for (int i = boldSections.size() - 1; i >= 0; --i)
+    {
+        if (!bold && boldSections.testBit(i))
+        {
+            ret.insert(i + 1, QL1S("</b>"));
+            bold = true;
+        }
+        else if (bold && !boldSections.testBit(i))
+        {
+            ret.insert(i + 1, QL1S("<b>"));
+            bold = false;
+        }
+    }
+    if (bold)
+        ret.insert(0, QL1S("<b>"));
+    return ret;
+}
+
+
 TextLabel::TextLabel(const QString &text, const QString &textToPointOut, QWidget *parent)
         : QLabel(parent)
+{
+    setTextFormat(Qt::RichText);
+    setMouseTracking(false);
+    QString t = text;
+    const bool wasItalic = t.startsWith(QL1S("<i>"));
+    if (wasItalic)
+        t.remove(QRegExp(QL1S("<[/ib]*>")));
+    t = Qt::escape(t);
+    QStringList words = Qt::escape(textToPointOut.simplified()).split(QL1C(' '));
+    t = highlightWordsInText(t, words);
+    if (wasItalic)
+        t = QL1S("<i>") + t + QL1S("</i>");
+    setText(t);
+    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);
+}
+
+
+TextLabel::TextLabel(QWidget *parent)
+        : QLabel(parent)
+{
+    setTextFormat(Qt::RichText);
+    setMouseTracking(false);
+    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);
+}
+
+
+void TextLabel::setEngineText(const QString &engine, const QString &text)
+{
+    setText(i18nc("%1=search engine, e.g. Google, Wikipedia %2=text to search for", "Search %1 for <b>%2</b>", engine, Qt::escape(text)));
+}
+
+
+// ---------------------------------------------------------------
+
+
+DescriptionLabel::DescriptionLabel(const QString &text, QWidget *parent)
+        : QLabel(parent)
 {
     QString t = text;
     const bool wasItalic = t.startsWith(QL1S("<i>"));
     if (wasItalic)
         t.remove(QRegExp("<[/ib]*>"));
 
-    t = Qt::escape(t);
-    QString ss = Qt::escape(textToPointOut);
-    t.replace(QRegExp('(' + ss + ')', Qt::CaseInsensitive), "<b>\\1</b>");
-
     if (wasItalic)
         t = QL1S("<i>") + t + QL1S("</i>");
 
+    setWordWrap(false); //TODO: why setWordWrap(true) make items have a strange behavior ?
     setText(t);
     setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);
 }
 
 
-TextLabel::TextLabel(QWidget *parent)
-    : QLabel(parent)
-{
-    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);    
-}
-
-
-void TextLabel::setEngineText(const QString &engine, const QString &text)
-{
-    setText( i18nc("%1=search engine, e.g. Google, Wikipedia %2=text to search for", "Search %1 for <b>%2</b>", engine, Qt::escape(text) ) );
-}
-
-
 //--------------------------------------------------------------------------------------------
 
 
@@ -298,14 +375,55 @@ PreviewLabel::PreviewLabel(const QString &url, int width, int height, QWidget *p
 // ---------------------------------------------------------------
 
 
+ImageLabel::ImageLabel(const QString &url, int width, int height, QWidget *parent)
+        : QLabel(parent),
+        m_url(url)
+{
+    setFixedSize(width, height);
+    if (WebSnap::existsImage(KUrl(url)))
+    {
+        QPixmap pix;
+        pix.load(WebSnap::imagePathFromUrl(url));
+        setPixmap(pix);
+    }
+    else
+    {
+        KIO::TransferJob *job = KIO::get(KUrl(url), KIO::NoReload, KIO::HideProgressInfo);
+        connect(job,  SIGNAL(data(KIO::Job *, const QByteArray &)),
+                this, SLOT(slotData(KIO::Job*, const QByteArray&)));
+        connect(job,  SIGNAL(result(KJob *)),
+                this, SLOT(slotResult(KJob *)));
+    }
+}
+
+
+void ImageLabel::slotData(KIO::Job *job, const QByteArray &data)
+{
+    Q_UNUSED(job);
+    m_data.append(data);
+}
+
+
+void ImageLabel::slotResult(KJob *)
+{
+    QPixmap pix;
+    if (!pix.loadFromData(m_data))
+        kDebug() << "error while loading image: ";
+    setPixmap(pix);
+    pix.save(WebSnap::imagePathFromUrl(m_url), "PNG");
+}
+
+
+// ---------------------------------------------------------------
+
+
 SearchListItem::SearchListItem(const UrlSearchItem &item, const QString &text, QWidget *parent)
         : ListItem(item, parent)
         , m_text(text)
 {
     m_iconLabel = new IconLabel(SearchEngine::buildQuery(UrlResolver::searchEngine(), ""), this);
-    QString query = SearchEngine::extractQuery(text);
     m_titleLabel = new TextLabel(this);
-    m_titleLabel->setEngineText(item.title, query);
+    m_titleLabel->setEngineText(UrlResolver::searchEngine()->name(), item.title);
     m_engineBar = new EngineBar(UrlResolver::searchEngine(), parent);
 
     QHBoxLayout *hLayout = new QHBoxLayout;
@@ -329,18 +447,10 @@ QString SearchListItem::text()
 }
 
 
-// QString SearchListItem::searchItemTitle(QString engine, QString text)
-// {
-//     return QString(i18nc("%1=search engine, e.g. Google, Wikipedia %2=text to search for", "Search %1 for %2", engine, text));
-// }
-
-
 void SearchListItem::changeSearchEngine(KService::Ptr engine)
 {
-    m_titleLabel->setEngineText(engine->name(), m_text);
-    m_iconLabel->setPixmap( Application::iconManager()->iconForUrl(KUrl(engine->property("Query").toString())).pixmap(16) );
-    m_url = SearchEngine::buildQuery(engine, m_text);
     UrlResolver::setSearchEngine(engine);
+    emit updateList();
 }
 
 
@@ -362,6 +472,8 @@ EngineBar::EngineBar(KService::Ptr selectedEngine, QWidget *parent)
     m_engineGroup = new QActionGroup(this);
     m_engineGroup->setExclusive(true);
 
+    if (SearchEngine::defaultEngine().isNull())
+        return;
     m_engineGroup->addAction(newEngineAction(SearchEngine::defaultEngine(), selectedEngine));
     foreach(KService::Ptr engine, SearchEngine::favorites())
     {
@@ -378,10 +490,10 @@ EngineBar::EngineBar(KService::Ptr selectedEngine, QWidget *parent)
 KAction *EngineBar::newEngineAction(KService::Ptr engine, KService::Ptr selectedEngine)
 {
     QUrl u = engine->property("Query").toUrl();
-    KUrl url = KUrl( u.toString( QUrl::RemovePath | QUrl::RemoveQuery ) );
-    
+    KUrl url = KUrl(u.toString(QUrl::RemovePath | QUrl::RemoveQuery));
+
     kDebug() << "Engine NAME: " << engine->name() << " URL: " << url;
-    KAction *a = new KAction(Application::iconManager()->iconForUrl(url), engine->name(), this);
+    KAction *a = new KAction(rApp->iconManager()->iconForUrl(url), engine->name(), this);
     a->setCheckable(true);
     if (engine->desktopEntryName() == selectedEngine->desktopEntryName()) a->setChecked(true);
     a->setData(engine->entryPath());
@@ -429,8 +541,8 @@ SuggestionListItem::SuggestionListItem(const UrlSearchItem &item, const QString
     QHBoxLayout *hLayout = new QHBoxLayout;
     hLayout->setSpacing(4);
 
-    hLayout->addWidget(new IconLabel(SearchEngine::buildQuery(UrlResolver::searchEngine(), ""), this));
-    hLayout->addWidget(new TextLabel(item.title, SearchEngine::extractQuery(text), this));
+    hLayout->addWidget(new IconLabel(item.url, this));
+    hLayout->addWidget(new TextLabel(item.title, text, this));
     hLayout->addWidget(new TypeIconLabel(item.type, this));
 
     setLayout(hLayout);
@@ -442,6 +554,53 @@ QString SuggestionListItem::text()
     return m_text;
 }
 
+
+// ---------------------------------------------------------------
+
+
+VisualSuggestionListItem::VisualSuggestionListItem(const UrlSearchItem &item, const QString &text, QWidget *parent)
+        : ListItem(item, parent)
+        , m_text(item.title)
+{
+
+    QHBoxLayout *hLayout = new QHBoxLayout;
+    hLayout->setSpacing(4);
+    QLabel *previewLabelIcon = new QLabel(this);
+
+    if (!item.image.isEmpty())
+    {
+        previewLabelIcon->setFixedSize(item.image_width + 10, item.image_height + 10);
+        new ImageLabel(item.image, item.image_width, item.image_height, previewLabelIcon);
+        IconLabel* icon = new IconLabel(item.url, previewLabelIcon);
+        icon->move(item.image_width - 10,  item.image_height - 10);
+    }
+    else
+    {
+        previewLabelIcon->setFixedSize(18, 18);
+        new IconLabel(item.url, previewLabelIcon);
+    }
+
+    hLayout->addWidget(previewLabelIcon);
+    QVBoxLayout *vLayout = new QVBoxLayout;
+    vLayout->setMargin(0);
+    vLayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::MinimumExpanding));
+    vLayout->addWidget(new TextLabel(item.title, text, this));
+    DescriptionLabel *d = new DescriptionLabel("", this);
+    vLayout->addWidget(d);
+    vLayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::MinimumExpanding));
+    hLayout->addLayout(vLayout);
+    hLayout->addWidget(new TypeIconLabel(item.type, this));
+    setLayout(hLayout);
+    d->setText("<i>" + item.description + "</i>");
+}
+
+
+QString VisualSuggestionListItem::text()
+{
+    return m_text;
+}
+
+
 // ---------------------------------------------------------------
 
 
@@ -468,31 +627,43 @@ BrowseListItem::BrowseListItem(const UrlSearchItem &item, const QString &text, Q
 
 ListItem *ListItemFactory::create(const UrlSearchItem &item, const QString &text, QWidget *parent)
 {
-    ListItem *newItem;
+    if (item.type & UrlSearchItem::Search)
+    {
+        kDebug() << "Search";
+        return new SearchListItem(item, text, parent);
+    }
 
     if (item.type & UrlSearchItem::Browse)
     {
-        newItem = new BrowseListItem(item, text, parent);
+        kDebug() << "Browse";
+        return new BrowseListItem(item, text, parent);
     }
-    else
+
+    if (item.type & UrlSearchItem::History)
     {
-        if (item.type & UrlSearchItem::Search)
-        {
-            newItem = new SearchListItem(item, text, parent);
-        }
-        else
-        {
-
-            if (item.type & UrlSearchItem::Suggestion)
-            {
-                newItem = new SuggestionListItem(item, text, parent);
-            }
-            else
-            {
-                newItem = new PreviewListItem(item, text, parent);
-            }
-        }
+        kDebug() << "History";
+        return new PreviewListItem(item, text, parent);
     }
 
-    return newItem;
+    if (item.type & UrlSearchItem::Bookmark)
+    {
+        kDebug() << "Bookmark";
+        return new PreviewListItem(item, text, parent);
+    }
+
+    if (item.type & UrlSearchItem::Suggestion)
+    {
+        kDebug() << "ITEM URL: " << item.url;
+        if (item.description.isEmpty())
+        {
+            kDebug() << "Suggestion";
+            return new SuggestionListItem(item, text, parent);
+        }
+
+        kDebug() << "Visual Suggestion";
+        return new VisualSuggestionListItem(item, text, parent);
+    }
+
+    kDebug() << "Undefined";
+    return new PreviewListItem(item, text, parent);
 }
