@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2010 by Matthieu Gicquel <matgic78 at gmail dot com>
+* Copyright (C) 2010-2011 by Matthieu Gicquel <matgic78 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -74,7 +74,7 @@ RSSWidget::RSSWidget(const QMap< KUrl, QString > &map, QWidget *parent)
     m_agregators->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
 
     m_agregators->addItem(KIcon("akregator"), QString("Akregator"));
-    m_agregators->addItem(Application::iconManager()->iconForUrl(KUrl("http://google.com/reader")), i18n("Google Reader"));
+    m_agregators->addItem(rApp->iconManager()->iconForUrl(KUrl("http://google.com/reader")), i18n("Google Reader"));
 
     layout->addRow(agregator, m_agregators);
 
@@ -107,16 +107,11 @@ RSSWidget::RSSWidget(const QMap< KUrl, QString > &map, QWidget *parent)
 }
 
 
-RSSWidget::~RSSWidget()
-{
-}
-
-
 void RSSWidget::showAt(const QPoint &pos)
 {
     adjustSize();
 
-    QPoint p(pos.x()-width(), pos.y()+10);
+    QPoint p(pos.x() - width(), pos.y() + 10);
     move(p);
     show();
 }
@@ -138,7 +133,7 @@ void RSSWidget::accept()
 void RSSWidget::addWithGoogleReader(const QString &url)
 {
     KUrl toLoad = KUrl("http://www.google.com/ig/add?feedurl=" + url);
-    Application::instance()->mainWindow()->currentTab()->view()->load(toLoad);
+    rApp->mainWindow()->currentTab()->view()->load(toLoad);
 }
 
 
