@@ -2,10 +2,10 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2009 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2009 by Domrachev Alexandr <alexandr.domrachev@gmail.com>
 * Copyright (C) 2009 by Paweł Prażak <pawelprazak at gmail dot com>
-* Copyright (C) 2009 by Lionel Chauvin <megabigbug@yahoo.fr>
+* Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -49,6 +49,7 @@
 // KDE Includes
 #include <KCompletionBox>
 #include <KStandardDirs>
+#include <KColorScheme>
 
 // Qt Includes
 #include <QtGui/QPainter>
@@ -79,7 +80,6 @@ void IconButton::mouseReleaseEvent(QMouseEvent* event)
 UrlBar::UrlBar(QWidget *parent)
         : KLineEdit(parent)
         , _tab(0)
-        , _privateMode(false)
         , _icon(new IconButton(this))
         , _suggestionTimer(new QTimer(this))
 {
@@ -89,6 +89,9 @@ UrlBar::UrlBar(QWidget *parent)
     // doesn't show the clear button
     setClearButtonShown(false);
 
+    // enable dragging
+    setDragEnabled(true);
+
     // insert decoded URLs
     setUrlDropsEnabled(true);
 
@@ -110,8 +113,8 @@ UrlBar::UrlBar(QWidget *parent)
     connect(_tab->view(), SIGNAL(loadStarted()), this, SLOT(clearRightIcons()));
     connect(_tab->view(), SIGNAL(iconChanged()), this, SLOT(refreshFavicon()));
 
-    // bookmark icon
-    connect(Application::bookmarkProvider()->bookmarkManager(), SIGNAL(changed(const QString &, const QString &)), this, SLOT(onBookmarksChanged()));
+    // search icon
+    connect(rApp->opensearchManager(), SIGNAL(openSearchEngineAdded(const QString &, const QString &, const QString &)), this, SLOT(updateRightIcons()));
 
     _suggestionTimer->setSingleShot(true);
     connect(_suggestionTimer, SIGNAL(timeout()), this, SLOT(suggest()));
@@ -122,8 +125,8 @@ UrlBar::UrlBar(QWidget *parent)
 
 UrlBar::~UrlBar()
 {
+    _suggestionTimer->stop();
     activateSuggestions(false);
-    delete _icon;
     _box.clear();
 }
 
@@ -148,27 +151,27 @@ void UrlBar::setQUrl(const QUrl& url)
 void UrlBar::activated(const KUrl& url, Rekonq::OpenType type)
 {
     activateSuggestions(false);
-
     clearFocus();
     setUrl(url);
-    Application::instance()->loadUrl(url, type);
+    rApp->loadUrl(url, type);
 }
 
 
 void UrlBar::paintEvent(QPaintEvent *event)
 {
+    KColorScheme colorScheme(palette().currentColorGroup());
     QColor backgroundColor;
     QColor foregroundColor;
 
-    if (_privateMode)
+    if (QWebSettings::globalSettings()->testAttribute(QWebSettings::PrivateBrowsingEnabled))
     {
         backgroundColor = QColor(220, 220, 220);  // light gray
         foregroundColor = Qt::black;
     }
     else
     {
-        backgroundColor = Application::palette().color(QPalette::Base);
-        foregroundColor = Application::palette().color(QPalette::Text);
+        backgroundColor = rApp->palette().color(QPalette::Base);
+        foregroundColor = rApp->palette().color(QPalette::Text);
     }
 
     // set background color of UrlBar
@@ -179,31 +182,31 @@ void UrlBar::paintEvent(QPaintEvent *event)
     {
         if (_tab->url().scheme() == QL1S("https"))
         {
-            backgroundColor = QColor(255, 255, 171);  // light yellow
-            foregroundColor = Qt::black;
+            backgroundColor = colorScheme.background(KColorScheme::NeutralBackground).color();  // light yellow
+            foregroundColor = colorScheme.foreground(KColorScheme::NormalText).color();
         }
         p.setBrush(QPalette::Base, backgroundColor);
         p.setBrush(QPalette::Text, foregroundColor);
     }
     else
     {
-        QColor highlight = Application::palette().color(QPalette::Highlight);
+        QColor highlight = rApp->palette().color(QPalette::Highlight);
 
-        int r = (highlight.red()+2*backgroundColor.red())/3;
-        int g = (highlight.green()+2*backgroundColor.green())/3;
-        int b = (highlight.blue()+2*backgroundColor.blue())/3;
+        int r = (highlight.red() + 2 * backgroundColor.red()) / 3;
+        int g = (highlight.green() + 2 * backgroundColor.green()) / 3;
+        int b = (highlight.blue() + 2 * backgroundColor.blue()) / 3;
 
         QColor loadingColor(r, g, b);
 
         if (abs(loadingColor.lightness() - backgroundColor.lightness()) < 20) //eg. Gaia color scheme
         {
-            r = (2*highlight.red()+backgroundColor.red())/3;
-            g = (2*highlight.green()+backgroundColor.green())/3;
-            b = (2*highlight.blue()+backgroundColor.blue())/3;
+            r = (2 * highlight.red() + backgroundColor.red()) / 3;
+            g = (2 * highlight.green() + backgroundColor.green()) / 3;
+            b = (2 * highlight.blue() + backgroundColor.blue()) / 3;
             loadingColor = QColor(r, g, b);
         }
 
-        QLinearGradient gradient( QPoint(0, 0), QPoint(width(), 0) );
+        QLinearGradient gradient(QPoint(0, 0), QPoint(width(), 0));
         gradient.setColorAt(0, loadingColor);
         gradient.setColorAt(((double)progr) / 100 - .000001, loadingColor);
         gradient.setColorAt(((double)progr) / 100, backgroundColor);
@@ -214,7 +217,7 @@ void UrlBar::paintEvent(QPaintEvent *event)
     // you need this before our code to draw inside the line edit..
     KLineEdit::paintEvent(event);
 
-    if( text().isEmpty() && progr == 0 )
+    if (text().isEmpty() && progr == 0)
     {
         QStyleOptionFrame option;
         initStyleOption(&option);
@@ -233,6 +236,13 @@ void UrlBar::keyPressEvent(QKeyEvent *event)
 {
     // this handles the Modifiers + Return key combinations
     QString currentText = text().trimmed();
+    if (event->modifiers() == Qt::AltModifier)
+    {
+        if (event->key() == Qt::Key_Return || event->key() == Qt::Key_Enter)
+        {
+            activated(currentText, Rekonq::NewFocusedTab);
+        }
+    }
     if ((event->key() == Qt::Key_Enter || event->key() == Qt::Key_Return)
             && !currentText.startsWith(QL1S("http://"), Qt::CaseInsensitive)
             && event->modifiers() != Qt::NoModifier)
@@ -267,7 +277,7 @@ void UrlBar::keyPressEvent(QKeyEvent *event)
     }
 
     else if ((event->key() == Qt::Key_Enter || event->key() == Qt::Key_Return)
-                && !currentText.isEmpty())
+             && !currentText.isEmpty())
     {
         loadTyped(currentText);
     }
@@ -290,12 +300,6 @@ void UrlBar::focusInEvent(QFocusEvent *event)
 }
 
 
-void UrlBar::setPrivateMode(bool on)
-{
-    _privateMode = on;
-}
-
-
 void UrlBar::dropEvent(QDropEvent *event)
 {
     KLineEdit::dropEvent(event);
@@ -305,9 +309,6 @@ void UrlBar::dropEvent(QDropEvent *event)
 
 void UrlBar::loadFinished()
 {
-    if (_tab->progress() != 0)
-        return;
-
     if (_tab->url().scheme() == QL1S("about"))
     {
         update();
@@ -339,6 +340,13 @@ void UrlBar::loadFinished()
         connect(bt, SIGNAL(clicked(QPoint)), _tab->page(), SLOT(showSSLInfo(QPoint)));
     }
 
+    // show add search engine
+    if (_tab->hasNewSearchEngine())
+    {
+        IconButton *bt = addRightIcon(UrlBar::SearchEngine);
+        connect(bt, SIGNAL(clicked(QPoint)), _tab, SLOT(showSearchEngine(QPoint)));
+    }
+
     // we need to update urlbar after the right icon settings
     // removing this code (where setStyleSheet automatically calls update) needs adding again
     // an update call
@@ -350,10 +358,10 @@ void UrlBar::loadFinished()
 
 void UrlBar::showBookmarkInfo(const QPoint &pos)
 {
-    if( _tab->url().scheme() == QL1S("about") )
+    if (_tab->url().scheme() == QL1S("about"))
         return;
 
-    KBookmark bookmark = Application::bookmarkProvider()->bookmarkForUrl(_tab->url());
+    KBookmark bookmark = rApp->bookmarkProvider()->bookmarkForUrl(_tab->url());
 
     IconButton *bt = qobject_cast<IconButton *>(this->sender());
     if (!bt)
@@ -361,26 +369,34 @@ void UrlBar::showBookmarkInfo(const QPoint &pos)
 
     if (bookmark.isNull())
     {
-        bookmark = Application::bookmarkProvider()->bookmarkOwner()->bookmarkCurrentPage();
+        bookmark = rApp->bookmarkProvider()->bookmarkOwner()->bookmarkCurrentPage();
+
+        // set bk icon
+        bt->setIcon(KIcon("bookmarks"));
+        bt->setToolTip(i18n("Edit this bookmark"));
     }
     else
     {
         BookmarkWidget *widget = new BookmarkWidget(bookmark, window());
+        connect(widget, SIGNAL(updateIcon()), this, SLOT(updateRightIcons()));
         widget->showAt(pos);
     }
 }
 
 
-void UrlBar::onBookmarksChanged()
+void UrlBar::updateRightIcons()
 {
-    clearRightIcons();
-    loadFinished();
+    if (!_tab->isPageLoading())
+    {
+        clearRightIcons();
+        loadFinished();
+    }
 }
 
 
 void UrlBar::loadTyped(const QString &text)
 {
-    activated( KUrl(text) );
+    activated(KUrl(text));
 }
 
 
@@ -432,9 +448,9 @@ IconButton *UrlBar::addRightIcon(UrlBar::icon ic)
         rightIcon->setToolTip(i18n("Show SSL Info"));
         break;
     case UrlBar::BK:
-        if (Application::bookmarkProvider()->bookmarkForUrl(_tab->url()).isNull())
+        if (rApp->bookmarkProvider()->bookmarkForUrl(_tab->url()).isNull())
         {
-            rightIcon->setIcon(KIcon("bookmarks").pixmap(32,32, QIcon::Disabled));
+            rightIcon->setIcon(KIcon("bookmarks").pixmap(32, 32, QIcon::Disabled));
             rightIcon->setToolTip(i18n("Bookmark this page"));
         }
         else
@@ -443,6 +459,17 @@ IconButton *UrlBar::addRightIcon(UrlBar::icon ic)
             rightIcon->setToolTip(i18n("Edit this bookmark"));
         }
         break;
+    case UrlBar::SearchEngine:
+    {
+        KIcon wsIcon("edit-web-search");
+        if (wsIcon.isNull())
+        {
+            wsIcon = KIcon("preferences-web-browser-shortcuts");
+        }
+        rightIcon->setIcon(wsIcon);
+        rightIcon->setToolTip(i18n("Add search engine"));
+        break;
+    }
     default:
         kDebug() << "ERROR.. default non extant case!!";
         break;
@@ -485,13 +512,13 @@ void UrlBar::resizeEvent(QResizeEvent *event)
 
 void UrlBar::detectTypedString(const QString &typed)
 {
-    if(typed.count() == 1)
+    if (typed.count() == 1)
     {
         QTimer::singleShot(0, this, SLOT(suggest()));
         return;
     }
 
-    if(_suggestionTimer->isActive())
+    if (_suggestionTimer->isActive())
         _suggestionTimer->stop();
     _suggestionTimer->start(50);
 }
@@ -499,17 +526,24 @@ void UrlBar::detectTypedString(const QString &typed)
 
 void UrlBar::suggest()
 {
-    if(!_box.isNull())
-        _box.data()->suggestUrls( text() );
+    if (!_box.isNull())
+        _box.data()->suggestUrls(text());
 }
 
 
 void UrlBar::refreshFavicon()
 {
+    if (QWebSettings::globalSettings()->testAttribute(QWebSettings::PrivateBrowsingEnabled))
+    {
+        _icon->setIcon(KIcon("view-media-artist"));
+        return;
+    }
+
     KUrl u = _tab->url();
-    if(u.scheme() == QL1S("about")) {
+    if (u.scheme() == QL1S("about"))
+    {
         _icon->setIcon(KIcon("arrow-right"));
         return;
     }
-    _icon->setIcon(Application::iconManager()->iconForUrl(u));
+    _icon->setIcon(rApp->iconManager()->iconForUrl(u));
 }
