@@ -2,10 +2,10 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2009 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2009 by Domrachev Alexandr <alexandr.domrachev@gmail.com>
 * Copyright (C) 2009 by Paweł Prażak <pawelprazak at gmail dot com>
-* Copyright (C) 2009 by Lionel Chauvin <megabigbug@yahoo.fr>
+* Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -42,7 +42,6 @@
 #include <QToolButton>
 
 // Forward Declarations
-class QLinearGradient;
 class QWidget;
 class CompletionWidget;
 class WebTab;
@@ -55,7 +54,7 @@ class IconButton : public QToolButton
 
 public:
     IconButton(QWidget *parent = 0);
-    
+
 signals:
     void clicked(QPoint);
 
@@ -80,16 +79,17 @@ public:
 
     enum icon
     {
-        KGet    = 0x00000001,
-        RSS     = 0x00000010,
-        SSL     = 0x00000100,
-        BK      = 0x00001000
+        KGet         = 0x00000001,
+        RSS          = 0x00000010,
+        SSL          = 0x00000100,
+        BK           = 0x00001000,
+        SearchEngine = 0x00010000
     };
 
     explicit UrlBar(QWidget *parent = 0);
     ~UrlBar();
 
-    void setPrivateMode(bool on);
+    void activateSuggestions(bool);
 
 public slots:
     void setQUrl(const QUrl &url);
@@ -101,15 +101,15 @@ private slots:
     void loadTyped(const QString &);
 
     void clearRightIcons();
-    
+    void updateRightIcons();
+
     void detectTypedString(const QString &);
     void suggest();
 
     void showBookmarkInfo(const QPoint &pos);
-    void onBookmarksChanged();
 
     void refreshFavicon();
-    
+
 protected:
     void paintEvent(QPaintEvent *event);
     void keyPressEvent(QKeyEvent *event);
@@ -120,11 +120,9 @@ protected:
 
 private:
     IconButton *addRightIcon(UrlBar::icon);
-    void activateSuggestions(bool);
 
     QWeakPointer<CompletionWidget> _box;
     WebTab *_tab;
-    bool _privateMode;
 
     IconButton *_icon;
     IconButtonPointerList _rightIconsList;
