@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2009 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2009-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -29,11 +29,9 @@
 #include "urlresolver.moc"
 
 // Local Includes
-#include "application.h"
 #include "historymanager.h"
 #include "bookmarkprovider.h"
 #include "searchengine.h"
-#include "opensearchmanager.h"
 
 // KDE Includes
 #include <KBookmark>
@@ -42,6 +40,7 @@
 #include <KService>
 #include <KConfig>
 #include <KConfigGroup>
+#include <KProtocolInfo>
 
 // Qt Includes
 #include <QByteArray>
@@ -70,42 +69,44 @@ QRegExp UrlResolver::_searchEnginesRegexp;
 UrlResolver::UrlResolver(const QString &typedUrl)
         : QObject()
         , _typedString(typedUrl.trimmed())
+        , _typedQuery()
 {
-    if (!_searchEngine ) _searchEngine = SearchEngine::defaultEngine();
+    if (!_searchEngine)
+        setSearchEngine(SearchEngine::defaultEngine());
 
-    if ( _browseRegexp.isEmpty() )
+    if (_browseRegexp.isEmpty())
     {
         kDebug() << "browse regexp empty. Setting value..";
 
-        QString protocol = "^(http://|https://|file://|ftp://|man:|info:|apt:|about:)";
+        QString protocol = QString("^(%1)").arg(KProtocolInfo::protocols().join("|"));
 
         QString localhost = "^localhost";
 
         QString local = "^/";
 
         QString ipv4 = "^0*([1-9]?\\d|1\\d\\d|2[0-4]\\d|25[0-5])\\.0*([1-9]?\\d|1\\d\\d|2[0-4]\\d|25[0-5])"\
-        "\\.0*([1-9]?\\d|1\\d\\d|2[0-4]\\d|25[0-5])\\.0*([1-9]?\\d|1\\d\\d|2[0-4]\\d|25[0-5])";
+                       "\\.0*([1-9]?\\d|1\\d\\d|2[0-4]\\d|25[0-5])\\.0*([1-9]?\\d|1\\d\\d|2[0-4]\\d|25[0-5])";
 
         QString ipv6 = "^([0-9a-fA-F]{4}|0)(\\:([0-9a-fA-F]{4}|0)){7}";
 
         QString address = "[\\d\\w-.]+\\.(a[cdefgilmnoqrstuwz]|b[abdefghijmnorstvwyz]|"\
-        "c[acdfghiklmnoruvxyz]|d[ejkmnoz]|e[ceghrstu]|f[ijkmnor]|g[abdefghilmnpqrstuwy]|"\
-        "h[kmnrtu]|i[delmnoqrst]|j[emop]|k[eghimnprwyz]|l[abcikrstuvy]|"\
-        "m[acdghklmnopqrstuvwxyz]|n[acefgilopruz]|om|p[aefghklmnrstwy]|qa|r[eouw]|"\
-        "s[abcdeghijklmnortuvyz]|t[cdfghjkmnoprtvwz]|u[augkmsyz]|v[aceginu]|w[fs]|"\
-        "y[etu]|z[amw]|aero|arpa|biz|com|coop|edu|info|int|gov|local|mil|museum|name|net|org|"\
-        "pro)";
+                          "c[acdfghiklmnoruvxyz]|d[ejkmnoz]|e[ceghrstu]|f[ijkmnor]|g[abdefghilmnpqrstuwy]|"\
+                          "h[kmnrtu]|i[delmnoqrst]|j[emop]|k[eghimnprwyz]|l[abcikrstuvy]|"\
+                          "m[acdghklmnopqrstuvwxyz]|n[acefgilopruz]|om|p[aefghklmnrstwy]|qa|r[eouw]|"\
+                          "s[abcdeghijklmnortuvyz]|t[cdfghjkmnoprtvwz]|u[augkmsyz]|v[aceginu]|w[fs]|"\
+                          "y[etu]|z[amw]|aero|arpa|biz|com|coop|edu|info|int|gov|local|mil|museum|name|net|org|"\
+                          "pro)";
 
-        _browseRegexp = QRegExp('(' + protocol + ")|(" + localhost + ")|(" + local + ")|(" + address + ")|(" + ipv6 + ")|(" + ipv4 +')');
+        _browseRegexp = QRegExp('(' + protocol + ")|(" + localhost + ")|(" + local + ")|(" + address + ")|(" + ipv6 + ")|(" + ipv4 + ')');
     }
 
-    if ( _searchEnginesRegexp.isEmpty() )
+    if (_searchEnginesRegexp.isEmpty())
     {
         QString reg;
         QString engineUrl;
         Q_FOREACH(KService::Ptr s, SearchEngine::favorites())
         {
-            engineUrl = QRegExp::escape(s->property("Query").toString()).replace("\\\\\\{@\\}","[\\d\\w-.]+");
+            engineUrl = QRegExp::escape(s->property("Query").toString()).replace("\\\\\\{@\\}", "[\\d\\w-.]+");
             if (reg.isEmpty())
                 reg = '(' + engineUrl + ')';
             else
@@ -118,20 +119,20 @@ UrlResolver::UrlResolver(const QString &typedUrl)
 
 UrlSearchList UrlResolver::orderedSearchItems()
 {
-    if( _typedString.startsWith( QL1S("about:") ) )
+    if (_typedString.startsWith(QL1S("about:")))
     {
         UrlSearchList list;
-        UrlSearchItem home(UrlSearchItem::Browse, QString("about:home"),       QL1S("home") );
+        UrlSearchItem home(UrlSearchItem::Browse, QString("about:home"),       QL1S("home"));
         list << home;
-        UrlSearchItem favs(UrlSearchItem::Browse, QString("about:favorites"),  QL1S("favorites") );
+        UrlSearchItem favs(UrlSearchItem::Browse, QString("about:favorites"),  QL1S("favorites"));
         list << favs;
-        UrlSearchItem clos(UrlSearchItem::Browse, QString("about:closedTabs"), QL1S("closed tabs") );
+        UrlSearchItem clos(UrlSearchItem::Browse, QString("about:closedTabs"), QL1S("closed tabs"));
         list << clos;
-        UrlSearchItem book(UrlSearchItem::Browse, QString("about:bookmarks"),  QL1S("bookmarks") );
+        UrlSearchItem book(UrlSearchItem::Browse, QString("about:bookmarks"),  QL1S("bookmarks"));
         list << book;
-        UrlSearchItem hist(UrlSearchItem::Browse, QString("about:history"),    QL1S("history") );
+        UrlSearchItem hist(UrlSearchItem::Browse, QString("about:history"),    QL1S("history"));
         list << hist;
-        UrlSearchItem down(UrlSearchItem::Browse, QString("about:downloads"),  QL1S("downloads") );
+        UrlSearchItem down(UrlSearchItem::Browse, QString("about:downloads"),  QL1S("downloads"));
         list << down;
 
         return list;
@@ -161,7 +162,7 @@ UrlSearchList UrlResolver::orderLists()
 
     UrlSearchList list;
 
-    if(_browseRegexp.indexIn(_typedString) != -1)
+    if (_browseRegexp.indexIn(_typedString) != -1)
     {
         list << _qurlFromUserInput;
         list << _webSearches;
@@ -181,7 +182,7 @@ UrlSearchList UrlResolver::orderLists()
     {
         privileged = privilegedItem(&_bookmarks);
     }
-    else if(privileged.type == UrlSearchItem::History && _bookmarks.removeOne(privileged))
+    else if (privileged.type == UrlSearchItem::History && _bookmarks.removeOne(privileged))
     {
         privileged.type |= UrlSearchItem::Bookmark;
     }
@@ -197,13 +198,13 @@ UrlSearchList UrlResolver::orderLists()
     UrlSearchList common;
     int commonCount = 0;
 
-    //prefer items which are history items als well bookmarks item
-    //if there are more than 1000 bookmark results, the performance impact is noticeable
-    if(bookmarksCount < 1000)
+    // prefer items which are history items as well bookmarks item
+    // if there are more than 1000 bookmark results, the performance impact is noticeable
+    if (bookmarksCount < 1000)
     {
         //add as many items to the common list as there are available entries in the dropdown list
         UrlSearchItem urlSearchItem;
-        for(int i = 0; i < _history.count(); i++)
+        for (int i = 0; i < _history.count(); i++)
         {
             if (_bookmarks.removeOne(_history.at(i)))
             {
@@ -211,7 +212,7 @@ UrlSearchList UrlResolver::orderLists()
                 urlSearchItem.type |= UrlSearchItem::Bookmark;
                 common << urlSearchItem;
                 commonCount++;
-                if(commonCount >= availableEntries)
+                if (commonCount >= availableEntries)
                 {
                     break;
                 }
@@ -219,7 +220,7 @@ UrlSearchList UrlResolver::orderLists()
         }
 
         commonCount = common.count();
-        if(commonCount >= availableEntries)
+        if (commonCount >= availableEntries)
         {
             common = common.mid(0, availableEntries);
             _history = UrlSearchList();
@@ -229,11 +230,11 @@ UrlSearchList UrlResolver::orderLists()
         else        //remove all items from the history and bookmarks list up to the remaining entries in the dropdown list
         {
             availableEntries -= commonCount;
-            if(historyCount >= availableEntries)
+            if (historyCount >= availableEntries)
             {
                 _history = _history.mid(0, availableEntries);
             }
-            if(bookmarksCount >= availableEntries)
+            if (bookmarksCount >= availableEntries)
             {
                 _bookmarks = _bookmarks.mid(0, availableEntries);
             }
@@ -242,17 +243,17 @@ UrlSearchList UrlResolver::orderLists()
     else        //if there are too many bookmarks items, remove all items up to the remaining entries in the dropdown list
     {
 
-        if(historyCount >= availableEntries)
+        if (historyCount >= availableEntries)
         {
             _history = _history.mid(0, availableEntries);
         }
-        if(bookmarksCount >= availableEntries)
+        if (bookmarksCount >= availableEntries)
         {
             _bookmarks = _bookmarks.mid(0, availableEntries);
         }
 
         UrlSearchItem urlSearchItem;
-        for(int i = 0; i < _history.count(); i++)
+        for (int i = 0; i < _history.count(); i++)
         {
             if (_bookmarks.removeOne(_history.at(i)))
             {
@@ -272,9 +273,9 @@ UrlSearchList UrlResolver::orderLists()
     kDebug() << "HISTORY COUNT: " << historyCount;
 
     //now fill the list to MAX_ELEMENTS
-    if(availableEntries > 0)
+    if (availableEntries > 0)
     {
-        int historyEntries = ((int) (availableEntries / 2)) + availableEntries % 2;
+        int historyEntries = ((int)(availableEntries / 2)) + availableEntries % 2;
         int bookmarksEntries = availableEntries - historyEntries;
 
         if (historyCount >= historyEntries && bookmarksCount >= bookmarksEntries)
@@ -284,14 +285,14 @@ UrlSearchList UrlResolver::orderLists()
         }
         else if (historyCount < historyEntries && bookmarksCount >= bookmarksEntries)
         {
-            if(historyCount + bookmarksCount > availableEntries)
+            if (historyCount + bookmarksCount > availableEntries)
             {
                 _bookmarks = _bookmarks.mid(0, availableEntries - historyCount);
             }
         }
         else if (historyCount >= historyEntries && bookmarksCount < bookmarksEntries)
         {
-            if(historyCount + bookmarksCount > availableEntries)
+            if (historyCount + bookmarksCount > availableEntries)
             {
                 _history = _history.mid(0, availableEntries - bookmarksCount);
             }
@@ -317,7 +318,7 @@ UrlSearchList UrlResolver::orderLists()
 // PRIVATE ENGINES
 
 
-//QUrl from User Input (easily the best solution... )
+// QUrl from User Input (easily the best solution... )
 void UrlResolver::computeQurlFromUserInput()
 {
     QString url = _typedString;
@@ -331,7 +332,7 @@ void UrlResolver::computeQurlFromUserInput()
 }
 
 
-//webSearches
+// webSearches
 void UrlResolver::computeWebSearches()
 {
     QString query = _typedString;
@@ -339,17 +340,23 @@ void UrlResolver::computeWebSearches()
     if (engine)
     {
         query = query.remove(0, _typedString.indexOf(SearchEngine::delimiter()) + 1);
-        _searchEngine = engine;
+        setSearchEngine(engine);
     }
 
-    _webSearches = (UrlSearchList() << UrlSearchItem(UrlSearchItem::Search, SearchEngine::buildQuery(_searchEngine, query), _searchEngine->name()));
+    if (_searchEngine)
+    {
+        UrlSearchItem item = UrlSearchItem(UrlSearchItem::Search, SearchEngine::buildQuery(_searchEngine, query), query);
+        UrlSearchList list;
+        list << item;
+        _webSearches = list;
+    }
 }
 
 
-//history
+// history
 void UrlResolver::computeHistory()
 {
-    QList<HistoryItem> found = Application::historyManager()->find(_typedString);
+    QList<HistoryItem> found = rApp->historyManager()->find(_typedString);
     qSort(found);
 
     Q_FOREACH(const HistoryItem &i, found)
@@ -366,7 +373,7 @@ void UrlResolver::computeHistory()
 // bookmarks
 void UrlResolver::computeBookmarks()
 {
-    QList<KBookmark> found = Application::bookmarkProvider()->find(_typedString);
+    QList<KBookmark> found = rApp->bookmarkProvider()->find(_typedString);
     kDebug() << "FOUND: " << found.count();
     Q_FOREACH(const KBookmark &b, found)
     {
@@ -376,40 +383,52 @@ void UrlResolver::computeBookmarks()
 }
 
 
-//opensearch suggestion
+// opensearch suggestion
 void UrlResolver::computeSuggestions()
 {
     // if a string startsWith /, it is probably a local path
     // so, no need for suggestions...
-    if(_typedString.startsWith('/'))
+    if (_typedString.startsWith('/') || !rApp->opensearchManager()->isSuggestionAvailable())
     {
         UrlSearchList list;
         emit suggestionsReady(list, _typedString);
         return;
     }
 
-    if (Application::opensearchManager()->isSuggestionAvailable())
+    QString query = _typedString;
+    KService::Ptr engine = SearchEngine::fromString(_typedString);
+    if (engine)
     {
-        connect(Application::opensearchManager(),
-                SIGNAL(suggestionReceived(const QString &, const QStringList &)),
-                this,
-                SLOT(suggestionsReceived(const QString &, const QStringList &)));
-
-        Application::opensearchManager()->requestSuggestion(_typedString);
+        query = query.remove(0, _typedString.indexOf(SearchEngine::delimiter()) + 1);
+        setSearchEngine(engine);
     }
+
+    connect(rApp->opensearchManager(),
+            SIGNAL(suggestionsReceived(const QString &, const ResponseList &)),
+            this,
+            SLOT(suggestionsReceived(const QString &, const ResponseList &)));
+
+    _typedQuery = query;
+    rApp->opensearchManager()->requestSuggestion(query);
 }
 
 
-void UrlResolver::suggestionsReceived(const QString &text, const QStringList &suggestions)
+void UrlResolver::suggestionsReceived(const QString &text, const ResponseList &suggestions)
 {
-    if(text != _typedString)
+    if (text != _typedQuery)
         return;
 
     UrlSearchList sugList;
-
-    Q_FOREACH(const QString &s, suggestions)
+    QString urlString;
+    Q_FOREACH(const Response &i, suggestions)
     {
-        UrlSearchItem gItem(UrlSearchItem::Suggestion, s, s);
+        urlString = i.url;
+        if (urlString.isEmpty())
+        {
+            urlString = SearchEngine::buildQuery(UrlResolver::searchEngine(), i.title);
+        }
+
+        UrlSearchItem gItem(UrlSearchItem::Suggestion, urlString, i.title, i.description, i.image, i.image_width, i.image_height);
         sugList << gItem;
     }
     emit suggestionsReady(sugList, _typedString);
@@ -424,7 +443,7 @@ UrlSearchItem UrlResolver::privilegedItem(UrlSearchList* list)
     QString test1 = QString(QL1C('/')) + _typedString + dot;
     QString test2 = dot + _typedString + dot;
 
-    for(int i = 0; i<list->count(); i++)
+    for (int i = 0; i < list->count(); i++)
     {
         item = list->at(i);
         //TODO: move this to AwesomeUrlCompletion::substringCompletion and add a priviledged flag to UrlSearchItem
