@@ -2,7 +2,7 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2009 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2009-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -31,6 +31,10 @@
 // Rekonq Includes
 #include "rekonq_defines.h"
 
+// Locale Includes
+#include "application.h"
+#include "opensearchmanager.h"
+
 // KDE Includes
 #include <KUrl>
 #include <KService>
@@ -38,7 +42,7 @@
 // Qt Includes
 #include <QString>
 #include <QList>
-#include <QStringList>
+#include "suggestionparser.h"
 
 
 class UrlSearchItem
@@ -52,42 +56,57 @@ public:
         Browse          = 0x00000010,
         History         = 0x00000100,
         Bookmark        = 0x00001000,
-        Suggestion      = 0x00010000,
+        Suggestion      = 0x00010000
     };
 
     int type;
     QString url;
     QString title;
     QString description;
+    QString image;
+    int image_width;
+    int image_height;
     QString bookmarkPath;
-    
+
     UrlSearchItem(const UrlSearchItem &item) : type(item.type),
-                                               url(item.url),
-                                               title(item.title),
-                                               description(item.description)
+            url(item.url),
+            title(item.title),
+            description(item.description),
+            image(item.image),
+            image_width(item.image_width),
+            image_height(item.image_height)
     {};
 
     UrlSearchItem() : type(UrlSearchItem::Undefined),
-                      url(QString()),
-                      title(QString()),
-                      description(QString())
+            url(QString()),
+            title(QString()),
+            description(QString()),
+            image(QString()),
+            image_width(0),
+            image_height(0)
     {};
-    
+
     UrlSearchItem(const int &_type,
                   const QString &_url,
                   const QString &_title = QString(),
-                  const QString   &description    = QString()
-                  )
-                  : type(_type),
-                  url(_url),
-                  title(_title),
-                  description(description)
+                  const QString &_description = QString(),
+                  const QString &_image = QString(),
+                  const int &_image_width = 0,
+                  const int &_image_height = 0
+                 )
+            : type(_type),
+            url(_url),
+            title(_title),
+            description(_description),
+            image(_image),
+            image_width(_image_width),
+            image_height(_image_height)
     {};
 
     inline bool operator==(const UrlSearchItem &i) const
     {
         return i.url == url;//TODO && i.title == title;
-    }    
+    }
 };
 
 typedef QList <UrlSearchItem> UrlSearchList;
@@ -113,12 +132,14 @@ public:
     static void setSearchEngine(KService::Ptr engine)
     {
         _searchEngine = engine;
+        if (engine)
+            rApp->opensearchManager()->setSearchProvider(engine->desktopEntryName());
     };
 
     void computeSuggestions();
 
 private Q_SLOTS:
-    void suggestionsReceived(const QString &text, const QStringList &suggestions);    
+    void suggestionsReceived(const QString &text, const ResponseList &suggestions);
 
 Q_SIGNALS:
     void suggestionsReady(const UrlSearchList &, const QString &);
@@ -132,19 +153,19 @@ private:
     UrlSearchItem privilegedItem(UrlSearchList* list);
     UrlSearchList orderLists();
 
-    
     QString _typedString;
+    QString _typedQuery;
 
     UrlSearchList _webSearches;
     UrlSearchList _qurlFromUserInput;
     UrlSearchList _history;
     UrlSearchList _bookmarks;
     UrlSearchList _suggestions;
-    
+
     static QRegExp _browseRegexp;
     static QRegExp _searchEnginesRegexp;
 
-    static KService::Ptr _searchEngine;    
+    static KService::Ptr _searchEngine;
 };
 
 // ------------------------------------------------------------------------------
