@@ -6,7 +6,7 @@
 * Copyright (C) 2008 Dirk Mueller <mueller@kde.org>
 * Copyright (C) 2008 Urs Wolfer <uwolfer @ kde.org>
 * Copyright (C) 2008 Michael Howell <mhowell123@gmail.com>
-* Copyright (C) 2008-2010 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2010 by Matthieu Gicquel <matgic78 at gmail dot com>
 * Copyright (C) 2009-2010 Dawit Alemayehu <adawit at kde dot org>
 *
@@ -54,6 +54,9 @@
 // KDE Includes
 #include <KIO/Job>
 #include <KIO/RenameDialog>
+#include <KIO/JobUiDelegate>
+
+#include <KTemporaryFile>
 #include <KStandardDirs>
 #include <KFileDialog>
 #include <KJobUiDelegate>
@@ -63,11 +66,18 @@
 #include <KService>
 #include <KToolInvocation>
 #include <KWebWallet>
+#include <KProtocolInfo>
+#include <KRun>
 
 #include <kparts/browseropenorsavequestion.h>
 
+#include <kio/scheduler.h>
+
 // Qt Includes
 #include <QtCore/QFileInfo>
+
+#include <QtGui/QTextDocument>
+
 #include <QtDBus/QDBusConnection>
 #include <QtDBus/QDBusConnectionInterface>
 #include <QtDBus/QDBusInterface>
@@ -96,43 +106,101 @@ static bool domainSchemeMatch(const QUrl& u1, const QUrl& u2)
 
 
 // NOTE
-// This is heavily based on the one from KdeWebKit and
-// extended to provide the extra functionality we need:
+// These 2 functions have been copied from the KWebPage class to implement a local version of the downloadResponse method.
+// In this way, we can easily provide the extra functionality we need:
 // 1. KGet Integration
 // 2. Save downloads history
-static bool downloadResource (const KUrl& srcUrl, const KIO::MetaData& metaData = KIO::MetaData(),
-                              QWidget* parent = 0, const QString& suggestedName = QString())
+static void extractSuggestedFileName(const QNetworkReply* reply, QString& fileName)
+{
+    fileName.clear();
+    const KIO::MetaData& metaData = reply->attribute(static_cast<QNetworkRequest::Attribute>(KIO::AccessManager::MetaData)).toMap();
+    if (metaData.value(QL1S("content-disposition-type")).compare(QL1S("attachment"), Qt::CaseInsensitive) == 0)
+        fileName = metaData.value(QL1S("content-disposition-filename"));
+
+    if (!fileName.isEmpty())
+        return;
+
+    if (!reply->hasRawHeader("Content-Disposition"))
+        return;
+
+    const QString value(QL1S(reply->rawHeader("Content-Disposition").simplified().constData()));
+    if (value.startsWith(QL1S("attachment"), Qt::CaseInsensitive) || value.startsWith(QL1S("inline"), Qt::CaseInsensitive))
+    {
+        const int length = value.size();
+        int pos = value.indexOf(QL1S("filename"), 0, Qt::CaseInsensitive);
+        if (pos > -1)
+        {
+            pos += 9;
+            while (pos < length && (value.at(pos) == QL1C(' ') || value.at(pos) == QL1C('=') || value.at(pos) == QL1C('"')))
+                pos++;
+
+            int endPos = pos;
+            while (endPos < length && value.at(endPos) != QL1C('"') && value.at(endPos) != QL1C(';'))
+                endPos++;
+
+            if (endPos > pos)
+                fileName = value.mid(pos, (endPos - pos)).trimmed();
+        }
+    }
+}
+
+
+static void extractMimeType(const QNetworkReply* reply, QString& mimeType)
+{
+    mimeType.clear();
+    const KIO::MetaData& metaData = reply->attribute(static_cast<QNetworkRequest::Attribute>(KIO::AccessManager::MetaData)).toMap();
+    if (metaData.contains(QL1S("content-type")))
+        mimeType = metaData.value(QL1S("content-type"));
+
+    if (!mimeType.isEmpty())
+        return;
+
+    if (!reply->hasRawHeader("Content-Type"))
+        return;
+
+    const QString value(QL1S(reply->rawHeader("Content-Type").simplified().constData()));
+    const int index = value.indexOf(QL1C(';'));
+    if (index == -1)
+        mimeType = value;
+    else
+        mimeType = value.left(index);
+}
+
+
+static bool downloadResource(const KUrl& srcUrl, const KIO::MetaData& metaData = KIO::MetaData(),
+                             QWidget * parent = 0, const QString & suggestedName = QString())
 {
     KUrl destUrl;
 
     int result = KIO::R_OVERWRITE;
-    const QUrl fileName ((suggestedName.isEmpty() ? srcUrl.fileName() : suggestedName));
+    const QString fileName((suggestedName.isEmpty() ? srcUrl.fileName() : suggestedName));
 
     do
     {
-        destUrl = KFileDialog::getSaveFileName(fileName, QString(), parent);
+        // follow bug:184202 fixes
+        destUrl = KFileDialog::getSaveFileName(KUrl(KGlobalSettings::downloadPath().append(fileName)), QString(), parent);
 
-        if(destUrl.isEmpty())
+        if (destUrl.isEmpty())
             return false;
 
         if (destUrl.isLocalFile())
         {
-            QFileInfo finfo (destUrl.toLocalFile());
+            QFileInfo finfo(destUrl.toLocalFile());
             if (finfo.exists())
             {
                 QDateTime now = QDateTime::currentDateTime();
-                QPointer<KIO::RenameDialog> dlg = new KIO::RenameDialog( parent,
-                                                                         i18n("Overwrite File?"),
-                                                                         srcUrl,
-                                                                         destUrl,
-                                                                         KIO::RenameDialog_Mode(KIO::M_OVERWRITE | KIO::M_SKIP),
-                                                                         -1,
-                                                                         finfo.size(),
-                                                                         now.toTime_t(),
-                                                                         finfo.created().toTime_t(),
-                                                                         now.toTime_t(),
-                                                                         finfo.lastModified().toTime_t()
-                                                                        );
+                QPointer<KIO::RenameDialog> dlg = new KIO::RenameDialog(parent,
+                        i18n("Overwrite File?"),
+                        srcUrl,
+                        destUrl,
+                        KIO::RenameDialog_Mode(KIO::M_OVERWRITE | KIO::M_SKIP),
+                        -1,
+                        finfo.size(),
+                        now.toTime_t(),
+                        finfo.created().toTime_t(),
+                        now.toTime_t(),
+                        finfo.lastModified().toTime_t()
+                                                                       );
                 result = dlg->exec();
                 delete dlg;
             }
@@ -141,7 +209,7 @@ static bool downloadResource (const KUrl& srcUrl, const KIO::MetaData& metaData
     while (result == KIO::R_CANCEL && destUrl.isValid());
 
     // Save download history
-    Application::instance()->addDownload(srcUrl.pathOrUrl() , destUrl.pathOrUrl());
+    rApp->addDownload(srcUrl.pathOrUrl() , destUrl.pathOrUrl());
 
     if (!KStandardDirs::findExe("kget").isNull() && ReKonfig::kgetDownload())
     {
@@ -207,13 +275,14 @@ WebPage::WebPage(QWidget *parent)
 
     // ----- last stuffs
     connect(manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(manageNetworkErrors(QNetworkReply*)));
+    connect(this, SIGNAL(downloadRequested(const QNetworkRequest &)), this, SLOT(downloadRequest(const QNetworkRequest &)));
     connect(this, SIGNAL(loadStarted()), this, SLOT(loadStarted()));
     connect(this, SIGNAL(loadFinished(bool)), this, SLOT(loadFinished(bool)));
 
     // protocol handler signals
     connect(&_protHandler, SIGNAL(downloadUrl(const KUrl &)), this, SLOT(downloadUrl(const KUrl &)));
 
-    connect(Application::iconManager(), SIGNAL(iconChanged()), mainFrame(), SIGNAL(iconChanged()));
+    connect(rApp->iconManager(), SIGNAL(iconChanged()), mainFrame(), SIGNAL(iconChanged()));
 }
 
 
@@ -225,7 +294,16 @@ WebPage::~WebPage()
 
 bool WebPage::acceptNavigationRequest(QWebFrame *frame, const QNetworkRequest &request, NavigationType type)
 {
-    _isOnRekonqPage = false;
+    if (_isOnRekonqPage)
+    {
+        WebView *view = qobject_cast<WebView *>(parent());
+        WebTab *tab = qobject_cast<WebTab *>(view->parent());
+        _isOnRekonqPage = false;
+        tab->setPart(0, KUrl());     // re-enable the view page
+    }
+
+    // reset webpage values
+    _suggestedFileName.clear();
     _loadingUrl = request.url();
 
     KIO::AccessManager *manager = qobject_cast<KIO::AccessManager*>(networkAccessManager());
@@ -235,7 +313,7 @@ bool WebPage::acceptNavigationRequest(QWebFrame *frame, const QNetworkRequest &r
     if (metaData.contains(QL1S("ssl_in_use")))
     {
         WebSslInfo info;
-        info.fromMetaData(metaData.toVariant());
+        info.restoreFrom(metaData.toVariant(), request.url());
         info.setUrl(request.url());
         _sslInfo = info;
     }
@@ -278,15 +356,6 @@ bool WebPage::acceptNavigationRequest(QWebFrame *frame, const QNetworkRequest &r
         default:
             break;
         }
-
-        if (frame == mainFrame())
-        {
-            setRequestMetaData("main_frame_request", "TRUE");
-        }
-        else
-        {
-            setRequestMetaData("main_frame_request", "FALSE");
-        }
     }
     return KWebPage::acceptNavigationRequest(frame, request, type);
 }
@@ -301,11 +370,11 @@ WebPage *WebPage::createWindow(QWebPage::WebWindowType type)
     WebTab *w = 0;
     if (ReKonfig::openTabNoWindow())
     {
-        w = Application::instance()->mainWindow()->mainView()->newWebTab( !ReKonfig::openTabsBack() );
+        w = rApp->mainWindow()->mainView()->newWebTab(!ReKonfig::openTabsBack());
     }
     else
     {
-        w = Application::instance()->newMainWindow()->mainView()->currentWebTab();
+        w = rApp->newMainWindow()->mainView()->currentWebTab();
     }
     return w->page();
 }
@@ -313,94 +382,52 @@ WebPage *WebPage::createWindow(QWebPage::WebWindowType type)
 
 void WebPage::handleUnsupportedContent(QNetworkReply *reply)
 {
-    Q_ASSERT (reply);
-    // NOTE:
-    // Until kio implements a way to resume/continue a network
-    // request. We must abort the reply to prevent a zombie process
-    // from continuing to download the unsupported content!
+    Q_ASSERT(reply);
+
+    // Put the job on hold...
+#if KDE_IS_VERSION( 4, 5, 96)
+    kDebug() << "PUT REPLY ON HOLD...";
+    KIO::Integration::AccessManager::putReplyOnHold(reply);
+#else
     reply->abort();
+#endif
 
     // This is probably needed just in ONE stupid case..
     if (_protHandler.postHandling(reply->request(), mainFrame()))
+    {
+        kDebug() << "POST HANDLING the unsupported...";
         return;
+    }
 
     if (reply->error() != QNetworkReply::NoError)
         return;
 
+    // get reply url...
     KUrl replyUrl = reply->url();
 
-    // HACK -------------------------------------------
-    // This is done to fix #231204 && #212808
-
-    QString mimeType;
-    QString suggestedFileName;
-
-    QString app = reply->header(QNetworkRequest::ContentTypeHeader).toString();
-    QStringList headerList = app.split( ';' );
-
-    if(headerList.count() > 0)
+    bool isLocal = replyUrl.isLocalFile();
+    if(isLocal && KProtocolInfo::isKnownProtocol(replyUrl))
     {
-        mimeType = headerList.takeFirst().trimmed();
-        Q_FOREACH(const QString &head, headerList)
-        {
-            if( head.contains( QL1S("name") ) )
-            {
-                // this is not so sure.. :)
-                suggestedFileName = head;
-                suggestedFileName = suggestedFileName.remove( QL1S("name=") );
-                suggestedFileName = suggestedFileName.remove( '"' );
-                suggestedFileName = suggestedFileName.trimmed();
-                break;
-            }
-        }
-    }
-    else
-    {
-        mimeType = reply->header(QNetworkRequest::ContentTypeHeader).toString();
+        kDebug() << "WARNING: launching a new app...";
+        new KRun(replyUrl, rApp->mainWindow());  // No need to delete KRun, it autodeletes itself
+        return;
     }
 
-    // NOTE
-    // This part has been copied from KWebPage::downloadResponse code
-    if (reply->hasRawHeader("Content-Disposition"))
-    {
-        KIO::MetaData metaData = reply->attribute(static_cast<QNetworkRequest::Attribute>(KIO::AccessManager::MetaData)).toMap();
-        if (metaData.value(QL1S("content-disposition-type")).compare(QL1S("attachment"), Qt::CaseInsensitive) == 0)
-        {
-            suggestedFileName = metaData.value(QL1S("content-disposition-filename"));
-        }
-        else
-        {
-            const QString value = QL1S(reply->rawHeader("Content-Disposition").simplified());
-            if (value.startsWith(QL1S("attachment"), Qt::CaseInsensitive))
-            {
-                const int length = value.size();
-                int pos = value.indexOf(QL1S("filename"), 0, Qt::CaseInsensitive);
-                if (pos > -1)
-                {
-                    pos += 9;
-                    while (pos < length && (value.at(pos) == QL1C(' ') || value.at(pos) == QL1C('=') || value.at(pos) == QL1C('"')))
-                        pos++;
+    // Get suggested file name...
+    extractSuggestedFileName(reply, _suggestedFileName);
 
-                    int endPos = pos;
-                    while (endPos < length && value.at(endPos) != QL1C('"') && value.at(endPos) != QL1C(';'))
-                        endPos++;
+    // Get mimeType...
+    extractMimeType(reply, _mimeType);
 
-                    if (endPos > pos)
-                    {
-                        suggestedFileName = value.mid(pos, (endPos-pos)).trimmed();
-                    }
-                }
-            }
-        }
-    }
+    // Convert executable text files to plain text...
+    if (KParts::BrowserRun::isTextExecutable(_mimeType))
+        _mimeType = QL1S("text/plain");
 
-    kDebug() << "Detected MimeType = " << mimeType;
-    kDebug() << "Suggested File Name = " << suggestedFileName;
+    kDebug() << "Detected MimeType = " << _mimeType;
+    kDebug() << "Suggested File Name = " << _suggestedFileName;
     // ------------------------------------------------
 
-    KService::Ptr appService = KMimeTypeTrader::self()->preferredService(mimeType);
-
-    bool isLocal = replyUrl.isLocalFile();
+    KService::Ptr appService = KMimeTypeTrader::self()->preferredService(_mimeType);
 
     if (appService.isNull())  // no service can handle this. We can just download it..
     {
@@ -408,23 +435,22 @@ void WebPage::handleUnsupportedContent(QNetworkReply *reply)
 
         isLocal
         ? KMessageBox::sorry(view(), i18n("No service can handle this file."))
-        : downloadReply(reply, suggestedFileName);
+        : downloadReply(reply, _suggestedFileName);
 
         return;
     }
 
     if (!isLocal)
     {
-
-        KParts::BrowserOpenOrSaveQuestion dlg(Application::instance()->mainWindow(), replyUrl, mimeType);
-        if(!suggestedFileName.isEmpty())
-            dlg.setSuggestedFileName(suggestedFileName);
+        KParts::BrowserOpenOrSaveQuestion dlg(rApp->mainWindow(), replyUrl, _mimeType);
+        if (!_suggestedFileName.isEmpty())
+            dlg.setSuggestedFileName(_suggestedFileName);
 
         switch (dlg.askEmbedOrSave())
         {
         case KParts::BrowserOpenOrSaveQuestion::Save:
             kDebug() << "user choice: no services, just download!";
-            downloadReply(reply, suggestedFileName);
+            downloadReply(reply, _suggestedFileName);
             return;
 
         case KParts::BrowserOpenOrSaveQuestion::Cancel:
@@ -435,63 +461,90 @@ void WebPage::handleUnsupportedContent(QNetworkReply *reply)
         }
     }
 
-    // case KParts::BrowserRun::Embed
-    KService::List partServices = KMimeTypeTrader::self()->query(mimeType, QL1S("KParts/ReadOnlyPart"));
-    if (partServices.count() > 0)
+    // Handle Post operations that return content...
+    if (reply->operation() == QNetworkAccessManager::PostOperation)
     {
-        QString p = replyUrl.pathOrUrl();
+        kDebug() << "POST OPERATION: downloading file...";
+        QFileInfo finfo(_suggestedFileName.isEmpty() ? _loadingUrl.fileName() : _suggestedFileName);
+        KTemporaryFile tempFile;
+        tempFile.setSuffix(QL1C('.') + finfo.suffix());
+        tempFile.setAutoRemove(false);
+        tempFile.open();
+        KUrl destUrl;
+        destUrl.setPath(tempFile.fileName());
+        kDebug() << "First save content to" << destUrl;
+        KIO::Job *job = KIO::file_copy(_loadingUrl, destUrl, 0600, KIO::Overwrite);
+        job->ui()->setWindow(rApp->mainWindow());
+        connect(job, SIGNAL(result(KJob *)), this, SLOT(copyToTempFileResult(KJob*)));
+        return;
+    }
 
-        // A part can handle this. Embed it!
-        QString html;
-        html += "<html>";
-        html += "<head>";
-        html += "<title>";
-        html += p;
-        html += "</title>";
-        html += "<style type=\"text/css\">";
-        html += "* { border: 0; padding: 0; margin: 0; }";
-        html += "</style>";
-        html += "</head>";
-        html += "<body>";
-        html += "<object type=\"" + mimeType + "\" data=\"" + p + "\" width=\"100%\" height=\"100%\" />";
-        html += "</body>";
-        html += "</html>";
+    // case KParts::BrowserRun::Embed
+    KParts::ReadOnlyPart *pa = KMimeTypeTrader::createPartInstanceFromQuery<KParts::ReadOnlyPart>(_mimeType, view(), this, QString());
+    if (pa)
+    {
+        kDebug() << "EMBEDDING CONTENT...";
 
-        mainFrame()->setHtml(html);
         _isOnRekonqPage = true;
-        Application::instance()->mainWindow()->mainView()->urlBar()->setQUrl(replyUrl);
-        Application::instance()->mainWindow()->updateActions();
+
+        WebView *view = qobject_cast<WebView *>(parent());
+        WebTab *tab = qobject_cast<WebTab *>(view->parent());
+        tab->setPart(pa, replyUrl);
+
+        UrlBar *bar = tab->urlBar();
+        bar->setQUrl(replyUrl);
+
+        rApp->mainWindow()->updateActions();
     }
     else
     {
         // No parts, just app services. Load it!
-        KRun::run(*appService, replyUrl, 0);
+        // If the app is a KDE one, publish the slave on hold to let it use it.
+        // Otherwise, run the app and remove it (the io slave...)
+        if (appService->categories().contains(QL1S("KDE"), Qt::CaseInsensitive))
+        {
+#if KDE_IS_VERSION( 4, 5, 96)
+            KIO::Scheduler::publishSlaveOnHold();
+#endif
+            KRun::run(*appService, replyUrl, 0, false, _suggestedFileName);
+            return;
+        }
+        KRun::run(*appService, replyUrl, 0, false, _suggestedFileName);
     }
 
+    // Remove any ioslave that was put on hold...
+#if KDE_IS_VERSION( 4, 5, 96)
+    kDebug() << "REMOVE SLAVES ON HOLD...";
+    KIO::Scheduler::removeSlaveOnHold();
+#endif
+
     return;
 }
 
 
 void WebPage::loadStarted()
 {
-    // HACK: 
-    // Chinese encoding Fix. See BUG: 251264
-    // Use gb18030 instead of gb2312
-    if(settings()->defaultTextEncoding() == QL1S("gb2312"))
-    {
-        settings()->setDefaultTextEncoding( QL1S("gb18030") );
-    }    
 }
 
 void WebPage::loadFinished(bool ok)
 {
     Q_UNUSED(ok);
 
+    // set zoom factor
+    QString val;
+    KSharedConfig::Ptr config = KGlobal::config();
+    KConfigGroup group(config, "Zoom");
+    val = group.readEntry(_loadingUrl.host(), QString("10"));
+
+    int value = val.toInt();
+    kDebug() << "ZOOM VALUE: " << _loadingUrl.host() << value;
+    mainFrame()->setZoomFactor(QVariant(value).toReal() / 10);  // Don't allox max +1 values
+
     // Provide site icon. Can this be moved to loadStarted??
-    Application::iconManager()->provideIcon(this, _loadingUrl);
+    rApp->iconManager()->provideIcon(this, _loadingUrl);
 
     // Apply adblock manager hiding rules
-    Application::adblockManager()->applyHidingRules(this);
+    rApp->adblockManager()->applyHidingRules(this);
 
     // KWallet Integration
     QStringList list = ReKonfig::walletBlackList();
@@ -508,6 +561,10 @@ void WebPage::manageNetworkErrors(QNetworkReply *reply)
 {
     Q_ASSERT(reply);
 
+    // check suggested file name
+    if (_suggestedFileName.isEmpty())
+        extractSuggestedFileName(reply, _suggestedFileName);
+
     QWebFrame* frame = qobject_cast<QWebFrame *>(reply->request().originatingObject());
     const bool isMainFrameRequest = (frame == mainFrame());
 
@@ -529,7 +586,7 @@ void WebPage::manageNetworkErrors(QNetworkReply *reply)
         if (isMainFrameRequest && !_sslInfo.isValid())
         {
             // Obtain and set the SSL information if any...
-            _sslInfo.fromMetaData(reply->attribute(static_cast<QNetworkRequest::Attribute>(KIO::AccessManager::MetaData)));
+            _sslInfo.restoreFrom(reply->attribute(static_cast<QNetworkRequest::Attribute>(KIO::AccessManager::MetaData)), reply->url());
             _sslInfo.setUrl(reply->url());
         }
         break;
@@ -559,11 +616,16 @@ void WebPage::manageNetworkErrors(QNetworkReply *reply)
         if (reply->url() == _loadingUrl)
         {
             frame->setHtml(errorPage(reply));
-            if(isMainFrameRequest)
+            if (isMainFrameRequest)
             {
-            _isOnRekonqPage = true;
-            Application::instance()->mainWindow()->mainView()->urlBar()->setQUrl(_loadingUrl);
-            Application::instance()->mainWindow()->updateActions();
+                _isOnRekonqPage = true;
+
+                WebView *view = qobject_cast<WebView *>(parent());
+                WebTab *tab = qobject_cast<WebTab *>(view->parent());
+                UrlBar *bar = tab->urlBar();
+                bar->setQUrl(_loadingUrl);
+
+                rApp->mainWindow()->updateActions();
             }
         }
         break;
@@ -589,7 +651,10 @@ QString WebPage::errorPage(QNetworkReply *reply)
     }
 
     QString title = i18n("There was a problem while loading the page");
-    QString urlString = reply->url().toString(QUrl::RemoveUserInfo | QUrl::RemoveQuery | QUrl::RemovePath);
+
+    // NOTE:
+    // this, to take care about XSS (see BUG 217464)...
+    QString urlString = Qt::escape(reply->url().toString());
 
     QString iconPath = QString("file://") + KIconLoader::global()->iconPath("dialog-warning" , KIconLoader::Small);
     iconPath.replace(QL1S("16"), QL1S("128"));
@@ -621,7 +686,7 @@ QString WebPage::errorPage(QNetworkReply *reply)
 
 void WebPage::downloadReply(const QNetworkReply *reply, const QString &suggestedFileName)
 {
-    downloadResource( reply->url(), KIO::MetaData(), view(), suggestedFileName);
+    downloadResource(reply->url(), KIO::MetaData(), view(), suggestedFileName);
 }
 
 
@@ -635,7 +700,7 @@ void WebPage::downloadRequest(const QNetworkRequest &request)
 
 void WebPage::downloadUrl(const KUrl &url)
 {
-    downloadResource( url, KIO::MetaData(), view() );
+    downloadResource(url, KIO::MetaData(), view());
 }
 
 
@@ -717,3 +782,12 @@ void WebPage::updateImage(bool ok)
         p.snapFinished();
     }
 }
+
+
+void WebPage::copyToTempFileResult(KJob* job)
+{
+    if (job->error())
+        job->uiDelegate()->showErrorMessage();
+    else
+        (void)KRun::runUrl(static_cast<KIO::FileCopyJob *>(job)->destUrl(), _mimeType, rApp->mainWindow());
+}
