@@ -6,7 +6,7 @@
 * Copyright (C) 2008 Dirk Mueller <mueller@kde.org>
 * Copyright (C) 2008 Urs Wolfer <uwolfer @ kde.org>
 * Copyright (C) 2008 Michael Howell <mhowell123@gmail.com>
-* Copyright (C) 2008-2010 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
 * Copyright (C) 2010 by Matthieu Gicquel <matgic78 at gmail dot com>
 *
 *
@@ -52,11 +52,33 @@ public:
     explicit WebPage(QWidget *parent = 0);
     ~WebPage();
 
-    inline bool hasNetworkAnalyzerEnabled() const { return _networkAnalyzer; };
-    inline void enableNetworkAnalyzer(bool b) { _networkAnalyzer = b; };
+    inline bool hasNetworkAnalyzerEnabled() const
+    {
+        return _networkAnalyzer;
+    };
+    inline void enableNetworkAnalyzer(bool b)
+    {
+        _networkAnalyzer = b;
+    };
 
-    inline bool isOnRekonqPage() const { return _isOnRekonqPage; };
-    inline void setIsOnRekonqPage(bool b) { _isOnRekonqPage = b; };
+    inline bool isOnRekonqPage() const
+    {
+        return _isOnRekonqPage;
+    };
+    inline void setIsOnRekonqPage(bool b)
+    {
+        _isOnRekonqPage = b;
+    };
+
+    inline KUrl loadingUrl()
+    {
+        return _loadingUrl;
+    };
+    
+    inline QString suggestedFileName()
+    {
+        return _suggestedFileName;
+    };
 
 public Q_SLOTS:
     void downloadAllContentsWithKGet(QPoint);
@@ -79,15 +101,21 @@ private Q_SLOTS:
     void showSSLInfo(QPoint);
     void updateImage(bool ok);
 
+    void copyToTempFileResult(KJob*);
+
+private:
     void downloadReply(const QNetworkReply *reply, const QString &suggestedFileName = QString());
 
 private:
     QString errorPage(QNetworkReply *reply);
-    QUrl _loadingUrl;
+    KUrl _loadingUrl;
 
     ProtocolHandler _protHandler;
     WebSslInfo _sslInfo;
 
+    QString _mimeType;
+    QString _suggestedFileName;
+
     bool _networkAnalyzer;
     bool _isOnRekonqPage;
 };
