@@ -2,8 +2,8 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2010 by Andrea Diamantini <adjam7 at gmail dot com>
-* Copyright (C) 2009-2010 by Lionel Chauvin <megabigbug@yahoo.fr>
+* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -33,14 +33,21 @@
 #include "rekonq_defines.h"
 
 // Local Includes
+#include "webpage.h"
 #include "webview.h"
 
+// KDE Includes
+#include <KParts/Part>
+
 // Qt Includes
 #include <QtGui/QWidget>
 
 // Forward Declarations
+class UrlBar;
 class PreviewSelectorBar;
 class WalletBar;
+class NotificationBar;
+class QPoint;
 
 
 class REKONQ_TESTS_EXPORT WebTab : public QWidget
@@ -51,32 +58,63 @@ public:
     explicit WebTab(QWidget *parent = 0);
     ~WebTab();
 
-    inline WebView *view() { return _view; }
-    inline WebPage *page() { return view()->page(); }
-    inline int progress() { return m_progress; }
+    inline WebView *view() const
+    {
+        return m_webView;
+    }
+    UrlBar *urlBar() const
+    {
+        return m_urlBar;
+    }
+    inline WebPage *page() const
+    {
+        return view()->page();
+    }
+    inline int progress() const
+    {
+        return m_progress;
+    }
 
     KUrl url();
     void createPreviewSelectorBar(int index);
+    void insertBar(NotificationBar* bar);
 
     bool hasRSSInfo();
 
-private slots:
+    bool isPageLoading();
+    bool hasNewSearchEngine();
+
+    KParts::ReadOnlyPart *part()
+    {
+        return m_part;
+    }
+    void setPart(KParts::ReadOnlyPart *p, const KUrl &u);
+
+private Q_SLOTS:
     void updateProgress(int progress);
     void loadFinished(bool);
 
     void createWalletBar(const QString &, const QUrl &);
-    void showRSSInfo(QPoint pos);
+    void showRSSInfo(const QPoint &pos);
+    void showSearchEngine(const QPoint &pos);
+    void openSearchEngineAdded();
 
-signals:
+Q_SIGNALS:
     void loadProgressing();
+    void titleChanged(const QString &);
 
 private:
-    WebView *_view;
+    KUrl extractOpensearchUrl(QWebElement e);
+
+    WebView *const m_webView;
+    UrlBar *const m_urlBar;
 
     int m_progress;
 
-    QWeakPointer<WalletBar> _walletBar;
-    QWeakPointer<PreviewSelectorBar> _previewSelectorBar;
+    QWeakPointer<WalletBar> m_walletBar;
+    QWeakPointer<PreviewSelectorBar> m_previewSelectorBar;
+
+    KParts::ReadOnlyPart *m_part;
 };
 
 #endif
