@@ -2,8 +2,8 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2010 by Andrea Diamantini <adjam7 at gmail dot com>
-* Copyright (C) 2009-2010 by Lionel Chauvin <megabigbug@yahoo.fr>
+* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -35,6 +35,7 @@
 // Local Includes
 #include "application.h"
 #include "bookmarkprovider.h"
+#include "bookmarkowner.h"
 #include "iconmanager.h"
 #include "mainview.h"
 #include "mainwindow.h"
@@ -42,6 +43,7 @@
 #include "urlbar.h"
 #include "webpage.h"
 #include "websnap.h"
+#include "webtab.h"
 
 // KDE Includes
 #include <KAction>
@@ -65,15 +67,16 @@
 
 WebView::WebView(QWidget* parent)
         : KWebView(parent, false)
-        , _mousePos(QPoint(0, 0))
-        , _autoScrollTimer(new QTimer(this))
-        , _VScrollSpeed(0)
-        , _HScrollSpeed(0)
-        , _canEnableAutoScroll(true)
-        , _isAutoScrollEnabled(false)
-        , _smoothScrollTimer(new QTimer(this))
-        , _smoothScrolling(false)
-        , _dy(0)
+        , m_mousePos(QPoint(0, 0))
+        , m_autoScrollTimer(new QTimer(this))
+        , m_vScrollSpeed(0)
+        , m_hScrollSpeed(0)
+        , m_canEnableAutoScroll(true)
+        , m_isAutoScrollEnabled(false)
+        , m_smoothScrollTimer(new QTimer(this))
+        , m_smoothScrolling(false)
+        , m_dy(0)
+        , m_smoothScrollSteps(0)
 {
     WebPage *page = new WebPage(this);
     setPage(page);
@@ -86,7 +89,8 @@ WebView::WebView(QWidget* parent)
     // // But it doesn't work :(
     // // We'll see in next KDE releases...
     QPalette p;
-    if (p.color(QPalette::ButtonText).lightness() > 50) { //if it is a dark theme
+    if (p.color(QPalette::ButtonText).lightness() > 50)   //if it is a dark theme
+    {
         QWindowsStyle s;
         p = s.standardPalette();
         setPalette(p);
@@ -94,34 +98,30 @@ WebView::WebView(QWidget* parent)
 
     // download system
     connect(this, SIGNAL(linkShiftClicked(const KUrl &)), page, SLOT(downloadUrl(const KUrl &)));
-    connect(page, SIGNAL(downloadRequested(const QNetworkRequest &)), page, SLOT(downloadRequest(const QNetworkRequest &)));
 
     // middle click || ctrl + click signal
     connect(this, SIGNAL(linkMiddleOrCtrlClicked(const KUrl &)), this, SLOT(loadUrlInNewTab(const KUrl &)));
 
     // loadUrl signal
     connect(this, SIGNAL(loadUrl(const KUrl &, const Rekonq::OpenType &)),
-            Application::instance(), SLOT(loadUrl(const KUrl &, const Rekonq::OpenType &)));
+            rApp, SLOT(loadUrl(const KUrl &, const Rekonq::OpenType &)));
 
     // Auto scroll timer
-    connect(_autoScrollTimer, SIGNAL(timeout()), this, SLOT(scrollFrameChanged()));
-    _autoScrollTimer->setInterval(100);
+    connect(m_autoScrollTimer, SIGNAL(timeout()), this, SLOT(scrollFrameChanged()));
+    m_autoScrollTimer->setInterval(100);
 
     // Smooth scroll timer
-    connect(_smoothScrollTimer, SIGNAL(timeout()), this, SLOT(scrollTick()));
-    _smoothScrollTimer->setInterval(16);
+    connect(m_smoothScrollTimer, SIGNAL(timeout()), this, SLOT(scrollTick()));
+    m_smoothScrollTimer->setInterval(16);
+
+    connect(this, SIGNAL(iconChanged()), this, SLOT(changeWindowIcon()));
 }
 
 
 WebView::~WebView()
 {
-    delete _autoScrollTimer;
-
-    if (_smoothScrolling)
+    if (m_smoothScrolling)
         stopScrolling();
-    delete _smoothScrollTimer;
-
-    disconnect();
 
     WebPage* p = page();
 
@@ -131,18 +131,32 @@ WebView::~WebView()
     preview.save(path);
 }
 
+void WebView::changeWindowIcon()
+{
+    if (ReKonfig::useFavicon())
+    {
+        MainWindow *const mainWindow = rApp->mainWindow();
+        if (url() == mainWindow->currentTab()->url())
+        {
+            const int index = mainWindow->mainView()->currentIndex();
+            mainWindow->changeWindowIcon(index);
+        }
+    }
+}
 
 WebPage *WebView::page()
 {
-    WebPage *page = qobject_cast<WebPage *>(KWebView::page());
+    WebPage *const page = qobject_cast<WebPage *>(KWebView::page());
+    Q_ASSERT(page);
     return page;
 }
 
 
+// TODO: refactor me!
 void WebView::contextMenuEvent(QContextMenuEvent *event)
 {
     QWebHitTestResult result = page()->mainFrame()->hitTestContent(event->pos());
-    MainWindow *mainwindow = Application::instance()->mainWindow();
+    MainWindow *mainwindow = rApp->mainWindow();
 
     KMenu menu(this);
     QAction *a;
@@ -164,6 +178,7 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
         connect(a, SIGNAL(triggered(bool)), this, SLOT(openLinkInNewWindow()));
         menu.addAction(a);
 
+        menu.addSeparator();
         menu.addAction(pageAction(KWebPage::DownloadLinkToDisk));
         menu.addAction(pageAction(KWebPage::CopyLinkToClipboard));
         menu.addSeparator();
@@ -185,6 +200,34 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
         else
             a->setText(i18n("Copy"));
         menu.addAction(a);
+        if (selectedText().contains('.') && selectedText().indexOf('.') < selectedText().length() && !selectedText().trimmed().contains(" "))
+        {
+            QString text = selectedText();
+            text = text.trimmed();
+            KUrl urlLikeText(text);
+            if (urlLikeText.isValid())
+            {
+                QString truncatedUrl = text;
+                const int maxTextSize = 18;
+                if (truncatedUrl.length() > maxTextSize)
+                {
+                    const int truncateSize = 15;
+                    truncatedUrl.truncate(truncateSize);
+                    truncatedUrl += QL1S("...");
+                }
+                //open selected text url in a new tab
+                QAction * const openInNewTabAction = new KAction(KIcon("tab-new"), i18n("Open '%1' in New Tab", truncatedUrl), this);
+                openInNewTabAction->setData(QUrl(urlLikeText));
+                connect(openInNewTabAction, SIGNAL(triggered(bool)), this, SLOT(openLinkInNewTab()));
+                menu.addAction(openInNewTabAction);
+                //open selected text url in a new window
+                QAction * const openInNewWindowAction = new KAction(KIcon("window-new"), i18n("Open '%1' in New Window", truncatedUrl), this);
+                openInNewWindowAction->setData(QUrl(urlLikeText));
+                connect(openInNewWindowAction, SIGNAL(triggered(bool)), this, SLOT(openLinkInNewWindow()));
+                menu.addAction(openInNewWindowAction);
+                menu.addSeparator();
+            }
+        }
     }
 
     // is content editable? Add PASTE
@@ -201,12 +244,16 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
         foreach(KService::Ptr engine, SearchEngine::favorites())
         {
             a = new KAction(engine->name(), this);
-            a->setIcon(Application::iconManager()->iconForUrl(SearchEngine::buildQuery(engine, "")));
+            a->setIcon(rApp->iconManager()->iconForUrl(SearchEngine::buildQuery(engine, "")));
             a->setData(engine->entryPath());
             connect(a, SIGNAL(triggered(bool)), this, SLOT(search()));
             searchMenu->addAction(a);
         }
 
+        a = new KAction(KIcon("edit-find"), i18n("On Current Page"), this);
+        connect(a, SIGNAL(triggered()), rApp->mainWindow(), SLOT(findSelectedText()));
+        searchMenu->addAction(a);
+
         if (!searchMenu->menu()->isEmpty())
         {
             menu.addAction(searchMenu);
@@ -214,7 +261,8 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
 
         menu.addSeparator();
 
-        menu.addAction(inspectAction);
+        if (ReKonfig::showDeveloperTools())
+            menu.addAction(inspectAction);
         // TODO Add translate, show translation
     }
 
@@ -237,45 +285,26 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
         connect(a, SIGNAL(triggered(Qt::MouseButtons, Qt::KeyboardModifiers)), this, SLOT(slotCopyImageLocation()));
         menu.addAction(a);
 
-        menu.addAction(pageAction(KWebPage::CopyLinkToClipboard));
         menu.addSeparator();
 
-        menu.addAction(inspectAction);
+        if (ReKonfig::showDeveloperTools())
+            menu.addAction(inspectAction);
     }
 
-    // Open url text in new tab/window
-    if (result.linkUrl().isEmpty())
-    {
+// TODO: REMOVE ME!
+// These actions are here just to NOT remove strings until stable release.
+// Just remove the lines here after that.
+if (false)
+{
+    QString truncatedURL;
 
-        QString text = selectedText();
-        text = text.trimmed();
-        if (text.startsWith(QL1S("http://"))
-                || text.startsWith(QL1S("https://"))
-                || text.startsWith(QL1S("www."))
-           )
-        {
-            QString truncatedURL = text;
-            if (text.length() > 18)
-            {
-                truncatedURL.truncate(15);
-                truncatedURL += "...";
-            }
+    //open selected text url in a new tab
+    a = new KAction(KIcon("tab-new"), i18n("Open '%1' in New Tab", truncatedURL), this);
 
-            //open selected text url in a new tab
-            a = new KAction(KIcon("tab-new"), i18n("Open '%1' in New Tab", truncatedURL), this);
-            a->setData(QUrl(text));
-            connect(a, SIGNAL(triggered(bool)), this, SLOT(openLinkInNewTab()));
-            menu.addAction(a);
-
-            //open selected text url in a new window
-            a = new KAction(KIcon("window-new"), i18n("Open '%1' in New Window", truncatedURL), this);
-            a->setData(QUrl(text));
-            connect(a, SIGNAL(triggered(bool)), this, SLOT(openLinkInNewWindow()));
-            menu.addAction(a);
-
-            menu.addSeparator();
-        }
-    }
+    //open selected text url in a new window
+    a = new KAction(KIcon("window-new"), i18n("Open '%1' in New Window", truncatedURL), this);
+}
+// TODO: ----------- END REMOVE
 
     // page actions
     if (!result.isContentSelected() && result.linkUrl().isEmpty())
@@ -299,7 +328,9 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
         {
             menu.addSeparator();
 
-            menu.addAction(mainwindow->actionByName("new_tab"));
+            if (!ReKonfig::alwaysShowTabBar() && mainwindow->mainView()->count() == 1)
+                menu.addAction(mainwindow->actionByName("new_tab"));
+
             menu.addAction(mainwindow->actionByName("new_window"));
 
             menu.addSeparator();
@@ -329,11 +360,13 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
                 menu.addAction(a);
             }
 
-            menu.addAction(mainwindow->actionByName("page_source"));
+            if (ReKonfig::showDeveloperTools())
+            {
+                menu.addAction(mainwindow->actionByName("page_source"));
+                menu.addAction(inspectAction);
+            }
 
-            menu.addAction(inspectAction);
-
-            a = Application::bookmarkProvider()->actionByName("rekonq_add_bookmark");
+            a = rApp->bookmarkProvider()->actionByName("rekonq_add_bookmark");
             menu.addAction(a);
         }
 
@@ -350,18 +383,18 @@ void WebView::contextMenuEvent(QContextMenuEvent *event)
 
 void WebView::mousePressEvent(QMouseEvent *event)
 {
-    if (_isAutoScrollEnabled)
+    if (m_isAutoScrollEnabled)
     {
         setCursor(Qt::ArrowCursor);
-        _VScrollSpeed = 0;
-        _HScrollSpeed = 0;
-        _autoScrollTimer->stop();
-        _isAutoScrollEnabled = false;
+        m_vScrollSpeed = 0;
+        m_hScrollSpeed = 0;
+        m_autoScrollTimer->stop();
+        m_isAutoScrollEnabled = false;
         return;
     }
 
     QWebHitTestResult result = page()->mainFrame()->hitTestContent(event->pos());
-    _canEnableAutoScroll = ReKonfig::autoScroll() && !result.isContentEditable()  && result.linkUrl().isEmpty();
+    m_canEnableAutoScroll = ReKonfig::autoScroll() && !result.isContentEditable()  && result.linkUrl().isEmpty();
 
     switch (event->button())
     {
@@ -374,11 +407,18 @@ void WebView::mousePressEvent(QMouseEvent *event)
         break;
 
     case Qt::MidButton:
-        if (_canEnableAutoScroll && !_isAutoScrollEnabled)
+        if (m_canEnableAutoScroll
+                && !m_isAutoScrollEnabled
+                && !page()->currentFrame()->scrollBarGeometry(Qt::Horizontal).contains(event->pos())
+                && !page()->currentFrame()->scrollBarGeometry(Qt::Vertical).contains(event->pos()))
         {
-            setCursor(KIcon("transform-move").pixmap(32));
-            _clickPos = event->pos();
-            _isAutoScrollEnabled = true;
+            if (!page()->currentFrame()->scrollBarGeometry(Qt::Horizontal).isNull()
+                    || !page()->currentFrame()->scrollBarGeometry(Qt::Vertical).isNull())
+            {
+                setCursor(KIcon("transform-move").pixmap(32));
+                m_clickPos = event->pos();
+                m_isAutoScrollEnabled = true;
+            }
         }
         break;
 
@@ -391,20 +431,20 @@ void WebView::mousePressEvent(QMouseEvent *event)
 
 void WebView::mouseMoveEvent(QMouseEvent *event)
 {
-    _mousePos = event->pos();
+    m_mousePos = event->pos();
 
-    if (_isAutoScrollEnabled)
+    if (m_isAutoScrollEnabled)
     {
-        QPoint r = _mousePos - _clickPos;
-        _HScrollSpeed = r.x() / 2;  // you are too fast..
-        _VScrollSpeed = r.y() / 2;
-        if (!_autoScrollTimer->isActive())
-            _autoScrollTimer->start();
+        QPoint r = m_mousePos - m_clickPos;
+        m_hScrollSpeed = r.x() / 2;  // you are too fast..
+        m_vScrollSpeed = r.y() / 2;
+        if (!m_autoScrollTimer->isActive())
+            m_autoScrollTimer->start();
 
         return;
     }
 
-    MainWindow *w = Application::instance()->mainWindow();
+    MainWindow *w = rApp->mainWindow();
     if (w->isFullScreen())
     {
         if (event->pos().y() >= 0 && event->pos().y() <= 4)
@@ -413,7 +453,7 @@ void WebView::mouseMoveEvent(QMouseEvent *event)
         }
         else
         {
-            if(!w->mainView()->urlBar()->hasFocus())
+            if (!w->mainView()->currentUrlBar()->hasFocus())
                 w->setWidgetsVisible(false);
         }
     }
@@ -421,6 +461,51 @@ void WebView::mouseMoveEvent(QMouseEvent *event)
 }
 
 
+void WebView::enterEvent(QEvent *event)
+{
+    if (m_isAutoScrollEnabled)
+        setCursor(KIcon("transform-move").pixmap(32));
+
+    KWebView::enterEvent(event);
+}
+
+
+void WebView::dropEvent(QDropEvent *event)
+{
+    bool isEditable = page()->frameAt(event->pos())->hitTestContent(event->pos()).isContentEditable();
+    if (event->mimeData()->hasFormat("application/rekonq-bookmark"))
+    {
+        QByteArray addresses = event->mimeData()->data("application/rekonq-bookmark");
+        KBookmark bookmark =  rApp->bookmarkProvider()->bookmarkManager()->findByAddress(QString::fromLatin1(addresses.data()));
+        if (bookmark.isGroup())
+        {
+            rApp->bookmarkProvider()->bookmarkOwner()->openFolderinTabs(bookmark.toGroup());
+        }
+        else
+        {
+            emit loadUrl(bookmark.url(), Rekonq::CurrentTab);
+        }
+    }
+    else if (event->mimeData()->hasUrls() && event->source() != this && !isEditable) //dropped links
+    {
+        Q_FOREACH (const QUrl &url, event->mimeData()->urls())
+        {
+            emit loadUrl(url, Rekonq::NewFocusedTab);
+        }
+    }
+    else if (event->mimeData()->hasFormat("text/plain") && event->source() != this && !isEditable) //dropped plain text with url format
+    {
+        QUrl url = QUrl::fromUserInput(event->mimeData()->data("text/plain"));
+
+        if (url.isValid())
+            emit loadUrl(url, Rekonq::NewFocusedTab);
+    }
+    else
+    {
+        KWebView::dropEvent(event);
+    }
+}
+
 void WebView::search()
 {
     KAction *a = qobject_cast<KAction*>(sender());
@@ -433,7 +518,7 @@ void WebView::search()
 
 void WebView::printFrame()
 {
-    Application::instance()->mainWindow()->printRequested(page()->currentFrame());
+    rApp->mainWindow()->printRequested(page()->currentFrame());
 }
 
 
@@ -455,17 +540,17 @@ void WebView::viewImage(Qt::MouseButtons buttons, Qt::KeyboardModifiers modifier
 void WebView::slotCopyImageLocation()
 {
     KAction *a = qobject_cast<KAction*>(sender());
-    KUrl imageUrl (a->data().toUrl());
+    KUrl imageUrl(a->data().toUrl());
 #ifndef QT_NO_MIMECLIPBOARD
     // Set it in both the mouse selection and in the clipboard
     QMimeData* mimeData = new QMimeData;
-    imageUrl.populateMimeData( mimeData );
-    QApplication::clipboard()->setMimeData( mimeData, QClipboard::Clipboard );
+    imageUrl.populateMimeData(mimeData);
+    QApplication::clipboard()->setMimeData(mimeData, QClipboard::Clipboard);
     mimeData = new QMimeData;
-    imageUrl.populateMimeData( mimeData );
-    QApplication::clipboard()->setMimeData( mimeData, QClipboard::Selection );
+    imageUrl.populateMimeData(mimeData);
+    QApplication::clipboard()->setMimeData(mimeData, QClipboard::Selection);
 #else
-    QApplication::clipboard()->setText( imageUrl.url() );
+    QApplication::clipboard()->setText(imageUrl.url());
 #endif
 }
 
@@ -505,7 +590,7 @@ void WebView::keyPressEvent(QKeyEvent *event)
         }
     }
 
-    if (!_canEnableAutoScroll)
+    if (!m_canEnableAutoScroll)
     {
         KWebView::keyPressEvent(event);
         return;
@@ -516,44 +601,44 @@ void WebView::keyPressEvent(QKeyEvent *event)
     {
         if (event->key() == Qt::Key_Up)
         {
-            _VScrollSpeed--;
-            if (!_autoScrollTimer->isActive())
-                _autoScrollTimer->start();
+            m_vScrollSpeed--;
+            if (!m_autoScrollTimer->isActive())
+                m_autoScrollTimer->start();
             return;
         }
 
         if (event->key() == Qt::Key_Down)
         {
-            _VScrollSpeed++;
-            if (!_autoScrollTimer->isActive())
-                _autoScrollTimer->start();
+            m_vScrollSpeed++;
+            if (!m_autoScrollTimer->isActive())
+                m_autoScrollTimer->start();
             return;
         }
 
         if (event->key() == Qt::Key_Right)
         {
-            _HScrollSpeed++;
-            if (!_autoScrollTimer->isActive())
-                _autoScrollTimer->start();
+            m_hScrollSpeed++;
+            if (!m_autoScrollTimer->isActive())
+                m_autoScrollTimer->start();
             return;
         }
 
         if (event->key() == Qt::Key_Left)
         {
-            _HScrollSpeed--;
-            if (!_autoScrollTimer->isActive())
-                _autoScrollTimer->start();
+            m_hScrollSpeed--;
+            if (!m_autoScrollTimer->isActive())
+                m_autoScrollTimer->start();
             return;
         }
 
-        if (_autoScrollTimer->isActive())
+        if (m_autoScrollTimer->isActive())
         {
-            _autoScrollTimer->stop();
+            m_autoScrollTimer->stop();
         }
         else
         {
-            if (_VScrollSpeed || _HScrollSpeed)
-                _autoScrollTimer->start();
+            if (m_vScrollSpeed || m_hScrollSpeed)
+                m_autoScrollTimer->start();
         }
     }
 
@@ -563,49 +648,66 @@ void WebView::keyPressEvent(QKeyEvent *event)
 
 void WebView::wheelEvent(QWheelEvent *event)
 {
-    // To let some websites (eg: google maps) to handle wheel events
-    int prevPos = page()->currentFrame()->scrollPosition().y();
-    KWebView::wheelEvent(event);
-    int newPos = page()->currentFrame()->scrollPosition().y();
-
-    // Sync with the zoom slider
-    if (event->modifiers() == Qt::ControlModifier)
+    if (event->orientation() == Qt::Vertical || !ReKonfig::hScrollWheelHistory())
     {
-        // Limits of the slider
-        if (zoomFactor() > 1.9)
-            setZoomFactor(1.9);
-        else if (zoomFactor() < 0.1)
-            setZoomFactor(0.1);
+        // To let some websites (eg: google maps) to handle wheel events
+        int prevPos = page()->currentFrame()->scrollPosition().y();
+        KWebView::wheelEvent(event);
+        int newPos = page()->currentFrame()->scrollPosition().y();
 
-        // Round the factor (Fix slider's end value)
-        int newFactor = zoomFactor() * 10;
-        if ((zoomFactor() * 10 - newFactor) > 0.5)
-            newFactor++;
+        // Sync with the zoom slider
+        if (event->modifiers() == Qt::ControlModifier)
+        {
+            // Limits of the slider
+            if (zoomFactor() > 1.9)
+                setZoomFactor(1.9);
+            else if (zoomFactor() < 0.1)
+                setZoomFactor(0.1);
 
-        emit zoomChanged(newFactor);
+            // Round the factor (Fix slider's end value)
+            int newFactor = zoomFactor() * 10;
+            if ((zoomFactor() * 10 - newFactor) > 0.5)
+                newFactor++;
+
+            emit zoomChanged(newFactor);
+        }
+        else if (ReKonfig::smoothScrolling() && prevPos != newPos)
+        {
+
+            page()->currentFrame()->setScrollPosition(QPoint(page()->currentFrame()->scrollPosition().x(), prevPos));
+
+            if ((event->delta() > 0) != !m_scrollBottom)
+                stopScrolling();
+
+            if (event->delta() > 0)
+                m_scrollBottom = false;
+            else
+                m_scrollBottom = true;
+
+
+            setupSmoothScrolling(abs(newPos - prevPos));
+        }
     }
-    else if (ReKonfig::smoothScrolling() && prevPos != newPos)
+    // use horizontal wheel events to go back and forward in tab history
+    else
     {
-
-        page()->currentFrame()->setScrollPosition(QPoint(page()->currentFrame()->scrollPosition().x(), prevPos));
-
-        if ((event->delta() > 0) != !_scrollBottom)
-            stopScrolling();
-
+        // left -> go to previous page
         if (event->delta() > 0)
-            _scrollBottom = false;
-        else
-            _scrollBottom = true;
-
-
-        setupSmoothScrolling(abs(newPos - prevPos));
+        {
+            emit openPreviousInHistory();
+        }
+        // right -> go to next page
+        if (event->delta() < 0)
+        {
+            emit openNextInHistory();
+        }
     }
 }
 
 
 void WebView::inspect()
 {
-    QAction *a = Application::instance()->mainWindow()->actionByName("web_inspector");
+    QAction *a = rApp->mainWindow()->actionByName("web_inspector");
     if (a && !a->isChecked())
         a->trigger();
     pageAction(QWebPage::InspectElement)->trigger();
@@ -621,46 +723,46 @@ void WebView::loadUrlInNewTab(const KUrl &url)
 void WebView::scrollFrameChanged()
 {
     // do the scrolling
-    page()->currentFrame()->scroll(_HScrollSpeed, _VScrollSpeed);
+    page()->currentFrame()->scroll(m_hScrollSpeed, m_vScrollSpeed);
 
     // check if we reached the end
     int y = page()->currentFrame()->scrollPosition().y();
     if (y == 0 || y == page()->currentFrame()->scrollBarMaximum(Qt::Vertical))
-        _VScrollSpeed = 0;
+        m_vScrollSpeed = 0;
 
     int x = page()->currentFrame()->scrollPosition().x();
     if (x == 0 || x == page()->currentFrame()->scrollBarMaximum(Qt::Horizontal))
-        _HScrollSpeed = 0;
+        m_hScrollSpeed = 0;
 }
 
 
 void WebView::setupSmoothScrolling(int posY)
 {
-    int ddy = qMax(_smoothScrollSteps ? abs(_dy)/_smoothScrollSteps : 0,3);
+    int ddy = qMax(m_smoothScrollSteps ? abs(m_dy) / m_smoothScrollSteps : 0, 3);
 
-    _dy += posY;
+    m_dy += posY;
 
-    if (_dy <= 0)
+    if (m_dy <= 0)
     {
         stopScrolling();
         return;
     }
 
-    _smoothScrollSteps = 8;
+    m_smoothScrollSteps = 8;
 
-    if (_dy / _smoothScrollSteps < ddy)
+    if (m_dy / m_smoothScrollSteps < ddy)
     {
-        _smoothScrollSteps = (abs(_dy)+ddy-1)/ddy;
-        if (_smoothScrollSteps < 1)
-            _smoothScrollSteps = 1;
+        m_smoothScrollSteps = (abs(m_dy) + ddy - 1) / ddy;
+        if (m_smoothScrollSteps < 1)
+            m_smoothScrollSteps = 1;
     }
 
-    _smoothScrollTime.start();
+    m_smoothScrollTime.start();
 
-    if (!_smoothScrolling)
+    if (!m_smoothScrolling)
     {
-        _smoothScrolling = true;
-        _smoothScrollTimer->start();
+        m_smoothScrolling = true;
+        m_smoothScrollTimer->start();
         scrollTick();
     }
 }
@@ -668,39 +770,39 @@ void WebView::setupSmoothScrolling(int posY)
 
 void WebView::scrollTick()
 {
-    if (_dy == 0)
+    if (m_dy == 0)
     {
         stopScrolling();
         return;
     }
 
-    if (_smoothScrollSteps < 1)
-        _smoothScrollSteps = 1;
+    if (m_smoothScrollSteps < 1)
+        m_smoothScrollSteps = 1;
 
-    int takesteps = _smoothScrollTime.restart() / 16;
+    int takesteps = m_smoothScrollTime.restart() / 16;
     int scroll_y = 0;
 
     if (takesteps < 1)
         takesteps = 1;
 
-    if (takesteps > _smoothScrollSteps)
-        takesteps = _smoothScrollSteps;
+    if (takesteps > m_smoothScrollSteps)
+        takesteps = m_smoothScrollSteps;
 
-    for(int i = 0; i < takesteps; i++)
+    for (int i = 0; i < takesteps; i++)
     {
-        int ddy = (_dy / (_smoothScrollSteps+1)) * 2;
+        int ddy = (m_dy / (m_smoothScrollSteps + 1)) * 2;
 
         // limit step to requested scrolling distance
-        if (abs(ddy) > abs(_dy))
-            ddy = _dy;
+        if (abs(ddy) > abs(m_dy))
+            ddy = m_dy;
 
         // update remaining scroll
-        _dy -= ddy;
+        m_dy -= ddy;
         scroll_y += ddy;
-        _smoothScrollSteps--;
+        m_smoothScrollSteps--;
     }
 
-    if (_scrollBottom)
+    if (m_scrollBottom)
         page()->currentFrame()->scroll(0, scroll_y);
     else
         page()->currentFrame()->scroll(0, -scroll_y);
@@ -709,7 +811,25 @@ void WebView::scrollTick()
 
 void WebView::stopScrolling()
 {
-    _smoothScrollTimer->stop();
-    _dy = 0;
-    _smoothScrolling = false;
+    m_smoothScrollTimer->stop();
+    m_dy = 0;
+    m_smoothScrolling = false;
+}
+
+
+void WebView::dragEnterEvent(QDragEnterEvent *event)
+{
+    if (event->mimeData()->hasUrls() || event->mimeData()->hasText())
+        event->acceptProposedAction();
+    else
+        KWebView::dragEnterEvent(event);
+}
+
+
+void WebView::dragMoveEvent(QDragMoveEvent *event)
+{
+    if (event->mimeData()->hasUrls() || event->mimeData()->hasText())
+        event->acceptProposedAction();
+    else
+        KWebView::dragMoveEvent(event);
 }
