@@ -2,8 +2,8 @@
 *
 * This file is a part of the rekonq project
 *
-* Copyright (C) 2008-2010 by Andrea Diamantini <adjam7 at gmail dot com>
-* Copyright (C) 2009-2010 by Lionel Chauvin <megabigbug@yahoo.fr>
+* Copyright (C) 2008-2011 by Andrea Diamantini <adjam7 at gmail dot com>
+* Copyright (C) 2009-2011 by Lionel Chauvin <megabigbug@yahoo.fr>
 *
 *
 * This program is free software; you can redistribute it and/or
@@ -52,16 +52,23 @@ public:
 
     WebPage *page();
 
-    inline QPoint mousePos() { return _mousePos; }
+    inline QPoint mousePos()
+    {
+        return m_mousePos;
+    }
 
 protected:
     void contextMenuEvent(QContextMenuEvent *event);
     void mousePressEvent(QMouseEvent *event);
     void mouseMoveEvent(QMouseEvent *event);
+    void enterEvent(QEvent *event);
     void keyPressEvent(QKeyEvent *event);
     void wheelEvent(QWheelEvent *event);
+    void dropEvent(QDropEvent *event);
+    void dragEnterEvent(QDragEnterEvent *event);
+    void dragMoveEvent(QDragMoveEvent * event);
 
-private slots:
+private Q_SLOTS:
     void search();
 
     void printFrame();
@@ -80,29 +87,32 @@ private slots:
     void setupSmoothScrolling(int posY);
 
     void stopScrolling();
+    void changeWindowIcon();
 
-signals:
+Q_SIGNALS:
     void loadUrl(const KUrl &, const Rekonq::OpenType &);
     void zoomChanged(int);
+    void openPreviousInHistory();
+    void openNextInHistory();
 
 private:
-    QPoint _mousePos;
-    QPoint _clickPos;
+    QPoint m_mousePos;
+    QPoint m_clickPos;
 
     // Auto Scroll
-    QTimer *_autoScrollTimer;
-    int _VScrollSpeed;
-    int _HScrollSpeed;
-    bool _canEnableAutoScroll;
-    bool _isAutoScrollEnabled;
+    QTimer *const m_autoScrollTimer;
+    int m_vScrollSpeed;
+    int m_hScrollSpeed;
+    bool m_canEnableAutoScroll;
+    bool m_isAutoScrollEnabled;
 
     // Smooth Scroll
-    QTimer *_smoothScrollTimer;
-    QTime _smoothScrollTime;
-    bool _scrollBottom;
-    bool _smoothScrolling;
-    int _dy;
-    int _smoothScrollSteps;
+    QTimer *const m_smoothScrollTimer;
+    QTime m_smoothScrollTime;
+    bool m_scrollBottom;
+    bool m_smoothScrolling;
+    int m_dy;
+    int m_smoothScrollSteps;
 };
 
 #endif
